#ifndef CYGONCE_IO_SDCARD_H
#define CYGONCE_IO_SDCARD_H

#include <pkgconf/system.h>
#include <pkgconf/devs_disk_generic_sdcard.h>
#include <cyg/infra/diag.h>

#define SDCARD_BLOCK_SIZE               (512)
#define SDCARD_MAX_TRANSFER_SIZE        (8*SDCARD_BLOCK_SIZE)

#define SDCARD_COMMAND_MASK             (    0x003F)
#define SDCARD_COMMAND_WITH_DATA        (  0x000080)
#define SDCARD_COMMAND_RESPONSE         (  0x000100)
#define SDCARD_COMMAND_LONG_RESPONSE    (  0x000200)

#define SDCARD_COMMAND_ASK_RESET        (  0x000400)
#define SDCARD_COMMAND_ASK_ABORT        (  0x000800)
#define SDCARD_COMMAND_ASK_MULTI        (  0x001000)
#define SDCARD_COMMAND_ASK_CRC          (  0x002000)
#define SDCARD_COMMAND_ASK_BUSY         (  0x004000)
#define SDCARD_COMMAND_ASK_OPCODE       (  0x008000)

extern cyg_uint8 sdcard_buffer[SDCARD_MAX_TRANSFER_SIZE];

typedef enum SDCARD_xError {
    SDCARD_ERROR_NONE = 0,
    SDCARD_ERROR_HARDWARE,
	SDCARD_ERROR_CRCFAIL,
    SDCARD_ERROR_TIMEOUT,
    SDCARD_ERROR_INDEX,
    SDCARD_ERROR_DATA_TIMEOUT,
    SDCARD_ERROR_DATA_HW_ERROR,
    SDCARD_ERROR_CMD_HW_ERROR,
    SDCARD_ERROR_DATA_FIFO_UNDERRUN, //or overrun
    SDCARD_ERROR_NOT_CONNECTED
} SDCARD_eError;

typedef enum SDCARD_xState {
    SDCARD_STATE_IDLE = 0,
    SDCARD_STATE_READY,
    SDCARD_STATE_IDENT,
    SDCARD_STATE_STBY,
    SDCARD_STATE_TRANS,
    SDCARD_STATE_DATA,
    SDCARD_STATE_RCV,
    SDCARD_STATE_PRG,
    SDCARD_STATE_DIS,
    SDCARD_STATE_RESERVED_9,
    SDCARD_STATE_RESERVED_10,
    SDCARD_STATE_RESERVED_11,
    SDCARD_STATE_RESERVED_12,
    SDCARD_STATE_RESERVED_13,
    SDCARD_STATE_RESERVED_14,
    SDCARD_STATE_RESERVED_IO,

    SDCARD_STATE_MASK = 0xF,
    SDCARD_STATE_UNKNOW = 0xFFFF
} SDCARD_eState;

typedef enum SDCARD_xDriverMode {
    SDCARD_DRIVERMODE_POLLING = 0,
    SDCARD_DRIVERMODE_INTERRUPT
} SDCARD_eDriverMode;

typedef enum SDCARD_xCardType{
    SDCARD_TYPE_NONE,
    SDCARD_TYPE_SDCARD,
    SDCARD_TYPE_SDHC,
    SDCARD_TYPE_MMC,
} SDCARD_eCardType;

typedef enum SDCARD_xDeviceType {
    SDCARD_DEVICE_SDCARD,
    SDCARD_DEVICE_INAND,
} SDCARD_eDeviceType;

typedef enum SDCARD_xVoltage {
    SDCARD_VOLTAGE_NONE,
    SDCARD_VOLTAGE_1V8,
    SDCARD_VOLTAGE_3V0,
    SDCARD_VOLTAGE_3V3,
} SDCARD_eVoltage;

typedef struct {
    cyg_bool            init;           // SDcard is in STANDBY state or not
    //Card info
    SDCARD_eCardType    sdcard_card_type;
    cyg_uint32          sdcard_serial_number;
    cyg_uint32          sdcard_block_count;
    cyg_bool            sdcard_connected;
    cyg_uint32          sdcard_freq;
    cyg_uint16          sdcard_rca;
    SDCARD_eVoltage     sdcard_voltage;

    SDCARD_eDeviceType  device_type;

    //Host controller
    SDCARD_eVoltage     ctrl_voltage;
    void                *priv;
    const SDCARD_eDriverMode driver_mode;
    cyg_uint8           *controller_base;
} cyg_sdcard_disk_info_t;

/*
 * Description
 * Clock speed dividers
 */
/*we must ensure a clock speed below 400kHz from a base clock up to 20MHz*/
#define SDCARD_CLK_SLOW           400000 // 400kHz
#define SDCARD_CLK_MMC_FULL     20000000 // 20MHz
#define SDCARD_CLK_MMC_HIGH     52000000 // 52MHz
#define SDCARD_CLK_SD_FULL      25000000 // 25MHz
#define SDCARD_CLK_SD_HIGH      50000000 // 50MHz

//Samsung 2412
#define SDI0_BASE_ADDRESS       ((cyg_uint8*)0x5A000000)

//P5/P5+
#define MCI0_BASE_ADDRESS 		((cyg_uint8*)0xD0060000)
//P5+ only
#define MCI1_BASE_ADDRESS 		((cyg_uint8*)0xD00C0000)

//P6/P6i
#define SDHCI0_BASE_ADDRESS 	((cyg_uint8*)0xC0700000)

//TI OMAP 3630
#define MMCHS1_BASE_ADDRESS     ((cyg_uint8*)0x4809C000)
#define MMCHS2_BASE_ADDRESS     ((cyg_uint8*)0x480B4000)
#define MMCHS3_BASE_ADDRESS     ((cyg_uint8*)0x480AD000)

#define PFX "sdcard: "

#if (CYGPKG_DEVS_DISK_GENERIC_SDCARD_DEBUG_LEVEL > 0)
#  define DEBUG1(_fmt, ...)     diag_printf(PFX _fmt, ## __VA_ARGS__)
#else
#  define DEBUG1(_fmt, ...)
#endif

#if (CYGPKG_DEVS_DISK_GENERIC_SDCARD_DEBUG_LEVEL > 1)
#  define DEBUG2(_fmt, ...)     diag_printf(PFX _fmt, ## __VA_ARGS__)
#else
#  define DEBUG2(_fmt, ...)
#endif

#if (CYGPKG_DEVS_DISK_GENERIC_SDCARD_DEBUG_LEVEL > 2)
#  define DEBUG3(_fmt, ...)     diag_printf(PFX _fmt, ## __VA_ARGS__)
#else
#  define DEBUG3(_fmt, ...)
#endif

//SDcard interface
SDCARD_eError sdcard_init(cyg_sdcard_disk_info_t *sdcard_info);
SDCARD_eError sdcard_lookup(cyg_sdcard_disk_info_t *sdcard_info);
SDCARD_eError sdcard_read(cyg_sdcard_disk_info_t *sdcard_info,
        cyg_uint8 *buffer, cyg_uint32 block, cyg_uint32 *len);
SDCARD_eError sdcard_write(cyg_sdcard_disk_info_t *sdcard_info,
        const cyg_uint8 *buffer, cyg_uint32 block, cyg_uint32 *len);
SDCARD_eError sdcard_state(cyg_sdcard_disk_info_t *sdcard_info,
        SDCARD_eState *state);

//Lower level interface must implement this
SDCARD_eError ll_sdcard_hw_init(cyg_sdcard_disk_info_t *sdcard_info);
SDCARD_eError ll_sdcard_card_init(cyg_sdcard_disk_info_t *sdcard_info);
SDCARD_eError ll_sdcard_send_command(cyg_sdcard_disk_info_t *sdcard_info,
        cyg_uint32 command, cyg_uint32 argument, cyg_uint32 *response);
SDCARD_eError ll_sdcard_send_data(cyg_sdcard_disk_info_t *sdcard_info,
        CYG_ADDRESS phys_addr, cyg_uint32 block_to_write);
SDCARD_eError ll_sdcard_receive_data(cyg_sdcard_disk_info_t *sdcard_info,
        CYG_ADDRESS phys_addr, cyg_uint32 block_to_read);

#endif /* CYGONCE_IO_SDCARD_H */
