//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  mci.h
* @brief SDcard low level driver functions
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     florent.bayendrian@parrot.fr
* @date       2007-01-08
*******************************************************************************
*/

#ifndef CYGONCE_MCI_H
#define CYGONCE_MCI_H

/*
 * Description:
 * The register offsets for the MMC controller registers are defined
 * below:
 */
#define MCI_REG_POWER           (0x00)
#define MCI_POWER_OFF           (0x00)
#define MCI_POWER_RESERVED      (0x01)
#define MCI_POWER_UP            (0x02)
#define MCI_POWER_ON            (0x03)
#define MCI_POWER_CTRL_MASK     (0x03)
#define MCI_POWER_VOLTAGE_SHIFT (2)
#define MCI_POWER_VOLTAGE_MASK  (0xF << MCI_POWER_VOLTAGE_SHIFT)
#define MCI_POWER_OPEN_DRAIN    (1<<6)
#define MCI_POWER_ROD           (1<<7)
#define MCI_REG_CLOCK           (0x04)
#define MCI_CLOCK_CLKDIV_MASK   (0xFF)
#define MCI_CLOCK_ENABLE        (1<<8)
#define MCI_CLOCK_PWRSAVE       (1<<9)
#define MCI_CLOCK_BYPASS        (1<<10)
#define MCI_CLOCK_WIDEBUS       (1<<11)
#define MCI_REG_ARGUMENT        (0x08)
#define MCI_REG_COMMAND         (0x0C)
#define MCI_COMMAND_RESPONSE        (1<<6)
#define MCI_COMMAND_LONG_RESPONSE   (1<<7)
#define MCI_COMMAND_INTERRUPT       (1<<8)
#define MCI_COMMAND_PENDING         (1<<9)
#define MCI_COMMAND_ENABLE          (1<<10)
#define MCI_COMMAND_RESET           (1<<11)
#define MCI_COMMAND_ABORT           (1<<12)
#define MCI_COMMAND_MULTI           (1<<13)
#define MCI_REG_RESPCMD         (0x10)
#define MCI_REG_RESPONSE0       (0x14)
#define MCI_REG_RESPONSE1       (0x18)
#define MCI_REG_RESPONSE2       (0x1C)
#define MCI_REG_RESPONSE3       (0x20)
#define MCI_REG_DATATIMER       (0x24)
#define MCI_REG_DATALENGTH      (0x28)
#define MCI_REG_DATACTRL        (0x2C)
#define MCI_DATACTRL_ENABLE         (1<<0)
#define MCI_DATACTRL_READ           (1<<1)
#define MCI_DATACTRL_MODE_STREAM    (1<<2)
#define MCI_DATACTRL_DMA_ENABLE     (1<<3)
#define MCI_DATACTRL_BLOCK_SIZE_SHIFT   (4)
#define MCI_DATACTRL_BLOCK_SIZE_512     (9 << MCI_DATACTRL_BLOCK_SIZE_SHIFT)
#define MCI_DATACTRL_BLOCK_SIZE_MASK    (0xF << MCI_DATACTRL_BLOCK_SIZE_SHIFT)
#define MCI_DATACTRL_CRC_DATA   (1<<8)
#define MCI_DATACTRL_CRC_OUT    (1<<9)
#define MCI_REG_DATACNT         (0x30)
#define MCI_REG_STATUS          (0x34)
#define MCI_STATUS_CMD_CRC_FAIL     (1<<0)
#define MCI_STATUS_DATA_CRC_FAIL    (1<<1)
#define MCI_STATUS_CMD_TIMEOUT      (1<<2)
#define MCI_STATUS_DATA_TIMEOUT     (1<<3)
#define MCI_STATUS_TX_UNDERRUN      (1<<4)
#define MCI_STATUS_RX_OVERRUN       (1<<5)
#define MCI_STATUS_CMD_RESP_END     (1<<6)
#define MCI_STATUS_CMD_SENT         (1<<7)
#define MCI_STATUS_DATA_END         (1<<8)
#define MCI_STATUS_START_BIT_ERR    (1<<9)
#define MCI_STATUS_DATA_BLOCK_END   (1<<10)
#define MCI_STATUS_CMD_ACTIVE       (1<<11)
#define MCI_STATUS_TX_ACTIVE        (1<<12)
#define MCI_STATUS_RX_ACTIVE        (1<<13)
#define MCI_STATUS_TX_FIFO_HALF     (1<<14)
#define MCI_STATUS_RX_FIFO_HALF     (1<<15)
#define MCI_STATUS_TX_FIFO_FULL     (1<<16)
#define MCI_STATUS_RX_FIFO_FULL     (1<<17)
#define MCI_STATUS_TX_FIFO_EMPTY    (1<<18)
#define MCI_STATUS_RX_FIFO_EMPTY    (1<<19)
#define MCI_STATUS_TX_DATA_AVAIL    (1<<20)
#define MCI_STATUS_RX_DATA_AVAIL    (1<<21)
#define MCI_STATUS_SDIO_INT         (1<<22)
#define MCI_REG_CLEAR           (0x38)
#define MCI_CLEAR_CMD_CRC_FAIL      (1<<0)
#define MCI_CLEAR_DATA_CRC_FAIL     (1<<1)
#define MCI_CLEAR_CMD_TIMEOUT       (1<<2)
#define MCI_CLEAR_DATA_TIMEOUT      (1<<3)
#define MCI_CLEAR_TX_UNDERRUN       (1<<4)
#define MCI_CLEAR_RX_OVERRUN        (1<<5)
#define MCI_CLEAR_CMD_RESP_END      (1<<6)
#define MCI_CLEAR_CMD_SENT          (1<<7)
#define MCI_CLEAR_DATA_END          (1<<8)
#define MCI_CLEAR_START_BIT_ERR     (1<<9)
#define MCI_CLEAR_DATA_BLOCK_END    (1<<10)
#define MCI_REG_MASK0           (0x3C)
#define MCI_REG_MASK1           (0x40)
#define MCI_REG_SDIO            (0x44)
#define MCI_SDIO_ENABLE         (1<<4)
#define MCI_REG_FIFOCNT         (0x48)
#define MCI_REG_FIFO            (0x80)

/* this mask is all the static (clearable) bits */
#define MCI_STATUS_STATIC_MASK      (0x0007FF)

/* Group of common status */
#define MCI_STATUS_ACTIVE   (MCI_STATUS_CMD_ACTIVE      | \
                             MCI_STATUS_TX_ACTIVE       | \
                             MCI_STATUS_RX_ACTIVE)

/* Command relative status */
#define MCI_INT_CMD_MASK    (MCI_STATUS_CMD_SENT        | \
                             MCI_STATUS_CMD_RESP_END    | \
                             MCI_STATUS_CMD_TIMEOUT     | \
                             MCI_STATUS_CMD_CRC_FAIL    | \
                             MCI_STATUS_START_BIT_ERR)
/* Data relative status */
#define MCI_INT_DATA_MASK   (MCI_STATUS_DATA_BLOCK_END  | \
                             MCI_STATUS_DATA_TIMEOUT    | \
                             MCI_STATUS_DATA_CRC_FAIL)

#define MCI_INT_ALL_MASK    (MCI_INT_CMD_MASK           | \
                             MCI_INT_DATA_MASK)

/* DMA definition */
#define MCI_DATA_WRITE          (0)
#define MCI_FIFO_UWORDS         (16)
#define MCI_FIFO_BYTES          (MCI_FIFO_UWORDS * 4)

#define MCI_DMA_CHANNEL         (0)

#define P5P_DMA_BASE_ADDRESS            (0xC0050000)

#define P5P_DMA_REG_INTSTATUS           (0x00)
#define P5P_DMA_REG_INTTCSTATUS         (0x04)
#define P5P_DMA_REG_INTTCCLEAR          (0x08)
#define P5P_DMA_REG_INTERRORSTATUS      (0x0C)
#define P5P_DMA_REG_INTERRCLR           (0x10)
#define P5P_DMA_REG_RAWINTTCSTATUS      (0x14)
#define P5P_DMA_REG_RAWINTERRORSTATUS   (0x18)
#define P5P_DMA_REG_ENBLDCHNS           (0x1C)
#define P5P_DMA_REG_SOFTBREQ            (0x20)
#define P5P_DMA_REG_SOFTSREQ            (0x24)
#define P5P_DMA_REG_SOFTLBREQ           (0x28)
#define P5P_DMA_REG_SOFTLSREQ           (0x2C)
#define P5P_DMA_REG_CONFIG              (0x30)
#define P5P_DMA_REG_SYNC                (0x34)

#define P5P_DMA_REG_CHX_SRCADDR         (0x100 + (MCI_DMA_CHANNEL * 0x20))
#define P5P_DMA_REG_CHX_DESTADDR        (0x104 + (MCI_DMA_CHANNEL * 0x20))
#define P5P_DMA_REG_CHX_LLI             (0x108 + (MCI_DMA_CHANNEL * 0x20))
#define P5P_DMA_REG_CHX_CTRL            (0x10C + (MCI_DMA_CHANNEL * 0x20))
#define P5P_DMA_REG_CHX_CONFIG          (0x110 + (MCI_DMA_CHANNEL * 0x20))


#define P5P_DMA_CH_CONFIG_ENABLE                    (1)

#define P5P_DMA_CH_CONFIG_SRCPERIPH_SHIFT           (1)
#define P5P_DMA_CH_CONFIG_DSTPERIPH_SHIFT           (6)
#define P5P_DMA_CH_CONFIG_FLOWCNTRL_SHIFT           (11)
#define P5P_DMA_CH_CONFIG_FLOWCNTRL_MEM2PERIPH      (5 << \
        P5P_DMA_CH_CONFIG_FLOWCNTRL_SHIFT)
#define P5P_DMA_CH_CONFIG_FLOWCNTRL_PERIPH2MEM      (6 << \
        P5P_DMA_CH_CONFIG_FLOWCNTRL_SHIFT)

#define P5P_DMA_CH_CTRL_SRC_WIDTH_WORD              (2 << 18)
#define P5P_DMA_CH_CTRL_DST_WIDTH_WORD              (2 << 21)
#define P5P_DMA_CH_CTRL_SRC_BURST_SIZE_8            (2 << 12)
#define P5P_DMA_CH_CTRL_DST_BURST_SIZE_8            (2 << 15)
#define P5P_DMA_CH_CTRL_IT                          (1<<31)
#define P5P_DMA_CH_CTRL_SI                          (1<<26)
#define P5P_DMA_CH_CTRL_DI                          (1<<27)


#endif /* CYGONCE_MCI_H */
