//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file     sdhci.h
* @brief    SDcard low level driver functions
*
* Copyright (C) 2007 Parrot S.A.
*
* @author    florent.bayendrian@parrot.com
* @date    2007-12-19
*******************************************************************************
*/

#ifndef CYGONCE_SDHCI_H
#define CYGONCE_SDHCI_H

/*
 * Description:
 * The register offsets for the SDHCI controller registers are defined
 * below:
 */
#define SDHCI_REG_DMA_ADDRESS               (0x00)
#define SDHCI_REG_BLOCK_SIZE                (0x04)
#define SDHCI_MAX_DMA_BUFFER_SHIFT          (12)
#define SDHCI_MAX_DMA_BUFFER_SIZE           (0x7<<SDHCI_MAX_DMA_BUFFER_SHIFT)
#define SDHCI_REG_BLOCK_COUNT               (0x06)
#define SDHCI_REG_ARGUMENT                  (0x08)
#define SDHCI_REG_TRANS_MODE                (0x0C)
#define SDHCI_TRANS_DMA_EN                  (1<<0)
#define SDHCI_TRANS_BLOCK_COUNT_EN          (1<<1)
#define SDHCI_TRANS_READ                    (1<<4)
#define SDHCI_TRANS_MULTIBLOCK              (1<<5)
#define SDHCI_REG_COMMAND                   (0x0E)
#define SDHCI_COMMAND_RSP_NONE              (0)
#define SDHCI_COMMAND_RSP136                (1)
#define SDHCI_COMMAND_RSP48                 (2)
#define SDHCI_COMMAND_RSP48_BUSY            (3)
#define SDHCI_COMMAND_CHECK_CRC             (1<<3)
#define SDHCI_COMMAND_CHECK_INDEX           (1<<4)
#define SDHCI_COMMAND_DATA                  (1<<5)
#define SDHCI_COMMAND_ABORT                 (3<<6)
#define SDHCI_COMMAND_INDEX_SHIFT           (8)
#define SDHCI_REG_RESPONSE0                 (0x10)
#define SDHCI_REG_RESPONSE1                 (0x12)
#define SDHCI_REG_RESPONSE2                 (0x14)
#define SDHCI_REG_RESPONSE3                 (0x16)
#define SDHCI_REG_RESPONSE4                 (0x18)
#define SDHCI_REG_RESPONSE5                 (0x1A)
#define SDHCI_REG_RESPONSE6                 (0x1C)
#define SDHCI_REG_RESPONSE7                 (0x1E)
#define SDHCI_REG_PRESENT_STATE             (0x24)
#define SDHCI_PRESENT_STATE_CMDINHIB        (1<<0)
#define SDHCI_PRESENT_STATE_DATAINHIB       (1<<1)
#define SDHCI_REG_HOSTCTRL                  (0x28)
#define SDHCI_HOSTCTRL_LEDON                (1<<0)
#define SDHCI_HOSTCTRL_4BITBUS              (1<<1)
#define SDHCI_HOSTCTRL_HIGHSPEED            (1<<2)
#define SDHCI_HOSTCTRL_8BITBUS              (1<<5)
#define SDHCI_REG_POWERCTRL                 (0x29)
#define SDHCI_POWER_ON                      (1<<0)
#define SDHCI_POWER_180                     (0xA)
#define SDHCI_POWER_300                     (0xC)
#define SDHCI_POWER_330                     (0xE)
#define SDHCI_REG_CLOCKCTRL                 (0x2C)
#define SDHCI_CLOCKCTRL_INT_CLOCK_ENABLE    (1<<0)
#define SDHCI_CLOCKCTRL_INT_CLOCK_STABLE    (1<<1)
#define SDHCI_CLOCKCTRL_CLOCK_ENABLE        (1<<2)
#define SDHCI_DIVIDER_SHIFT                 (8)
#define SDHCI_REG_TIMEOUT_CTRL              (0x2E)
#define SDHCI_MAX_TIMEOUT                   (0xE)
#define SDHCI_REG_SOFT_RESET                (0x2F)
#define SDHCI_SOFT_RESET_ALL                (1<<0)
#define SDHCI_SOFT_RESET_CMD                (1<<1)
#define SDHCI_SOFT_RESET_DATA               (1<<2)
#define SDHCI_REG_INT_STATUS                (0x30)
#define SDHCI_REG_INT_ENABLE                (0x34)
#define SDHCI_REG_SIGNAL_ENABLE             (0x38)
#define SDHCI_INT_CMD_COMPLETE              (1<<0)
#define SDHCI_INT_DATA_END                  (1<<1)
#define SDHCI_INT_DATA_BLOCKGAP             (1<<2)
#define SDHCI_INT_DMA_END                   (1<<3)
#define SDHCI_INT_SDIO                      (1<<8)
#define SDHCI_INT_ERR_CMD_TIMEOUT           (1<<16)
#define SDHCI_INT_ERR_CMD_CRC               (1<<17)
#define SDHCI_INT_ERR_CMD_ENDBIT            (1<<18)
#define SDHCI_INT_ERR_CMD_INDEX             (1<<19)
#define SDHCI_INT_ERR_DATA_TIMEOUT          (1<<20)
#define SDHCI_INT_ERR_DATA_CRC              (1<<21)
#define SDHCI_INT_ERR_DATA_ENDBIT           (1<<22)
#define SDHCI_INT_ERR_BUS_POWER             (1<<23)
#define SDHCI_INT_ERR_ACMD12                (1<<24)
#define SDHCI_INT_ERR_DMA                   (1<<25)
#define SDHCI_REG_CAPABILITIES              (0x40)
#define SDHCI_CAPS_TIMEOUT_CLK_UNIT         (1<<7)
#define SDHCI_CAPS_SUPPORT_HIGHSPEED        (1<<21)
#define SDHCI_CAPS_SUPPORT_DMA              (1<<22)
#define SDHCI_CAPS_SUPPORT_3V3              (1<<24)
#define SDHCI_CAPS_SUPPORT_3V0              (1<<25)
#define SDHCI_CAPS_SUPPORT_1V8              (1<<26)
#define SDHCI_CAPS_SUPPORT_INT              (1<<27)
#define SDHCI_CAPS_TIMEOUT_CLK_SHIFT        (0)
#define SDHCI_CAPS_TIMEOUT_CLK_MASK         (0x3F << SDHCI_CAPS_TIMEOUT_CLK_SHIFT)
#define SDHCI_CAPS_CLOCK_BASE_SHIFT         (8)
#define SDHCI_CAPS_CLOCK_BASE_MASK          (0x00003F << SDHCI_CAPS_CLOCK_BASE_SHIFT)
#define SDHCI_CAPS_BLOCKSIZE_SHIFT          (16)
#define SDHCI_CAPS_BLOCKSIZE_MASK           (0x3 << SDHCI_CAPS_BLOCKSIZE_SHIFT)
#define SDHCI_REG_HOSTC_VERSION             (0xFE)
#define SDHCI_HOSTC_VERSION_VENDOR_SHIFT    (8)
#define SDHCI_HOSTC_VERSION_VENDOR_MASK     (0xFF << SDHCI_HOSTC_VERSION_VENDOR_SHIFT)
#define SDHCI_HOSTC_VERSION_SPEC_SHIFT      (0)
#define SDHCI_HOSTC_VERSION_SPEC_MASK       (0xFF << SDHCI_HOSTC_VERSION_SPEC_SHIFT)

#define SDHCI_INT_CMD_MASK  (SDHCI_INT_CMD_COMPLETE     | \
                             SDHCI_INT_ERR_CMD_TIMEOUT  | \
                             SDHCI_INT_ERR_CMD_CRC      | \
                             SDHCI_INT_ERR_CMD_ENDBIT   | \
                             SDHCI_INT_ERR_CMD_INDEX)

#define SDHCI_INT_DATA_MASK (SDHCI_INT_DATA_END         | \
                             SDHCI_INT_ERR_DATA_TIMEOUT | \
                             SDHCI_INT_ERR_DATA_CRC     | \
                             SDHCI_INT_ERR_DATA_ENDBIT)

#define SDHCI_INT_MISC_MASK (SDHCI_INT_ERR_BUS_POWER    | \
                             SDHCI_INT_ERR_DMA)

#define SDHCI_INT_ALL_MASK  (SDHCI_INT_CMD_MASK         | \
                             SDHCI_INT_DATA_MASK        | \
                             SDHCI_INT_MISC_MASK)

#endif /* CYGONCE_SDHCI_H */
