/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_i2c.h
 * @brief  Interface with IC driver.
 *
 * Code is based on the following datasheets:
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4245_F1.pdf
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4265_F1.pdf
 *  
 * @author David Tylski
 * @date   Jan. 2007
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */
#ifndef _CS42XX_I2C_H_
#define _CS42XX_I2C_H_

#include "cs42xx.h"


// Attach a bus to a symbolic device.
extern Cyg_ErrNo
CS42XX_I2C_AttachBus(CS42XX_DEVID,
		const cyg_i2c_bus *bus, cyg_bool ad0, cyg_bool ad1);

// Write one or successive bytes in device memory, at given location.
extern Cyg_ErrNo
CS42XX_I2C_Write(CS42XX_DEVID,
		CYG_BYTE addr, const CYG_BYTE *buf, cyg_count8 len);

// Read one or successive bytes from device memory, at given location.
extern Cyg_ErrNo
CS42XX_I2C_Read(CS42XX_DEVID,
		CYG_BYTE addr, CYG_BYTE *buf, cyg_count8 len);


#endif // _CS42XX_I2C_H_

