//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_test.c
 * @brief  Tests applicable on CS42XX driver.
 *
 * @author David Tylski
 * @date   Mar. 2007
 *
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */

#include <cyg/devs/cs42xx_api.h>

#include <cyg/devs/test/cs42xx_test.h>
#include <cyg/devs/test/cs42xx_testhmi.h>
#include <cyg/devs/test/cs42xx_testctrl.h>
#include <cyg/devs/test/cs42xx_testi2c.h>
#include <cyg/devs/test/cs42xx_testaudio.h>


#ifdef CS42XX_TEST_SUPPORT



/*
 * Local variables.
 *******************/

/** Device under control. */
static CS42XX_DEVID CS42XXTestCtrlDeviceId;
static cyg_mutex_t  CS42XXTestCtrlDeviceIdMutex; ///< (Self-explanatory.)



/*
 * Definition of functions.
 ***************************/


/** Get the device Id selected.
 ****************************************************************************/
 CS42XX_DEVID
CS42XX_Test_GetDeviceId(void)
{
	// Since this device Id is atomic, no mutex is required to read it.
	return CS42XXTestCtrlDeviceId;
}


/** Switch the device to be controlled.
 *
 * This function has a meaning only if multiple chips are handled on a same
 * platform.
 ****************************************************************************/
static Cyg_test_code
CS42XX_Test_SwitchDeviceId(void)
{
	cyg_mutex_lock(&CS42XXTestCtrlDeviceIdMutex);
	if (CS42XXTestCtrlDeviceId < CS42XX_NBDEVICES)
	{
		CS42XXTestCtrlDeviceId++;
	}
	else
	{
		CS42XXTestCtrlDeviceId = 0;
	}
	CS42XX_TEST_PRINTF("Selected dev.%d", CS42XXTestCtrlDeviceId);
	cyg_mutex_unlock(&CS42XXTestCtrlDeviceIdMutex);

	return CYGNUM_TEST_PASS;
}

static Cyg_test_code
CS42XX_Test_SetLoopBack(void)
{
	Cyg_ErrNo err;
	cyg_bool state = !CS42XX_GetLoopMode(CS42XXTestCtrlDeviceId, &err);

	CS42XX_TEST_PRINTF("Loop = %d", state);
	CS42XX_ConfigLoopMode(CS42XXTestCtrlDeviceId, state, CS42XX_CACHE_SYNC);

	return CYGNUM_TEST_PASS;
}


/** Configure HAL for testing purposes.
 *
 * @retval  ENOERR  Successfully initialized
 * @retval -EINVAL  Invalid configuration, not set or incomplete
 ****************************************************************************/
static Cyg_ErrNo
CS42XX_Test_InitHAL(void)
{
	Cyg_ErrNo     err;
	CS42XX_HALCFG halcfg;

	err = CS42XX_HAL_GetConfig(&halcfg);
	if (ENOERR == err)
	{
		halcfg.reset.pin       = CS42XX_TEST_RESETPIN;
		halcfg.intr.pin        = CS42XX_TEST_INTRPIN;
		halcfg.intr.vector     = CS42XX_TEST_INTRVECTOR;
		halcfg.intr.activehigh = CS42XX_TEST_INTRACTIVEHIGH;
		halcfg.intr.trigmode   = CS42XX_TEST_TRIGMODE;
		err = CS42XX_HAL_SetConfig(&halcfg);
	}

	return err;
}


/** Set classic device settings.
 ****************************************************************************/
static Cyg_ErrNo
CS42XX_Test_InitDevice(CS42XX_DEVID devid)
{
	Cyg_ErrNo       err = ENOERR;
	CS42XX_CONFIG   cs42xxcfg;
	CS42XX_SETTINGS audiosettings;

	// Configure CS42XX driver
	err = CS42XX_Config_Get(0, &cs42xxcfg);
	if (ENOERR != err)
	{
		CS42XX_TEST_PRINTF("\r\nCS42XX(%d): no config! (errno=%d)\r\n",
				devid, err);
	}
	else
	{
		cs42xxcfg.async = true;

		cs42xxcfg.in.format  = CS42XX_DIF_I2S;
		cs42xxcfg.in.master  = CS42XX_DIFMS_SLAVE;
		cs42xxcfg.in.mclkdiv = CS42XX_MCLK_DIV_1;
		cs42xxcfg.in.srate   = CS42XX_SAMPLERATE_SINGLE;

		cs42xxcfg.out.format  = CS42XX_DIF_I2S;
		cs42xxcfg.out.master  = CS42XX_DIFMS_SLAVE;
		cs42xxcfg.out.mclkdiv = CS42XX_MCLK_DIV_1;
		cs42xxcfg.out.srate   = CS42XX_SAMPLERATE_SINGLE;

		cs42xxcfg.inputsel  = CS42XX_AINSEL_MIC;
		cs42xxcfg.auxoutsel = CS42XX_AOUTSEL_PGA;
		cs42xxcfg.loopmode  = false;

		err = CS42XX_Config_Set(0, &cs42xxcfg);
		if (ENOERR != err)
		{
			CS42XX_TEST_PRINTF("\r\nCS42XX(%d): config not set! (errno=%d)\r\n",
					devid, err);
		}
	}

	// Power it up.
	CS42XX_PowerUp(devid);
	CS42XX_TEST_PRINTF("\r\nCS42XX(%d): powered on.", devid);

	// Get current settings
	err = CS42XX_Settings_Get(devid, &audiosettings);
	if (ENOERR != err)
	{
		CS42XX_TEST_PRINTF("\r\nCS42XX: no settings! (errno=%d)\r\n",
				devid, err);
	}
	else
	{
		// Set gain to 0dB, volume to max level
		audiosettings.adc.a.gain = CS42XX_PGAGAIN_0DB;
		audiosettings.adc.b.gain = CS42XX_PGAGAIN_0DB;
		audiosettings.dac.a.vol  = CS42XX_VOLUME_MAX; // 0dB
		audiosettings.dac.b.vol  = CS42XX_VOLUME_MAX; // 0dB

		// Unmute input and output
		audiosettings.adc.mute = false;
		audiosettings.dac.mute = false;

		// Apply new settings
		err = CS42XX_Settings_Set(devid, &audiosettings);
		if (ENOERR != err)
		{
			CS42XX_TEST_PRINTF("\r\nCS42XX(%d): settings not set! (errno=%d)\r\n",
					devid, err);
		}
	}

	return err;
}


/** Declare all CS42XX chips to be used by driver.
 ****************************************************************************/
static Cyg_ErrNo
CS42XX_Test_InitAllDevices(void)
{
	static cyg_uint8 buf[32];  // (Keep it static to save stack!)
	Cyg_ErrNo    err = ENOERR;
	CS42XX_DEVID devid;

	for (devid = 0; ((ENOERR == err) && (devid < CS42XX_NBDEVICES)); devid++)
	{
		err = CS42XX_Init_AttachBus(devid,
					&parrot5_i2c_bus,
					/*AD0*/((devid >> 0) & 0x01),
					/*AD1*/((devid >> 1) & 0x01));
		if (ENOERR != err)
		{
			CS42XX_TEST_FAIL_ERRNO("AttachBus", err);
		}
		else
		{
			err = CS42XX_Supp_GetChipName(devid, buf, sizeof(buf));
			if (ENOERR == err)
			{
				CS42XX_TEST_PRINTF("\r\nDetected: %s", buf);
				err = CS42XX_Test_InitDevice(devid);
			}
			else
			{
				CS42XX_TEST_PRINTF("\r\nDetected unknown device!");
			}
		}
	}

	return err;
}


/** Initialize CS42XX driver for test session.
 *
 * @retval true   Success
 * @retval false  Error during initialization
 ****************************************************************************/
static cyg_bool
CS42XX_Test_InitDriver(void)
{
	Cyg_ErrNo err;

	// Initialize HAL of driver
	err = CS42XX_Test_InitHAL();
	if (ENOERR != err)
	{
		CS42XX_TEST_FAIL_ERRNO("InitHAL", err);
	}
	else
	{
		// Initialize driver - This will reset all connected devices
		CS42XX_TEST_PRINTF("\r\nResetting CS42XX devices...");
		err = CS42XX_Init();
		if (ENOERR != err)
		{
			CS42XX_TEST_FAIL_ERRNO("Init", err);
		}
		else
		{
			// Initialize all connected devices
			err = CS42XX_Test_InitAllDevices();
		}
	}

	return (ENOERR == err);
}


/** Initialize CS42XX driver testing module.
 *
 * @retval true   Success
 * @retval false  Error during initialization
 ****************************************************************************/
static cyg_bool
CS42XX_Test_InitTest(void)
{
	cyg_bool rv = true;

	rv = CS42XX_TestCtrl_Init();

#ifdef CS42XX_TEST_SUPPORT_AUDIO
	if (rv)
	{
		rv = CS42XX_TestAudio_Init();
	}
	// else previous failure.
#endif // CS42XX_TEST_SUPPORT_AUDIO

#ifdef CS42XX_TEST_SUPPORT_I2C
	if (rv)
	{
		rv = CS42XX_TestI2C_Init();
	}
	// else previous failure.
#endif // CS42XX_TEST_SUPPORT_I2C

	return rv;
}


/** Initialize all stuff for the testing session.
 *
 * @retval true   Success
 * @retval false  Error during initialization
 ****************************************************************************/
static cyg_bool
CS42XX_Test_Init(void)
{
	cyg_bool rv;

	// Initialize I�S pins (with Master Clock at last)
	SELECT_PIN(AC_INS);
	SELECT_PIN(AC_OUTS);
	SELECT_PIN(AC_SYNC);
	SELECT_PIN(AC_CLK);
	SELECT_PIN(MCLK);

	// Initialize test-specific global variables
	CS42XXTestCtrlDeviceId = (CS42XX_DEVID)0;

	rv = CS42XX_Test_InitDriver();
	if (!rv)
	{
		CS42XX_TEST_FAIL("Driver init");
	}
	else
	{
		rv = CS42XX_Test_InitTest();
		if (!rv)
		{
			CS42XX_TEST_FAIL("Test module init");
		}
		// else success.
	}

	return rv;
}


/** Thread to manage tests (to be thread-created by application).
 *
 * @param data   (Not used)
 ****************************************************************************/
 void CS42XX_Test_ManagerThread(cyg_addrword_t data)
{
	CS42XX_TEST_ID	testid;
	cyg_bool			bshowmenu = true;

	// Initialize all stuff for the testing session
	CS42XX_Test_Init();

	while(1)
	{
		CS42XX_TEST_PRINTF("\r\n***");

		// Select the test to be started
		testid = CS42XX_TestHMI_Menu(bshowmenu);

		// Start the appropriate test session
		switch(testid)
		{
		case CS42XX_TEST_NONE:
			bshowmenu = !bshowmenu;
			// Nothing to be done
			break;

		case CS42XX_TEST_SWITCHDEVICE:
			// Switch between CS42XX devices to be tested
			CS42XX_Test_SwitchDeviceId();
		case CS42XX_TEST_SETLOOPBACK:
			CS42XX_Test_SetLoopBack();
			break;

		case CS42XX_TEST_I2C_NONE:
		case CS42XX_TEST_I2C_WRITE:
		case CS42XX_TEST_I2C_READ:
		case CS42XX_TEST_I2C_READWRITE:
			CS42XX_TestI2C_Run(testid);
			break;

		case CS42XX_TEST_AUDIO_NONE:
		case CS42XX_TEST_AUDIO_PLAYSILENCE:
		case CS42XX_TEST_AUDIO_PLAYSQ_HF1:
		case CS42XX_TEST_AUDIO_PLAYSQ_HF1OPP:
		case CS42XX_TEST_AUDIO_PLAYSQ_LF1:
		case CS42XX_TEST_AUDIO_PLAYSINUS:
		case CS42XX_TEST_AUDIO_ACQUIRE:
		case CS42XX_TEST_AUDIO_ACQUIRE_RESTIT:
		case CS42XX_TEST_AUDIO_LOOPINOUT:
		case CS42XX_TEST_AUDIO_SENDRAWFILE:
			CS42XX_TestAudio_Run(testid);
			break;

		case CS42XX_TEST_CTRL_ADC_SWITCH:
		case CS42XX_TEST_CTRL_ADC_MUTE:
		case CS42XX_TEST_CTRL_ADC_GAINUP:
		case CS42XX_TEST_CTRL_ADC_GAINDOWN:
		case CS42XX_TEST_CTRL_DAC_SWITCHAUX:
		case CS42XX_TEST_CTRL_DAC_MUTE:
		case CS42XX_TEST_CTRL_DAC_VOLUMEUP:
		case CS42XX_TEST_CTRL_DAC_VOLUMEDOWN:
			CS42XX_TestCtrl_Run(testid);
			break;

		case CS42XX_TEST_MENU_SHOW_HIDE:
			bshowmenu = !bshowmenu;
			// Nothing to be done
			break;

		default:
			break;
		}

		// ... and return to menu (loop).
	}
}



#endif // CS42XX_TEST_SUPPORT
