//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_test.h
 * @brief  Tests applicable on CS42XX driver.
 *
 * @author David Tylski
 * @date   Jan. 2007
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */
#ifndef _CS42XX_TEST_H_
#define _CS42XX_TEST_H_

#include <stdio.h>
#include <errno.h>
#include <cyg/io/io.h>
#include <cyg/infra/diag.h>
#include <cyg/infra/testcase.h>       // Cyg_test_code
#include <pkgconf/devices_cs42xx.h>

#include <cyg/devs/cs42xx_api.h>      // CS42XX_DEVID


// Build options
#ifdef CYGPKG_DEVICES_CS42XX_TEST
#  define CS42XX_TEST_SUPPORT
#  ifdef CYGPKG_DEVICES_CS42XX_TEST_I2C
#    define CS42XX_TEST_SUPPORT_I2C
#  endif
#  ifdef CYGPKG_DEVICES_CS42XX_TEST_AUDIO
#    define CS42XX_TEST_SUPPORT_AUDIO
#  endif
#endif


/** Failure message. */
#define CS42XX_TEST_FAIL(_msg_)										\
		CYG_MACRO_START												\
		diag_printf("\r\n*** CS42XXTest failed: " _msg_);			\
		CYG_MACRO_END

/** Failure message with err. */
#define CS42XX_TEST_FAIL_ERRNO(_msg_, _errno_)						\
		CYG_MACRO_START												\
		diag_printf("\r\n*** CS42XXTest failed: " _msg_);			\
		CS42XX_TEST_PERROR(_errno_);								\
		CYG_MACRO_END

/** Personalized perror() so that any err can be displayed. */
#define CS42XX_TEST_PERROR(_errno_)									\
		CYG_MACRO_START												\
		diag_printf(" %s (%d)", strerror(_errno_), _errno_);		\
		CYG_MACRO_END

/** If pointer is not null, copy the given value in pointed area. */
#define CS42XX_TEST_STORE(_ptr_, _val_)								\
		CYG_MACRO_START												\
		if (NULL != (_ptr_))										\
		{															\
			*(_ptr_) = (_val_);										\
		}															\
		/* else pointer is NULL, so do no copy. */					\
		CYG_MACRO_END

/** Show data to the user. */
#define CS42XX_TEST_PRINTF(str...) \
		diag_printf("\r\nCS42XX_Test: " str)



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *             DATA SPECIFIC TO THE TEST-PLATFORM HARDWARE                 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 * Inputs / outputs.
 *
 * For CK42xx: (GPIO01, GPIO18, CYGNUM_HAL_INTERRUPT_GPIO0, false,
 * CS42XX_TRIGMODE_FALLING)
 */

#define CS42XX_TEST_RESETPIN		CYGPKG_DEVICES_CS42XX_TEST_HWCFG_RESET
#define CS42XX_TEST_INTRPIN			CYGPKG_DEVICES_CS42XX_TEST_HWCFG_INTR
#define CS42XX_TEST_INTRVECTOR		CYGPKG_DEVICES_CS42XX_TEST_HWCFG_INTRVEC
#define CS42XX_TEST_INTRACTIVEHIGH	false
#define CS42XX_TEST_TRIGMODE		CS42XX_TRIGMODE_RISING



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                           MANAGEMENT OF TESTS                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#undef CS42XX_TEST_ID_DECL
#ifdef CS42XX_TEST_IDLIST
#define CS42XX_TEST_ID_DECL(id,descr)	[CS42XX_TEST_##id] = descr
#else
#define CS42XX_TEST_ID_DECL(id,descr)	CS42XX_TEST_##id
#endif

/** Identifier for test.
 * @typedef CS42XX_TEST_ID
 */
#ifdef CS42XX_TEST_IDLIST
static const char* CS42XX_TEST_IDLIST[] =
#else
typedef enum CS42XX_TEST_ID_TAG CS42XX_TEST_ID;
enum CS42XX_TEST_ID_TAG
#endif
{
	CS42XX_TEST_ID_DECL(NONE, ""),

	CS42XX_TEST_ID_DECL(SWITCHDEVICE, "Switch device"),
	CS42XX_TEST_ID_DECL(SETLOOPBACK, "Switch Loopback On/Off"),

	CS42XX_TEST_ID_DECL(I2C_NONE,            "I2C: Idle"),
	CS42XX_TEST_ID_DECL(I2C_WRITE,           "I2C: Write"),
	CS42XX_TEST_ID_DECL(I2C_READ,            "I2C: Read"),
	CS42XX_TEST_ID_DECL(I2C_READWRITE,       "I2C: Read/Write"),

	CS42XX_TEST_ID_DECL(AUDIO_NONE,          "Audio: Idle"),
	CS42XX_TEST_ID_DECL(AUDIO_PLAYSILENCE,   "Audio: Silence"),
	CS42XX_TEST_ID_DECL(AUDIO_PLAYSQ_HF1,    "Audio: HighFreq1"),
	CS42XX_TEST_ID_DECL(AUDIO_PLAYSQ_HF1OPP, "Audio: HighFreq1 Opp"),
	CS42XX_TEST_ID_DECL(AUDIO_PLAYSQ_LF1,    "Audio: LowFreq1"),
	CS42XX_TEST_ID_DECL(AUDIO_PLAYSINUS,     "Audio: Sinus"),
	CS42XX_TEST_ID_DECL(AUDIO_ACQUIRE,       "Audio: Acquire"),
	CS42XX_TEST_ID_DECL(AUDIO_ACQUIRE_RESTIT,"Audio: Acquire & restitute"),
	CS42XX_TEST_ID_DECL(AUDIO_LOOPINOUT,     "Audio: Loop In->Out"),
	CS42XX_TEST_ID_DECL(AUDIO_SENDRAWFILE,   "Audio: Send RAW file in diag"),

	CS42XX_TEST_ID_DECL(CTRL_NONE,           "Ctrl: Idle"),
	CS42XX_TEST_ID_DECL(CTRL_ADC_SWITCH,     "Ctrl: ADC: Switch input"),
	CS42XX_TEST_ID_DECL(CTRL_ADC_MUTE,       "Ctrl: ADC: (Un)Mute"),
	CS42XX_TEST_ID_DECL(CTRL_ADC_GAINUP,     "Ctrl: ADC: Gain +"),
	CS42XX_TEST_ID_DECL(CTRL_ADC_GAINDOWN,   "Ctrl: ADC: Gain -"),

	CS42XX_TEST_ID_DECL(CTRL_DAC_SWITCHAUX,  "Ctrl: DAC: Switch aux out"),
	CS42XX_TEST_ID_DECL(CTRL_DAC_MUTE,       "Ctrl: DAC: (Un)Mute"),
	CS42XX_TEST_ID_DECL(CTRL_DAC_VOLUMEUP,   "Ctrl: DAC: Vol +"),
	CS42XX_TEST_ID_DECL(CTRL_DAC_VOLUMEDOWN, "Ctrl: DAC: Vol -"),

	CS42XX_TEST_ID_DECL(MENU_SHOW_HIDE,		 "Show/Hide this menu"),

	CS42XX_TEST_ID_DECL(NB, ""),

	CS42XX_TEST_ID_DECL(ALL, ""),
};



/*
 * Exported functions.
 **********************/


// Get the device Id selected.
extern CS42XX_DEVID
CS42XX_Test_GetDeviceId(void);

// Thread to manage tests (to be thread-created by application).
extern void
CS42XX_Test_ManagerThread(CYG_ADDRWORD data);



#endif // _CS42XX_TEST_H_
