//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_testhmi.c
 * @brief  Tests applicable on CS42XX driver - HMI part.
 *
 * This module displays an HMI on the generic debug interface, so that test
 * functions can be easily available.
 *
 * @author David Tylski
 * @date   Mar. 2007
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */


#include <cyg/hal/hal_diag.h>
#include <cyg/infra/diag.h>

#include <cyg/devs/test/cs42xx_test.h>


#ifdef CS42XX_TEST_SUPPORT


/*
 * Trick: this define lets cs42xx_test.h generate the constant table
 * CS42XXTestHMIDescr[] containing a description of the test Ids.
 * Thus, cs42xx_test.h shall be included _before_ and _after_ this define.
 */
#define CS42XX_TEST_IDLIST		CS42XXTestHMIDescr
#undef _CS42XX_TEST_H_
#include <cyg/devs/test/cs42xx_test.h>
#undef CS42XX_TEST_IDLIST

#include <cyg/devs/test/cs42xx_testhmi.h>


/** Display the main interface of test module.
 *
 * @return  Identifier of the test to be started
 ****************************************************************************/
 CS42XX_TEST_ID
CS42XX_TestHMI_Menu(cyg_bool bshow)
{
	CS42XX_TEST_ID testid = CS42XX_TEST_NONE;

#ifdef CS42XX_TESTHMI_PREDEF
	static cyg_bool testchosen = false;

	if (false == testchosen)
	{
		testid = CS42XX_TESTHMI_PREDEF;
		testchosen = true;

		diag_printf("\r\nPredefined test: %s\r\n",
				CS42XXTestHMIDescr[testid]);
	}
	// else not 1st function call, so CS42XX_TEST_NONE is selected.

#else // !CS42XX_TESTHMI_PREDEF
	char c;

	if (bshow)
	{
		// Display all available items
		diag_printf("\r\n----- CS42XX driver testing -----");

		for (testid = CS42XX_TEST_NONE+1; testid < CS42XX_TEST_NB; testid++)
		{
			diag_printf("\r\n%c: %s",
					('a' + testid - 1), CS42XXTestHMIDescr[testid]);
		}
	}
	diag_printf("\r\n-----\r\nPress a key: ");

	// Wait for a choice from the tester
	testid = CS42XX_TEST_NONE;
	HAL_DIAG_READ_CHAR(c);

	if (((cyg_uint8)c >= (cyg_uint8)'a'-1) &&
		((cyg_uint8)c <  (cyg_uint8)'a'+CS42XX_TEST_NB-1))
	{
		testid = (CS42XX_TEST_ID)((cyg_uint8)c + 1 - 'a');
		diag_printf("\r\nSelected %c: %s\r\n",
				c, CS42XXTestHMIDescr[testid]);
	}
	else
	{
		diag_printf("\r\nInvalid: %c\r\n", c);
	}

#endif // CS42XX_TESTHMI_PREDEF

	return testid;
}


/** Display the error code of a classic function (err and associated
 *  string).
 *
 * @param[in] testresult   Test result to be displayed
 ****************************************************************************/
 void
CS42XX_TestHMI_PostResult(Cyg_test_code testresult)
{
	cyg_test_output(testresult, "", 0, "");
}



#endif // CS42XX_TEST_SUPPORT
