//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Texas Instrument TPS65023 PMU.
 *
 * @file   src/tps65023.c
 * @brief  Wolfson tps65023 driver.
 *
 * Code is based on the DS_tps65023.pdf in the doc directory of this package
 *
 * This driver is using the i2c bus to configure the codec
 * @author Yves LEMOINE
 * @date   2007-04-23
 *
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.com
 ****************************************************************************
 */

#include <cyg/io/devtab.h>
#include <cyg/infra/cyg_ass.h>
#include <cyg/io/i2c.h>
#include <pkgconf/devs_io_i2c_arm_parrot5.h> //CYGNUM_DEVS_IO_I2C_ARM_PARROT5_PRESCALE
#include <cyg/devs/tps65023_regs.h>                 // TPS65023 PMU configuration


static cyg_bool  tps65023_init     ( struct cyg_devtab_entry * );

static Cyg_ErrNo tps65023_lookup   ( struct cyg_devtab_entry **tab, 
                                   struct cyg_devtab_entry *sub_tab,
                                   const char              *name );
static Cyg_ErrNo tps65023_getconfig(cyg_io_handle_t handle, 
                                  cyg_uint32      key, 
                                  void            *buf, 
                                  cyg_uint32      *len);
static Cyg_ErrNo tps65023_setconfig(cyg_io_handle_t handle, 
                                  cyg_uint32      key, 
                                  const void      *buf, 
                                  cyg_uint32      *len);
/** Item for the table of devices. */
DEVIO_TABLE(cyg_io_tps65023_devio, NULL, NULL, 0, tps65023_getconfig, tps65023_setconfig);
/** Operations applicable to the device. */
DEVTAB_ENTRY(tps65023, 
             "/dev/tps65023",
             0,                     // Does not depend on a lower level interface
             &cyg_io_tps65023_devio, 
             tps65023_init, 
             tps65023_lookup,
             NULL
    );


// -------------------------------------------------------------------------
//
// Configuration of the TPS65023 PMU 
//
extern cyg_i2c_bus    parrot5_i2c_bus;
//extern void parrot5_i2c_init( cyg_i2c_bus* bus);
   CYG_I2C_DEVICE(TPS65023_i2c_device,
                  &parrot5_i2c_bus,
                  TPS65023_ADDRESS,
                  0x00,
                  CYG_I2C_DEFAULT_DELAY);


/** \brief Write a register in the TPS65023 device through the i2c bus
 *
 * @param addr Register address (coded on 8 bits)
 * @param data (coded on 8 bits)
 */
void tps65023_i2c_write ( cyg_uint8 addr , cyg_uint8 data)
{
  cyg_uint32 result;

  cyg_i2c_transaction_begin(&TPS65023_i2c_device); // lock
  result = cyg_i2c_transaction_tx(&TPS65023_i2c_device, true, &addr, 1, false);   // send the address: start, no stop
  CYG_ASSERT(result == 1 , "ERROR : Sending on i2c");
  result = cyg_i2c_transaction_tx(&TPS65023_i2c_device, false, &data, 1, true);// send the data: start, stop
  CYG_ASSERT(result == 1 , "ERROR : Sending on i2c");
  cyg_i2c_transaction_end(&TPS65023_i2c_device);// unlock
}


/** \brief Read a register in the TPS65023 device through the i2c bus
 *
 * @param addr Register address (coded on 8 bits)
 * @pointer on param data (coded on 8 bits)
 */
void tps65023_i2c_read ( cyg_uint8 addr , cyg_uint8 * data)
{
  cyg_uint32 result;

  cyg_i2c_transaction_begin(&TPS65023_i2c_device); // lock
  result = cyg_i2c_transaction_tx(&TPS65023_i2c_device, true, &addr, 1, true);   // send the address: start, no stop
  CYG_ASSERT(result == 1 , "ERROR : Sending on i2c");
  result = cyg_i2c_transaction_rx(&TPS65023_i2c_device, true, data, 1, true, true);// send the data: start, stop
  CYG_ASSERT(result == 1 , "ERROR : receiving on i2c");
  cyg_i2c_transaction_end(&TPS65023_i2c_device);// unlock
}

/** \brief Program the core voltage in mV of TPS65023 device through the i2c bus
 *
 * @param mVCore : DCDC1 core voltage in mV (coded on 32 bits)
 */
void tps65023_set_core_voltage ( cyg_uint32 mVCore)
{
  cyg_uint8 value;
  
//  tps65023_i2c_read(PMU_VERSION,&value);
//  printf("TPS65023 version is 0x%x\n", value);
  if(mVCore < 1000)
  	printf("The value is too low (<1000mV)\n");
  if(mVCore > 1400)
  	printf("The value is too high (>1400mV)\n");
	  
  tps65023_i2c_write(PMU_CON_CTRL2,(char) 0x0); //to allow defcore adjust
  tps65023_i2c_write(PMU_DEFCORE,(char) (mVCore/25 - 800));
  tps65023_i2c_write(PMU_CON_CTRL2,(char) 0x80); //set GO bit to program defcore 
  tps65023_i2c_read(PMU_DEFCORE,&value);
  printf("TPS65023 core voltage is %d mV\n", 800 + (int)value*25);
  
}




/** \brief Lookup interface of TPS65023 PMU
 *
 *
 */
static Cyg_ErrNo tps65023_lookup(struct cyg_devtab_entry **tab, 
                          struct cyg_devtab_entry *sub_tab,
                          const char              *name)
{
   Cyg_ErrNo err = ENOERR;
   return err;
}



/** \brief SetConfig interface of TPS65023 PMU
 *
 *
 */
static Cyg_ErrNo tps65023_setconfig( cyg_io_handle_t handle, 
                              cyg_uint32      key, 
                              const void      *buf, 
                              cyg_uint32      *len)
{



   Cyg_ErrNo err = ENOERR;
   return err;

}


/** \brief GetConfig interface of TPS65023 PMU
 *
 *
 */
static Cyg_ErrNo tps65023_getconfig( cyg_io_handle_t handle, 
                              cyg_uint32      key, 
                              void      *buf, 
                              cyg_uint32      *len)
{



   Cyg_ErrNo err = ENOERR;
   return err;

}


/** \brief Iinitialize the TPS65023 PMU
 *
 * Configure the i2s bus.
 * Configure the i2c bus master clock with the value defined in the configtool
 *
 */
static cyg_bool tps65023_init( struct cyg_devtab_entry *tab )
{
   Cyg_ErrNo err = ENOERR;

   // i2c master selection
   SELECT_PIN(SCL1);
   SELECT_PIN(SDA1);
   
   HAL_WRITE_UINT32 ( PARROT5_SYS  + _SYS_CEN , SYS_CEN_I2CMCLK );


   return err;
}





