//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Wolfson WM8731 stereo audio codec.
 *
 * @file   WM8731.c
 * @brief  Core of driver.
 *
 * Code is based on the WM8731.pdf
 *
 * @author Marc Flambard
 * @date   2007-05-24
 *
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */

/*#include<string.h>*/
#include<cyg/devs/wm8731.h>
#include<cyg/devs/wm8731_i2c.h>

void my_memcpy(void* target,const void* source,int size)
{
 	while(size) {size--;((char*)target)[size]=((char*)source)[size];}
 	return;
}


/*
 * Defines.
 ***********/

/** Reset values of WM8731 Settings.
 * @def WM8731_SETTINGS_RESET
 */
#define WM8731_SETTINGS_RESET		\
	{								\
	.linvol=WM8731_INVOL_0DB,		\
	.linmute=1,						\
	.lrinboth=0,						\
	.rinvol=WM8731_INVOL_0DB,		\
	.rinmute=1,						\
	.rlinboth=0,						\
	.lhpvol=WM8731_HPVOL_0DB,		\
	.lhpvol_old=WM8731_HPVOL_0DB,	\
	.lhpmute=0,						\
	.lzcen=0,						\
	.lrhpboth=0,						\
	.rhpvol=WM8731_HPVOL_0DB,		\
	.rhpvol_old=WM8731_HPVOL_0DB,	\
	.rhpmute=0,						\
	.rzcen=0,						\
	.rlhpboth=0,						\
	.micboost=0,						\
	.mutemic=1,						\
	.insel=WM8731_INSEL_LINE,		\
	.bypass=1,						\
	.dacsel=0,						\
	.sidetone=0,						\
	.sideatt=WM8731_SIDEATT_M6DB		\
	}

/** Reset values of WM8731 Configs.
 * @def WM8731_CONFIG_RESET
 */
#define WM8731_CONFIG_RESET			\
	{								\
	.adchpd=0,						\
	.deemp=WM8731_DEEMP_DISABLE,		\
	.dacmu=1,						\
	.hpor=0,							\
	.power.lineinpd=1,				\
	.power.micpd=1,					\
	.power.adcpd=1,					\
	.power.dacpd=1,					\
	.power.outpd=1,					\
	.power.oscpd=0,					\
	.power.clkoutpd=0,				\
	.power.poweroff=1,				\
	.format=WM8731_FORMAT_I2S,		\
	.iwl=WM8731_IWL_24B,				\
	.lrp=0,							\
	.lrswap=0,						\
	.ms=0,							\
	.bclkinv=0,						\
	.usb_norm=0,						\
	.bosr=0,							\
	.sr=0,							\
	.clkidiv2=0,						\
	.clkodiv2=0,						\
	.active=0						\
	}

/**
 * defines to fill WM8731 register
 * with WM8731_settings_actu and WM8731_config_actu
 */
#define R0_COMPOSE(x) 								\
	((WM8731_settings_actu[(x)].linvol&0x1F) |		\
	(WM8731_settings_actu[(x)].linmute? 1:0)<<7 |	\
	(WM8731_settings_actu[(x)].lrinboth? 1:0)<<8)

#define R1_COMPOSE(x) 								\
	((WM8731_settings_actu[(x)].rinvol&0x1F) |		\
	(WM8731_settings_actu[(x)].rinmute? 1:0)<<7 |	\
	(WM8731_settings_actu[(x)].rlinboth? 1:0)<<8)

#define R2_COMPOSE(x) 								\
	((WM8731_settings_actu[(x)].lhpvol&0x7F) |		\
	(WM8731_settings_actu[(x)].lzcen? 1:0)<<7 |		\
	(WM8731_settings_actu[(x)].lrhpboth? 1:0)<<8)

#define R3_COMPOSE(x) 								\
	((WM8731_settings_actu[(x)].rhpvol&0x7F) |		\
	(WM8731_settings_actu[(x)].rzcen? 1:0)<<7 |		\
	(WM8731_settings_actu[(x)].rlhpboth? 1:0)<<8)

#define R4_COMPOSE(x) 								\
	((WM8731_settings_actu[(x)].micboost? 1:0) |		\
	(WM8731_settings_actu[(x)].mutemic? 1:0)<<1 |	\
	(WM8731_settings_actu[(x)].insel? 1:0)<<2 |		\
	(WM8731_settings_actu[(x)].bypass? 1:0)<<3 |	\
	(WM8731_settings_actu[(x)].dacsel? 1:0)<<4 |	\
	(WM8731_settings_actu[(x)].sidetone? 1:0)<<5 |	\
	(WM8731_settings_actu[(x)].sideatt&3)<<6)

#define R5_COMPOSE(x) 								\
	((WM8731_config_actu[(x)].adchpd? 1:0) |			\
	(WM8731_config_actu[(x)].deemp&3)<<1 |			\
	(WM8731_config_actu[(x)].dacmu? 1:0)<<3 |		\
	(WM8731_config_actu[(x)].hpor? 1:0)<<4)

#define R6_COMPOSE(x) 								\
	((WM8731_config_actu[(x)].power.lineinpd? 1:0) |		\
	(WM8731_config_actu[(x)].power.micpd? 1:0)<<1 |		\
	(WM8731_config_actu[(x)].power.adcpd? 1:0)<<2 |		\
	(WM8731_config_actu[(x)].power.dacpd? 1:0)<<3 |		\
	(WM8731_config_actu[(x)].power.outpd? 1:0)<<4 |		\
	(WM8731_config_actu[(x)].power.oscpd? 1:0)<<5 |		\
	(WM8731_config_actu[(x)].power.clkoutpd? 1:0)<<6 |	\
	(WM8731_config_actu[(x)].power.poweroff? 1:0)<<7)

#define R7_COMPOSE(x) 								\
	((WM8731_config_actu[(x)].format&3) |			\
	(WM8731_config_actu[(x)].iwl&3)<<2 |			\
	(WM8731_config_actu[(x)].lrp? 1:0)<<4 |			\
	(WM8731_config_actu[(x)].lrswap? 1:0)<<5 |		\
	(WM8731_config_actu[(x)].ms? 1:0)<<6 |			\
	(WM8731_config_actu[(x)].bclkinv? 1:0)<<7)

#define R8_COMPOSE(x) 								\
	((WM8731_config_actu[(x)].usb_norm? 1:0) |		\
	(WM8731_config_actu[(x)].bosr? 1:0)<<1 |		\
	(WM8731_config_actu[(x)].sr&0xF)<<2 |			\
	(WM8731_config_actu[(x)].clkidiv2? 1:0)<<6 |	\
	(WM8731_config_actu[(x)].clkodiv2? 1:0)<<7)

#define R9_COMPOSE(x) 								\
	(WM8731_config_actu[(x)].active&1)


/*
 * Consts.
 *************/

/**
 * @var WM8731_config_reset
 */
const WM8731_CONFIG WM8731_config_reset=WM8731_CONFIG_RESET;

/**
 * @var WM8731_settings_reset
 */
const WM8731_SETTINGS WM8731_settings_reset=WM8731_SETTINGS_RESET;

/*
 * Variables.
 *************/

/** List of WM8731 devices.
 *
 * @var WM8731_config_actu
 */

WM8731_CONFIG WM8731_config_actu[WM8731_NBDEVICES] = {
	WM8731_CONFIG_RESET,
#if WM8731_NBDEVICES > 1
	WM8731_CONFIG_RESET
#endif
};

/**
 * @var WM8731_settings_actu
 */

WM8731_SETTINGS WM8731_settings_actu[WM8731_NBDEVICES] = {
	WM8731_SETTINGS_RESET,
#if WM8731_NBDEVICES > 1
	WM8731_SETTINGS_RESET
#endif
};


/** Initialize WM8731.
 * Initialize an attach WM8731 to I2c bus.
 *
 * @param[in] devid  Symbolic device Id
 * @param[in] bus    Bus to attach device to
 * @param[in] ad0    State of ad0 pin on audio device
 *
 * @retval  ENOERR  Operation successful
 * @retval -EINVAL  Invalid parameters
 * @retval -ENODEV  No such device
 * @retval -EIO     Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Init(const WM8731_DEVID devid, const cyg_i2c_bus *bus,
		cyg_bool ad0)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		err=WM8731_I2C_AttachBus(devid,bus,ad0);
		if(err==ENOERR) err=WM8731_Reset(devid);
		}
	
	return err;
}


/** Reset WM8731.
 * Reset all WM8731 register.
 *
 * @param[in] devid  Symbolic device Id
 *
 * @retval  ENOERR  Operation successful
 * @retval -ENODEV  No such device
 * @retval -EIO     Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Reset(const WM8731_DEVID devid)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)&(WM8731_config_actu[devid]),(void*)&WM8731_config_reset,sizeof(WM8731_CONFIG));
		my_memcpy((void*)&(WM8731_settings_actu[devid]),(void*)&WM8731_settings_reset,sizeof(WM8731_SETTINGS));
		err=WM8731_I2C_Write(devid,WM8731_R10_RESET,WM8731_R_RESET);
		}
	
	return err;
}

/** Fill a config structure according to the current configuration.
 *
 * @param[in]  devid          Symbolic device Id
 * @param[out] config_target  Pointer to configuration structure
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Config_Get(const WM8731_DEVID devid, WM8731_CONFIG *config_target)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)config_target,(void*)&(WM8731_config_actu[devid]),sizeof(WM8731_CONFIG));
		err=ENOERR;
		}
	
	return err;
}

/** Apply configuration structure.
 *
 * @note for power control, please use WM8731_Power_Config_Set to avoid pops
 *
 * @param[in] devid           Symbolic device Id
 * @param[in] config_source   Pointer to configuration structure
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Config_Set(const WM8731_DEVID devid, const WM8731_CONFIG *config_source)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)&(WM8731_config_actu[devid]),(void*)config_source,sizeof(WM8731_CONFIG));
		err=WM8731_I2C_Write(devid,WM8731_R5_D_AUDIO_PATH,R5_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R7_D_AUDIO_FRMT,R7_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R8_SAMPLING,R8_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R9_ACTIVE,R9_COMPOSE(devid));
		}
	
	return err;
}

/** Fill a settings structure according to the current settings.
 *
 * @param[in]  devid            Symbolic device Id
 * @param[out] settings_target  Pointer to settings structure
 * 
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Settings_Get(const WM8731_DEVID devid, WM8731_SETTINGS *settings_target)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)settings_target,(void*)&(WM8731_settings_actu[devid]),sizeof(WM8731_SETTINGS));
		err=ENOERR;
		}
	
	return err;
}

/** Apply settings structure.
 *
 * @param[in] devid             Symbolic device Id
 * @param[in] settings_source   Pointer to settings structure
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Settings_Set(const WM8731_DEVID devid, const WM8731_SETTINGS *settings_source)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)&(WM8731_settings_actu[devid]),(void*)settings_source,sizeof(WM8731_SETTINGS));
		err=WM8731_I2C_Write(devid,WM8731_R0_LIN,R0_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R1_RIN,R1_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R2_LHP,R2_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R3_RHP,R3_COMPOSE(devid));
		if(err==ENOERR) err=WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid));
		}
	
	return err;
}

/** Get vol state of left line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] vol     Pointer to volume setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetLInVol(const WM8731_DEVID devid, cyg_uint8 *vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*vol=WM8731_settings_actu[devid].linvol;
		err=ENOERR;
		}
	
	return err;
}

/** Set vol state of left line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  vol     New volume setting
 *
 * @note  volume must be set between 0 and WM8731_INVOL_MAX
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EINVAL   Invalid parameters
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetLInVol(const WM8731_DEVID devid, const cyg_uint8 vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		err=-EINVAL;
		if(vol<=WM8731_INVOL_MAX)
			{
			WM8731_settings_actu[devid].lrinboth=0;
			WM8731_settings_actu[devid].linvol=vol;
			err=WM8731_I2C_Write(devid,WM8731_R0_LIN,R0_COMPOSE(devid));
			}
		}
	
	return err;
}

/** Get mute state of left line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] mute    Pointer to mute setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetLInMute(const WM8731_DEVID devid, cyg_bool *mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*mute=WM8731_settings_actu[devid].linmute;
		err=ENOERR;
		}
	
	return err;
}

/** Set mute state of left line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  mute    New mute setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetLInMute(const WM8731_DEVID devid, const cyg_bool mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		WM8731_settings_actu[devid].lrinboth=0;
		WM8731_settings_actu[devid].linmute=mute;
		err=WM8731_I2C_Write(devid,WM8731_R0_LIN,R0_COMPOSE(devid));
		}
	
	return err;
}

/** Get vol state of right line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] vol     Pointer to volume setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetRInVol(const WM8731_DEVID devid, cyg_uint8 *vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*vol=WM8731_settings_actu[devid].rinvol;
		err=ENOERR;
		}
	
	return err;
}

/** Set vol state of right line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  vol     New volume setting
 *
 * @note  volume must be set between 0 and WM8731_INVOL_MAX
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EINVAL   Invalid parameters
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetRInVol(const WM8731_DEVID devid, const cyg_uint8 vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		err=-EINVAL;
		if(vol<=WM8731_INVOL_MAX)
			{
			WM8731_settings_actu[devid].rlinboth=0;
			WM8731_settings_actu[devid].rinvol=vol;
			err=WM8731_I2C_Write(devid,WM8731_R1_RIN,R1_COMPOSE(devid));
			}
		}
	
	return err;
}

/** Get mute state of right line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] mute    Pointer to mute setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetRInMute(const WM8731_DEVID devid, cyg_bool *mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*mute=WM8731_settings_actu[devid].rinmute;
		err=ENOERR;
		}
	
	return err;
}

/** Set mute state of right line input.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  mute    New mute setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetRInMute(const WM8731_DEVID devid, const cyg_bool mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		WM8731_settings_actu[devid].rlinboth=0;
		WM8731_settings_actu[devid].rinmute=mute;
		err=WM8731_I2C_Write(devid,WM8731_R1_RIN,R1_COMPOSE(devid));
		}
	
	return err;
}

/** Get vol state of left head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] vol     Pointer to volume setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetLHpVol(const WM8731_DEVID devid, cyg_uint8 *vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		if(!WM8731_settings_actu[devid].lhpmute)
			*vol=WM8731_settings_actu[devid].lhpvol;
		else
			*vol=WM8731_settings_actu[devid].lhpvol_old;
		err=ENOERR;
		}
	
	return err;
}

/** Set vol state of left head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  vol     New volume setting
 *
 * @note  volume must be set between WM8731_HPVOL_MIN and WM8731_HPVOL_MAX
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EINVAL   Invalid parameters
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetLHpVol(const WM8731_DEVID devid, const cyg_uint8 vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		err=-EINVAL;
		if(vol>=WM8731_HPVOL_MIN && vol<=WM8731_HPVOL_MAX)
			{
			if(!WM8731_settings_actu[devid].lhpmute)
				{
				WM8731_settings_actu[devid].lrhpboth=0;
				WM8731_settings_actu[devid].lhpvol=vol;
				err=WM8731_I2C_Write(devid,WM8731_R2_LHP,R2_COMPOSE(devid));
				}
			else
				{
				WM8731_settings_actu[devid].lhpvol_old=vol;
				err=ENOERR;
				}
			}
		}
	
	return err;
}

/** Get mute state of left head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] mute    Pointer to mute setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetLHpMute(const WM8731_DEVID devid, cyg_bool *mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*mute=WM8731_settings_actu[devid].lhpmute;
		err=ENOERR;
		}
	
	return err;
}

/** Set mute state of left head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  mute    New mute setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetLHpMute(const WM8731_DEVID devid, const cyg_bool mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		err=ENOERR;
		WM8731_settings_actu[devid].lrhpboth=0;
		if(mute && !WM8731_settings_actu[devid].lhpmute)
			{
			WM8731_settings_actu[devid].lhpmute=1;
			WM8731_settings_actu[devid].lhpvol_old=WM8731_settings_actu[devid].lhpvol;
			WM8731_settings_actu[devid].lhpvol=0;
			err=WM8731_I2C_Write(devid,WM8731_R2_LHP,R2_COMPOSE(devid));
			}
		else if(!mute && WM8731_settings_actu[devid].lhpmute)
			{
			WM8731_settings_actu[devid].lhpmute=0;
			WM8731_settings_actu[devid].lhpvol=WM8731_settings_actu[devid].lhpvol_old;
			err=WM8731_I2C_Write(devid,WM8731_R2_LHP,R2_COMPOSE(devid));
			}
		}
	
	return err;
}

/** Get zcen state of left head phone output.
 * Left Channel Zero Cross detect to minimises clicks and zipper noise
 * as the gain values are changed or the channel muted.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] zcen    Pointer to zcen setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetLHpZcen(const WM8731_DEVID devid, cyg_bool *zcen)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*zcen=WM8731_settings_actu[devid].lzcen;
		err=ENOERR;
		}
	
	return err;
}

/** Set zcen state of left head phone output.
 * Left Channel Zero Cross detect to minimises clicks and zipper noise
 * as the gain values are changed or the channel muted.
 *
 * @note That this circuit has no time out so if only DC levels are being
 *       applied to the gain stage input of more than approximately 20mV,
 *       then the gain will not be updated
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  zcen    New zcen setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetLHpZcen(const WM8731_DEVID devid, const cyg_bool zcen)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		WM8731_settings_actu[devid].lrhpboth=0;
		WM8731_settings_actu[devid].lzcen=zcen;
		err=WM8731_I2C_Write(devid,WM8731_R2_LHP,R2_COMPOSE(devid));
		}
	
	return err;
}

/** Get vol state of right head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] vol     Pointer to volume setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetRHpVol(const WM8731_DEVID devid, cyg_uint8 *vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		if(!WM8731_settings_actu[devid].rhpmute)
			*vol=WM8731_settings_actu[devid].rhpvol;
		else
			*vol=WM8731_settings_actu[devid].rhpvol_old;
		err=ENOERR;
		}
	
	return err;
}

/** Set vol state of right head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  vol     New volume setting
 *
 * @note  volume must be set between WM8731_HPVOL_MIN and WM8731_HPVOL_MAX
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EINVAL   Invalid parameters
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetRHpVol(const WM8731_DEVID devid, const cyg_uint8 vol)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
	{
		err=-EINVAL;
		if(vol>=WM8731_HPVOL_MIN && vol<=WM8731_HPVOL_MAX)
		{
			if(!WM8731_settings_actu[devid].rhpmute)
			{
				WM8731_settings_actu[devid].rlhpboth=0;
				WM8731_settings_actu[devid].rhpvol=vol;
				err=WM8731_I2C_Write(devid,WM8731_R3_RHP,R3_COMPOSE(devid));
			}
			else
			{
				WM8731_settings_actu[devid].rhpvol_old=vol;
				err=ENOERR;
			}
		}
	}
	
	return err;
}

/** Get mute state of right head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] mute    Pointer to mute setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetRHpMute(const WM8731_DEVID devid, cyg_bool *mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*mute=WM8731_settings_actu[devid].rhpmute;
		err=ENOERR;
		}
	
	return err;
}

/** Set mute state of right head phone output.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  mute    New mute setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetRHpMute(const WM8731_DEVID devid, const cyg_bool mute)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
	{
		err=ENOERR;
		WM8731_settings_actu[devid].rlhpboth=0;
		if(mute && !WM8731_settings_actu[devid].rhpmute)
		{
			WM8731_settings_actu[devid].rhpmute=1;
			WM8731_settings_actu[devid].rhpvol_old=WM8731_settings_actu[devid].rhpvol;
			WM8731_settings_actu[devid].rhpvol=0;
			err=WM8731_I2C_Write(devid,WM8731_R3_RHP,R3_COMPOSE(devid));
		}
		else if(!mute && WM8731_settings_actu[devid].rhpmute)
		{
			WM8731_settings_actu[devid].rhpmute=0;
			WM8731_settings_actu[devid].rhpvol=WM8731_settings_actu[devid].rhpvol_old;
			err=WM8731_I2C_Write(devid,WM8731_R3_RHP,R3_COMPOSE(devid));
		}
	}
	
	return err;
}

/** Get zcen state of right head phone output.
 * Right Channel Zero Cross detect to minimises clicks and zipper noise
 * as the gain values are changed or the channel muted.
 *
 * @param[in]  devid   Symbolic device Id
 * @param[out] zcen    Pointer to zcen setting variable
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_GetRHpZcen(const WM8731_DEVID devid, cyg_bool *zcen)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		*zcen=WM8731_settings_actu[devid].rzcen;
		err=ENOERR;
		}
	
	return err;
}

/** Set zcen state of right head phone output.
 * Right Channel Zero Cross detect to minimises clicks and zipper noise
 * as the gain values are changed or the channel muted.
 *
 * @note That this circuit has no time out so if only DC levels are being
 *       applied to the gain stage input of more than approximately 20mV,
 *       then the gain will not be updated
 *
 * @param[in]  devid   Symbolic device Id
 * @param[in]  zcen    New zcen setting
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_SetRHpZcen(const WM8731_DEVID devid, const cyg_bool zcen)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		WM8731_settings_actu[devid].rlhpboth=0;
		WM8731_settings_actu[devid].rzcen=zcen;
		err=WM8731_I2C_Write(devid,WM8731_R3_RHP,R3_COMPOSE(devid));
		}
	
	return err;
}

/** Fill a power config structure according to the current configuration.
 *
 * @param[in]  devid          Symbolic device Id
 * @param[out] config_target  Pointer to power configuration structure
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Power_Config_Get(const WM8731_DEVID devid, WM8731_POWER *config_target)
{
	Cyg_ErrNo       err = -ENODEV;

	if (devid<WM8731_NBDEVICES)
		{
		my_memcpy((void*)config_target,(void*)&(WM8731_config_actu[devid].power),sizeof(WM8731_POWER));
		err=ENOERR;
		}
	
	return err;
}

/** Apply power configuration structure.
 *
 * @note This function tend to avoid pops on power changes
 *
 * @param[in] devid           Symbolic device Id
 * @param[in] config_source   Pointer to power configuration structure
 *
 * @retval  ENOERR   Success
 * @retval -ENODEV   No such device
 * @retval -EIO      Error during access to device
 ****************************************************************************/
Cyg_ErrNo
WM8731_Power_Config_Set(const WM8731_DEVID devid, const WM8731_POWER *config_source)
{
	Cyg_ErrNo       err = -ENODEV;
	
	if (devid<WM8731_NBDEVICES)
		{
		/*
		 * anti-pop routines
		 */
		/* mic anti-pop */
		if(config_source->micpd!=WM8731_config_actu[devid].power.micpd
		   && WM8731_settings_actu[devid].insel==WM8731_INSEL_MIC)
			{
			/*line selection*/
			WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)&~WM8731_R_INSEL);
			/*toggle Microphone Input power*/
			WM8731_config_actu[devid].power.micpd=config_source->micpd;
			WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
			/*reselect mic*/
			WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)|WM8731_R_INSEL);
			}
		/* adc anti-pop */
		if(config_source->adcpd!=WM8731_config_actu[devid].power.adcpd)
			{
			/*mic is seleced and not muted*/
			if(WM8731_settings_actu[devid].insel==WM8731_INSEL_MIC)
				{
				if(!WM8731_settings_actu[devid].mutemic)
					{
					/*mic mute*/
					WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)|WM8731_R_MUTEMIC);
					/*toggle adc power*/
					WM8731_config_actu[devid].power.adcpd=config_source->adcpd;
					WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
					/*demute mic*/
					WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)&~WM8731_R_MUTEMIC);
					}
				}
			/*line is seleced and not muted*/
			else if(!WM8731_settings_actu[devid].linmute || !WM8731_settings_actu[devid].rinmute)
				{
				/*memorise initial line mute for each channels*/
				cyg_bool linmute=WM8731_settings_actu[devid].linmute;
				cyg_bool rinmute=WM8731_settings_actu[devid].rinmute;
				/*mute line*/
				WM8731_I2C_Write(devid,WM8731_R0_LIN,R0_COMPOSE(devid)|WM8731_R_LINMUTE);
				WM8731_I2C_Write(devid,WM8731_R1_RIN,R1_COMPOSE(devid)|WM8731_R_RINMUTE);
				/*toggle adc power*/
				WM8731_config_actu[devid].power.adcpd=config_source->adcpd;
				WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
				/*demute line if originaly not muted*/
				if(!linmute) WM8731_I2C_Write(devid,WM8731_R0_LIN,R0_COMPOSE(devid)&~WM8731_R_LINMUTE);
				if(!rinmute) WM8731_I2C_Write(devid,WM8731_R1_RIN,R1_COMPOSE(devid)&~WM8731_R_RINMUTE);
				}
			}
		/* dac anti-pop */
		if(config_source->dacpd!=WM8731_config_actu[devid].power.dacpd
		   && WM8731_settings_actu[devid].dacsel)
			{
			/*mute dac if originaly not*/
			if(!WM8731_config_actu[devid].dacmu)
				WM8731_I2C_Write(devid,WM8731_R5_D_AUDIO_PATH,R5_COMPOSE(devid)|WM8731_R_DACMU);
			/*deselect dac*/
			WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)&~WM8731_R_DACSEL);
			/*toggle dac power*/
			WM8731_config_actu[devid].power.dacpd=config_source->dacpd;
			WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
			/*reselect dac*/
			WM8731_I2C_Write(devid,WM8731_R4_A_AUDIO_PATH,R4_COMPOSE(devid)|WM8731_R_DACSEL);
			/*demute dac if originaly not muted*/
			if(!WM8731_config_actu[devid].dacmu)
				WM8731_I2C_Write(devid,WM8731_R5_D_AUDIO_PATH,R5_COMPOSE(devid)&~WM8731_R_DACMU);
			}
		/* out anti-pop */
		if(config_source->outpd!=WM8731_config_actu[devid].power.outpd
		   && !WM8731_config_actu[devid].dacmu)
			{
			/*mute dac*/
			WM8731_I2C_Write(devid,WM8731_R5_D_AUDIO_PATH,R5_COMPOSE(devid)|WM8731_R_DACMU);
			/*toggle dac power*/
			WM8731_config_actu[devid].power.outpd=config_source->outpd;
			WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
			/*demute dac*/
			WM8731_I2C_Write(devid,WM8731_R5_D_AUDIO_PATH,R5_COMPOSE(devid)&~WM8731_R_DACMU);
			}
		/*synchronise cash and register*/
		my_memcpy((void*)&(WM8731_config_actu[devid].power),(void*)config_source,sizeof(WM8731_POWER));
		err=WM8731_I2C_Write(devid,WM8731_R6_POWER,R6_COMPOSE(devid));
		}
	
	return err;
}

