//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ***************************************************************************
 * @file wm8974.h
 *
 * @brief WM8974 digital amplifier driver.
 *
 * Copyright (C) 2006 Parrot S.A.
 *
 * @author	francois.ferrand@parrot.fr
 * @author	jerome.leveque@parrot.fr
 * @date	07/2006
 ***************************************************************************
 */

#include <cyg/infra/cyg_ass.h>
#include <cyg/devs/wm8974.h>

/** Digital amplifier I2C device.
 */
CYG_I2C_DEVICE(WM8974_i2c_device,		\
			   NULL,					\
			   0x1A,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);

typedef struct init_reg_
{
	cyg_uint8 name;
	cyg_uint16 init;
} init_reg;

static init_reg regs[] = {
	{WM8974_REG_POWER_MNG_1, 0x00},
	{WM8974_REG_POWER_MNG_2, 0x00},
	{WM8974_REG_POWER_MNG_3, 0x00},
	
	
	{0xFF, 0x00} //End of the list
};

/** Initialize the amplifier.
 * This function should be used on startup, to configure the amplifier. It
 * does not perform a reset, so the caller should take care of that before
 * calling the function, to ensure the parameters are properly set.
 *
 * Once the function has been called, the other APIs can be used to perform
 * other commands (setup, volume change...).
 *
 * @param config A structure containing the configuration of the amplifier.
 * This structure is used only during the call, it does not need to be valid
 * afterward.
 */
cyg_bool WM8974_Init(cyg_i2c_bus* bus)
{
	cyg_bool result = true;
	int i = 0;

	CYG_ASSERT(bus, "NULL pointer arguments: bus");
	CYG_ASSERT(config, "NULL pointer arguments: config");

	WM8974_i2c_device.i2c_bus = bus;
	
	while (regs[i].name != 0xFF)
	{
		result &= WM8974_Write(regs[0].name, regs[0].init);
		if (result)
			i++;
		else
			break;
	}
	return result;
}

/** Mute any of the channels.
 * @param mute A combination of the following flags:
 * 			- WM8974_MUTE_MASTER mute all channels
 *			- WM8974_MUTE_CHAN1 mute channel 1
 * 			- WM8974_MUTE_CHAN2 mute channel 2
 * 			- WM8974_MUTE_CHAN3 mute channel 3
 *
 * To unmute, call the WM8974_UnMute() function.
 * A channel is actually muted if WM8974_MUTE_CHAN<x> or WM8974_MUTE_MASTER
 * is set. To unmute it, both flags must be passed to the unmute function.
 */
/*
void WM8974_Mute(cyg_uint8 mute)
{
	cyg_mutex_lock(&WM8974_mute_mutex);
	if ((WM8974_mute | mute) != WM8974_mute)
	{
		WM8974_mute |= mute;
		WM8974_Write(WM8974_REG_Mute, &WM8974_mute, 1);
	}
	cyg_mutex_unlock(&WM8974_mute_mutex);
}
*/
/** Unmute any of the channels.
 * @param mute A combination of the following flags:
 * 			- WM8974_MUTE_MASTER unmute all channels
 *			- WM8974_MUTE_CHAN1 unmute channel 1
 * 			- WM8974_MUTE_CHAN2 unmute channel 2
 * 			- WM8974_MUTE_CHAN3 unmute channel 3
 *
 * A channel is actually muted if WM8974_MUTE_CHAN<x> or WM8974_MUTE_MASTER
 * is set. To unmute it, both flags must be passed to the function.
 */
/*
void WM8974_UnMute(cyg_uint8 mute)
{
	cyg_mutex_lock(&WM8974_mute_mutex);
	if (WM8974_mute & mute)
	{
		WM8974_mute &= ~mute;
		WM8974_Write(WM8974_REG_Mute, &WM8974_mute, 1);
	}
	cyg_mutex_unlock(&WM8974_mute_mutex);
}
*/
#define NB_DATA 2

cyg_bool WM8974_Write(cyg_uint8 addr, const cyg_uint16 data)
{
	cyg_uint8 packed_data[NB_DATA];
	cyg_uint32 result;

	CYG_ASSERT(WM8974_i2c_device.i2c_bus, "Call WM8974_Init() first");
	CYG_ASSERT(buf, "NULL pointer arguments: buf");

	packed_data[0] = addr << 1 | ((data >> 8) & 1);
	packed_data[1] = data & 0xFF;

	// lock
	cyg_i2c_transaction_begin(&WM8974_i2c_device);

	result = cyg_i2c_transaction_tx(&WM8974_i2c_device, true, packed_data, NB_DATA, true);

	// unlock
	cyg_i2c_transaction_end(&WM8974_i2c_device);
	
	return result == NB_DATA;
}

/*
void WM8974_Coeff_Write(WM8974_Coeff coeff, int value)
{
	char command[20];
	int pos = 0;

	CYG_ASSERT(WM8974_i2c_device.i2c_bus, "Call WM8974_Init() first");

	//lock
	cyg_i2c_transaction_begin(&WM8974_i2c_device);

	// First, send the coefficient
	command[0] = WM8974_REG_Cfaddr2;
	cyg_i2c_transaction_tx(&WM8974_i2c_device, true, command, 1, false);

	command[pos++] = coeff;
	CYG_ASSERT(value >=0 && value < 0x1000000, "coeffs are 24bit.");
	command[pos++] = (value >> 16) & 0xff;
	command[pos++] = (value >> 8) & 0xff;
	command[pos++] = (value) & 0xff;
	cyg_i2c_transaction_tx(&WM8974_i2c_device, false, command, pos, true);

	// Then, send write command
	command[0] = WM8974_REG_Cfud;
	cyg_i2c_transaction_tx(&WM8974_i2c_device, true, command, 1, false);

	command[0] = WM8974_Coeff_WriteOne;
	cyg_i2c_transaction_tx(&WM8974_i2c_device, false, command, 1, true);

	// unlock
	cyg_i2c_transaction_end(&WM8974_i2c_device);
}
*/
/*
void WM8974_Coeff_WriteBiquad(WM8974_Coeff_BIQUAD biquad, int b1, int b2, int a1, int a2, int b0)
{
	char command[20];
	int coeffs[5] = { b1, b2, a1, a2, b0 };
	int pos = 0;
	int i;

	command[pos++] = biquad;
	for(i=0; i<5; i++)
	{
		CYG_ASSERT(coeffs[i] >=0 && coeffs[i] < 0x1000000, "coeffs are 24bit.");
		command[pos++] = (coeffs[i] >> 16) & 0xff;
		command[pos++] = (coeffs[i] >> 8) & 0xff;
		command[pos++] = (coeffs[i]) & 0xff;
	}
	command[pos++] = WM8974_Coeff_WriteAll;

	WM8974_Write(WM8974_REG_Cfaddr2, command, pos);
}
*/
