//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
 * @file   omap3630_serial.c
 * @brief  OMAP3630 serial driver
 *
 * Copyright (C) 2004-2011 Parrot S.A.
 *
 * @author karl.leplat@parrot.com
 * @date   2011-03-31
*******************************************************************************
 */

#include <pkgconf/hal.h>
#include <pkgconf/infra.h>
#include <pkgconf/system.h>
#include <pkgconf/io_serial.h>
#include <pkgconf/io.h>
#include <pkgconf/kernel.h>

#include <cyg/io/io.h>
#include <cyg/io/serialio.h>
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/hal_io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_if.h>
#include <cyg/io/devtab.h>
#include <cyg/io/serial.h>
#include <cyg/infra/diag.h>
#include <cyg/infra/cyg_type.h>
#include <cyg/infra/cyg_ass.h>

#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630

#include "omap3630_serial.h"

#include <cyg/hal/omap3630.h>           // Register definition
#include <cyg/hal/omap3630_uart.h>
#include <cyg/hal/omap3630_cm.h>

#include <cyg/hal/hal_platform_ints.h>

//==========================================================================

#define OMAP3630_RXBUFSIZE      (64)

typedef struct omap3630_serial_info
{
    CYG_WORD            uart;
    CYG_ADDRWORD        base;
    CYG_WORD            int_num;
    CYG_WORD            fcr;
    CYG_WORD            ier;
    CYG_WORD            mcr;
    CYG_BYTE            hw_stopped; 

    volatile cyg_uint8  buf[OMAP3630_RXBUFSIZE];
    volatile int        buf_head;
    volatile int        buf_tail;

    cyg_interrupt       serial_interrupt;
    cyg_handle_t        serial_interrupt_handle;
} omap3630_serial_info;

#define CYG_IO_SERIAL_OMAP_INT_PRIORITY 4

//==========================================================================

static bool omap3630_serial_init(struct cyg_devtab_entry *tab);
static bool omap3630_serial_putc_interrupt(serial_channel *chan, unsigned char c);
#if (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE == 0)
static bool omap3630_serial_putc_polled(serial_channel *chan, unsigned char c);
#endif
static Cyg_ErrNo omap3630_serial_lookup(struct cyg_devtab_entry **tab,
                                    struct cyg_devtab_entry *sub_tab,
                                    const char *name);
static unsigned char omap3630_serial_getc_interrupt(serial_channel *chan);
#if (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE == 0)
static unsigned char omap3630_serial_getc_polled(serial_channel *chan);
#endif
static Cyg_ErrNo omap3630_serial_set_config(serial_channel *chan, cyg_uint32 key,
                                        const void *xbuf, cyg_uint32 *len);
static void omap3630_serial_start_xmit(serial_channel *chan);
static void omap3630_serial_stop_xmit(serial_channel *chan);

static cyg_uint32 omap3630_serial_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void       omap3630_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);

//==========================================================================

#if (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE > 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE > 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE > 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE > 0)
static SERIAL_FUNS(omap3630_serial_funs_interrupt,
                   omap3630_serial_putc_interrupt,
                   omap3630_serial_getc_interrupt,
                   omap3630_serial_set_config,
                   omap3630_serial_start_xmit,
                   omap3630_serial_stop_xmit
    );
#endif

#if (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE == 0) \
 || (defined(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3) && CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE == 0)
static SERIAL_FUNS(omap3630_serial_funs_polled,
                   omap3630_serial_putc_polled,
                   omap3630_serial_getc_polled,
                   omap3630_serial_set_config,
                   omap3630_serial_start_xmit,
                   omap3630_serial_stop_xmit
    );
#endif

//==========================================================================

#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0
static omap3630_serial_info omap3630_serial_info0 = {
    uart            : 0,
    base            : (CYG_ADDRWORD)OMAP3630_UART1_BASE,
    int_num         : CYGNUM_HAL_INTERRUPT_UART1_IRQ,
};

#if CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE > 0
static unsigned char omap3630_uart_write_reg_buf0[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE];
static unsigned char omap3630_serial_in_buf0[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BUFSIZE];

static SERIAL_CHANNEL_USING_INTERRUPTS(omap3630_serial_channel0,
                                       omap3630_serial_funs_interrupt,
                                       omap3630_serial_info0,
                                       CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BAUD),
                                       CYG_SERIAL_STOP_DEFAULT,
                                       CYG_SERIAL_PARITY_DEFAULT,
                                       CYG_SERIAL_WORD_LENGTH_DEFAULT,
                                       CYG_SERIAL_FLAGS_DEFAULT,
                                       &omap3630_uart_write_reg_buf0[0], sizeof(omap3630_uart_write_reg_buf0),
                                       &omap3630_serial_in_buf0[0], sizeof(omap3630_serial_in_buf0)
    );
#else
static SERIAL_CHANNEL(omap3630_serial_channel0,
                      omap3630_serial_funs_polled,
                      omap3630_serial_info0,
                      CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_BAUD),
                      CYG_SERIAL_STOP_DEFAULT,
                      CYG_SERIAL_PARITY_DEFAULT,
                      CYG_SERIAL_WORD_LENGTH_DEFAULT,
                      CYG_SERIAL_FLAGS_DEFAULT
    );
#endif

DEVTAB_ENTRY(omap3630_serial_io0,
             CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0_NAME,
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio,
             omap3630_serial_init,
             omap3630_serial_lookup,     // Serial driver may need initializing
             &omap3630_serial_channel0
    );
#endif //  CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL0

//==========================================================================

#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1
static omap3630_serial_info omap3630_serial_info1 = {
    uart            : 1,
    base            : (CYG_ADDRWORD)OMAP3630_UART2_BASE,
    int_num         : CYGNUM_HAL_INTERRUPT_UART2_IRQ,
};

#if CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE > 0
static unsigned char omap3630_uart_write_reg_buf1[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE];
static unsigned char omap3630_serial_in_buf1[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BUFSIZE];

static SERIAL_CHANNEL_USING_INTERRUPTS(omap3630_serial_channel1,
                                       omap3630_serial_funs_interrupt,
                                       omap3630_serial_info1,
                                       CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BAUD),
                                       CYG_SERIAL_STOP_DEFAULT,
                                       CYG_SERIAL_PARITY_DEFAULT,
                                       CYG_SERIAL_WORD_LENGTH_DEFAULT,
                                       CYG_SERIAL_FLAGS_DEFAULT,
                                       &omap3630_uart_write_reg_buf1[0], sizeof(omap3630_uart_write_reg_buf1),
                                       &omap3630_serial_in_buf1[0], sizeof(omap3630_serial_in_buf1)
    );
#else
static SERIAL_CHANNEL(omap3630_serial_channel1,
                      omap3630_serial_funs_polled,
                      omap3630_serial_info1,
                      CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_BAUD),
                      CYG_SERIAL_STOP_DEFAULT,
                      CYG_SERIAL_PARITY_DEFAULT,
                      CYG_SERIAL_WORD_LENGTH_DEFAULT,
                      CYG_SERIAL_FLAGS_DEFAULT
    );
#endif

DEVTAB_ENTRY(omap3630_serial_io1,
             CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1_NAME,
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio,
             omap3630_serial_init,
             omap3630_serial_lookup,     // Serial driver may need initializing
             &omap3630_serial_channel1
    );
#endif //  CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL1

//==========================================================================

#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2
static omap3630_serial_info omap3630_serial_info2 = {
    uart            : 2,
    base            : (CYG_ADDRWORD)OMAP3630_UART3_BASE,
    int_num         : CYGNUM_HAL_INTERRUPT_UART3_IRQ,
};

#if CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE > 0
static unsigned char omap3630_uart_write_reg_buf2[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE];
static unsigned char omap3630_serial_in_buf2[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BUFSIZE];

static SERIAL_CHANNEL_USING_INTERRUPTS(omap3630_serial_channel2,
                                       omap3630_serial_funs_interrupt,
                                       omap3630_serial_info2,
                                       CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BAUD),
                                       CYG_SERIAL_STOP_DEFAULT,
                                       CYG_SERIAL_PARITY_DEFAULT,
                                       CYG_SERIAL_WORD_LENGTH_DEFAULT,
                                       CYG_SERIAL_FLAGS_DEFAULT,
                                       &omap3630_uart_write_reg_buf2[0], sizeof(omap3630_uart_write_reg_buf2),
                                       &omap3630_serial_in_buf2[0], sizeof(omap3630_serial_in_buf2)
    );
#else
static SERIAL_CHANNEL(omap3630_serial_channel2,
                      omap3630_serial_funs_polled,
                      omap3630_serial_info2,
                      CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_BAUD),
                      CYG_SERIAL_STOP_DEFAULT,
                      CYG_SERIAL_PARITY_DEFAULT,
                      CYG_SERIAL_WORD_LENGTH_DEFAULT,
                      CYG_SERIAL_FLAGS_DEFAULT
    );
#endif

DEVTAB_ENTRY(omap3630_serial_io2,
             CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2_NAME,
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio,
             omap3630_serial_init,
             omap3630_serial_lookup,     // Serial driver may need initializing
             &omap3630_serial_channel2
    );
#endif //  CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL2

#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3
static omap3630_serial_info omap3630_serial_info3 = {
    uart            : 3,
    base            : (CYG_ADDRWORD)OMAP3630_UART4_BASE,
    int_num         : CYGNUM_HAL_INTERRUPT_UART4_IRQ,
};

#if CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE > 0
static unsigned char omap3630_uart_write_reg_buf3[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE];
static unsigned char omap3630_serial_in_buf3[CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BUFSIZE];

static SERIAL_CHANNEL_USING_INTERRUPTS(omap3630_serial_channel3,
                                       omap3630_serial_funs_interrupt,
                                       omap3630_serial_info3,
                                       CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BAUD),
                                       CYG_SERIAL_STOP_DEFAULT,
                                       CYG_SERIAL_PARITY_DEFAULT,
                                       CYG_SERIAL_WORD_LENGTH_DEFAULT,
                                       CYG_SERIAL_FLAGS_DEFAULT,
                                       &omap3630_uart_write_reg_buf3[0], sizeof(omap3630_uart_write_reg_buf3),
                                       &omap3630_serial_in_buf3[0], sizeof(omap3630_serial_in_buf3)
    );
#else
static SERIAL_CHANNEL(omap3630_serial_channel3,
                      omap3630_serial_funs_polled,
                      omap3630_serial_info3,
                      CYG_SERIAL_BAUD_RATE(CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_BAUD),
                      CYG_SERIAL_STOP_DEFAULT,
                      CYG_SERIAL_PARITY_DEFAULT,
                      CYG_SERIAL_WORD_LENGTH_DEFAULT,
                      CYG_SERIAL_FLAGS_DEFAULT
    );
#endif

DEVTAB_ENTRY(omap3630_serial_io3,
             CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3_NAME,
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio,
             omap3630_serial_init,
             omap3630_serial_lookup,     // Serial driver may need initializing
             &omap3630_serial_channel3
    );
#endif //  CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_SERIAL3

/*
 * Work Around for Errata i202 (3430 - 1.12, 3630 - 1.6)
 * The access to uart register after MDR1 Access
 * causes UART to corrupt data.
 *
 * Need a delay =
 * 5 L4 clock cycles + 5 UART functional clock cycle (@48MHz = ~0.2uS)
 * give 5 times as much
 *
 * uart_no : Should be a Zero Based Index Value always.
 */
static void omap_uart_mdr1_errataset(serial_channel* chan, CYG_BYTE mdr1_val,
                CYG_BYTE fcr_val)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *)chan->dev_priv;

    /* 10 retries, in this the FiFO's should get cleared */
    CYG_BYTE timeout = 0x05;

    uart_write_reg(omap3630_chan->base, OMAP3630_UART_MDR1, mdr1_val);
    hal_delay_us(1);
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_FCR, fcr_val | UART_FCR_CLEAR_XMIT |
	    UART_FCR_CLEAR_RCVR);
    /*
     * Wait for FIFO to empty: when empty, RX_FIFO_E bit is 0 and
     * TX_FIFO_E bit is 1.
     */
    while (UART_LSR_THRE != (uart_read_reg(omap3630_chan->base, UART_LSR) &
		(UART_LSR_THRE | UART_LSR_DR))) {
	timeout--;
	if (!timeout) {
	    /* Should *never* happen. we warn and carry on */
	    uart_read_reg(omap3630_chan->base, UART_LSR);
	    break;
	}
	hal_delay_us(1);
    }
}

#ifdef CYGOPT_IO_SERIAL_FLOW_CONTROL_SOFTWARE
static void omap3630_serial_uart_configure_xonxoff(serial_channel* chan)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *)chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    cyg_serial_info_t config = chan->config;
    cyg_uint32 lcr, efr, new_efr;

    lcr = uart_read_reg(base, OMAP3630_UART_LCR);
    uart_write_reg(base, OMAP3630_UART_LCR, OMAP_UART_LCR_CONF_MDB);

    efr = uart_read_reg(base, UART_EFR);
    uart_write_reg(base, OMAP3630_UART_EFR, efr & ~UART_EFR_ECB);

    uart_write_reg(base, OMAP3630_UART_XON1_ADDR1, CYGDAT_IO_SERIAL_FLOW_CONTROL_XON_CHAR);
    uart_write_reg(base, OMAP3630_UART_XOFF1, CYGDAT_IO_SERIAL_FLOW_CONTROL_XOFF_CHAR);

    /* clear SW control mode bits */
    new_efr = efr;
    new_efr &= OMAP_UART_SW_CLR;

    /*
     * IXON Flag:
     * Enable XON/XOFF flow control on output.
     * Transmit XON1, XOFF1
     */
    if (config.flags & CYGNUM_SERIAL_FLOW_XONXOFF_TX)
	new_efr |= OMAP_UART_SW_TX;

    /*
     * IXOFF Flag:
     * Enable XON/XOFF flow control on input.
     * Receiver compares XON1, XOFF1.
     */
    if (config.flags & CYGNUM_SERIAL_FLOW_XONXOFF_RX)
	new_efr |= OMAP_UART_SW_RX;

    uart_write_reg(base, OMAP3630_UART_EFR, efr | UART_EFR_ECB);
    uart_write_reg(base, OMAP3630_UART_LCR, UART_LCR_DLAB);

    omap3630_chan->mcr = uart_read_reg(base, OMAP3630_UART_MCR);

    /*
     * IXANY Flag:
     * Enable any character to restart output.
     * Operation resumes after receiving any
     * character after recognition of the XOFF character
     */
#if 0
    if (termios->c_iflag & IXANY)
	omap3630_chan->mcr |= UART_MCR_XONANY;
#endif
    //Only for testing
    omap3630_chan->mcr |= UART_MCR_XONANY;

    uart_write_reg(base, OMAP3630_UART_MCR, omap3630_chan->mcr | UART_MCR_TCRTLR);
    uart_write_reg(base, OMAP3630_UART_LCR, OMAP_UART_LCR_CONF_MDB);
    uart_write_reg(base, OMAP3630_UART_TCR, 0x0F);
    /* Enable special char function UARTi.EFR_REG[5] and
     * load the new software flow control mode IXON or IXOFF
     * and restore the UARTi.EFR_REG[4] ENHANCED_EN value.
     */
    uart_write_reg(base, OMAP3630_UART_EFR, new_efr | UART_EFR_SCD);
    uart_write_reg(base, OMAP3630_UART_LCR, UART_LCR_DLAB);

    uart_write_reg(base, OMAP3630_UART_MCR, omap3630_chan->mcr & ~UART_MCR_TCRTLR);
    uart_write_reg(base, OMAP3630_UART_LCR, lcr);
}
#endif
//==========================================================================
// Internal function to actually configure the hardware to desired baud
// rate, etc.

static void
omap3630_serial_uart_setbaud(serial_channel* chan, const CYG_ADDRWORD base, cyg_uint32 baud, cyg_uint32 cval)
{
    cyg_uint32 efr, lcr, baud_divisor;
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *)chan->dev_priv;

    //Protocol, Baud rate and Interrupt settings
    //For explanation see omap3630_diag.c or the datasheet

    //1
    omap_uart_mdr1_errataset(chan, 0x7, omap3630_chan->fcr);
    //2
    lcr = uart_read_reg(base, OMAP3630_UART_LCR);
    uart_write_reg(base, OMAP3630_UART_LCR, 0x00BF);
    //3
    efr = uart_read_reg(base, OMAP3630_UART_EFR);
    uart_write_reg(base, OMAP3630_UART_EFR, (efr | 0x0010));
    //4
    uart_write_reg(base, OMAP3630_UART_LCR, 0x0000);
    //5
    uart_write_reg(base, OMAP3630_UART_IER, 0x0000);
    //6
    uart_write_reg(base, OMAP3630_UART_LCR, 0x00BF);
    //7 calculate and store baudrate
    baud_divisor = (OMAP3630_UART_CLOCK_RATE / 16) / baud;
    uart_write_reg(base, OMAP3630_UART_DLL, baud_divisor & 0xFF);
    uart_write_reg(base, OMAP3630_UART_DLH, baud_divisor >> 8);
    //8
    uart_write_reg(base, OMAP3630_UART_LCR, 0x0000);
    //9
    uart_write_reg(base, OMAP3630_UART_IER, omap3630_chan->ier);
    //10
    uart_write_reg(base, OMAP3630_UART_LCR, 0x00BF);
    //11
    uart_write_reg(base, OMAP3630_UART_EFR, efr);
    //12
    uart_write_reg(base, OMAP3630_UART_LCR, cval);
    //13
    //Uart 16x mode
    omap_uart_mdr1_errataset(chan, 0x0, omap3630_chan->fcr);
}

static bool
omap3630_serial_config_port(serial_channel *chan, cyg_serial_info_t *new_config, bool init)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *)chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    CYG_WORD cval;
    cyg_uint32 efr;

    cval = select_parity[new_config->parity];
    cval |= select_word_length[new_config->word_length-CYGNUM_SERIAL_WORD_LENGTH_5];
    cval |= select_stop_bits[new_config->stop];

    // Set up FIFO buffer
    omap3630_chan->buf_head = omap3630_chan->buf_tail = 0;

    // Set up baud rate
    omap3630_serial_uart_setbaud(chan, base, select_baud[new_config->baud], cval);

#ifdef CYGOPT_IO_SERIAL_FLOW_CONTROL_HW
    /* Hardware Flow Control Configuration */
    if (omap3630_chan->uart != 3) {

	uart_write_reg(omap3630_chan->base, OMAP3630_UART_LCR, UART_LCR_DLAB);

	omap3630_chan->mcr = uart_read_reg(omap3630_chan->base, OMAP3630_UART_MCR);
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_MCR, omap3630_chan->mcr | UART_MCR_TCRTLR);

	uart_write_reg(omap3630_chan->base, OMAP3630_UART_LCR, OMAP_UART_LCR_CONF_MDB);
	efr = uart_read_reg(omap3630_chan->base, OMAP3630_UART_EFR);
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_EFR, efr | UART_EFR_ECB);

        //Set Trigger halt transmission to max (60 bytes)
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_TCR, 0x0F);

        if ( new_config->flags & CYGNUM_SERIAL_FLOW_RTSCTS_TX ) {
             efr |= UART_EFR_CTS;
        }
        if ( new_config->flags & CYGNUM_SERIAL_FLOW_RTSCTS_RX ) {
            efr |= UART_EFR_RTS;
        }
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_EFR, efr); /* Enable AUTORTS and AUTOCTS */

	uart_write_reg(omap3630_chan->base, OMAP3630_UART_LCR, UART_LCR_DLAB);
#ifdef CYGPKG_IO_SERIAL_CORTEXA_OMAP3630_LOOPBACK
        omap3630_chan->mcr |= UART_MCR_LOOP;
#endif
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_MCR, omap3630_chan->mcr | UART_MCR_RTS);
	uart_write_reg(omap3630_chan->base, OMAP3630_UART_LCR, cval);
    }
#endif

#ifdef CYGOPT_IO_SERIAL_FLOW_CONTROL_SOFTWARE
    omap3630_serial_uart_configure_xonxoff(chan);
#endif

    if (new_config != &chan->config)
        chan->config = *new_config;

    return true;
}

//==========================================================================
// Function to initialize the device.  Called at bootstrap time.
/*      
 * These are the definitions for the Interrupt Enable Register
 */
static bool
omap3630_serial_init(struct cyg_devtab_entry *tab)
{
    serial_channel * const chan = (serial_channel *) tab->priv;
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    int res = true;

    //Enable clock for the UART
    if (omap3630_chan->uart == 0)
    {
        cyg_uint32 reg = cm_read_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN1);
        reg |= (1 << 13);
        cm_write_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN1, reg);

        reg = cm_read_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN3);
        reg |= (1 << 13);
        cm_write_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN3, reg);
    }
    else if (omap3630_chan->uart == 1)
    {
        cyg_uint32 reg = cm_read_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN1);
        reg |= (1 << 14);
        cm_write_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN1, reg);

        reg = cm_read_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN3);
        reg |= (1 << 14);
        cm_write_reg(OMAP3630_CM_CORE_BASE, OMAP3630_CM_CORE_FCLKEN3, reg);
    }
    else if (omap3630_chan->uart == 2)
    {
        cyg_uint32 reg = cm_read_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_FCLKEN);
        reg |= (1 << 11);
        cm_write_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_FCLKEN, reg);

        reg = cm_read_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_ICLKEN);
        reg |= (1 << 11);
        cm_write_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_ICLKEN, reg);
    }
    else if (omap3630_chan->uart == 3)
    {
        cyg_uint32 reg = cm_read_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_FCLKEN);
        reg |= (1 << 18);
        cm_write_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_FCLKEN, reg);

        reg = cm_read_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_ICLKEN);
        reg |= (1 << 18);
        cm_write_reg(OMAP3630_CM_PER_BASE, OMAP3630_CM_PER_ICLKEN, reg);
    }
    else
    {
        //Nothing
    }

    (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices

    if (chan->out_cbuf.len != 0) {
	cyg_drv_interrupt_create(omap3630_chan->int_num,
		CYG_IO_SERIAL_OMAP_INT_PRIORITY,
		(cyg_addrword_t)chan,
		omap3630_serial_ISR,
		omap3630_serial_DSR,
		&omap3630_chan->serial_interrupt_handle,
		&omap3630_chan->serial_interrupt);
	cyg_drv_interrupt_attach(omap3630_chan->serial_interrupt_handle);
	cyg_drv_interrupt_unmask(omap3630_chan->int_num);
    }

    omap3630_chan->fcr = UART_FCR_R_TRIG_01 | UART_FCR_T_TRIG_01 |
	UART_FCR_ENABLE_FIFO;

    /* FCR can be changed only when the
     * baud clock is not running
     * DLL_REG and DLH_REG set to 0.
     */
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_DLL, 0);
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_DLH, 0);
    
    /*
     * Clear the FIFO buffers and disable them.
     * (they will be reenabled in set_termios())
     */
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_FCR, UART_FCR_ENABLE_FIFO);
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_FCR, UART_FCR_ENABLE_FIFO |
	    UART_FCR_CLEAR_RCVR | UART_FCR_CLEAR_XMIT);

    /* FIFO ENABLE */
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_FCR, omap3630_chan->fcr);

    res = omap3630_serial_config_port(chan, &chan->config, true);

    /*
     * Clear the interrupt registers.
     */
    uart_read_reg(omap3630_chan->base, OMAP3630_UART_LSR);
    if (uart_read_reg(omap3630_chan->base, OMAP3630_UART_LSR) & UART_LSR_DR)
	uart_read_reg(omap3630_chan->base, OMAP3630_UART_RHR);
    uart_read_reg(omap3630_chan->base, OMAP3630_UART_IIR);
     
    // Clear status register 
    uart_read_reg(omap3630_chan->base, OMAP3630_UART_MSR);

    // Enable RX interrupts
    omap3630_chan->ier |= UART_IER_RLSI | UART_IER_RDI;
    uart_write_reg(omap3630_chan->base, OMAP3630_UART_IER, omap3630_chan->ier);

    return res;
}

//==========================================================================
// This routine is called when the device is "looked" up (i.e. attached)

static Cyg_ErrNo
omap3630_serial_lookup(struct cyg_devtab_entry **tab,
                  struct cyg_devtab_entry *sub_tab,
                  const char *name)
{
    serial_channel * const chan = (serial_channel *) (*tab)->priv;

    (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices
    return ENOERR;
}

//==========================================================================
// Send a character to the device output buffer.
// Return 'true' if character is sent to device

static bool
omap3630_serial_putc_interrupt(serial_channel *chan, unsigned char c)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    cyg_uint32 status;

    status = uart_read_reg(base, OMAP3630_UART_LSR);

    if (status & 0x0020)
    {
        uart_write_reg(base, OMAP3630_UART_THR, c);
        return true;
    }
    else
        return false;
}

//==========================================================================

static bool
omap3630_serial_putc_polled(serial_channel *chan, unsigned char c)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    cyg_uint32 status;

    do {// Wait for empty fifo
        status = uart_read_reg(base, OMAP3630_UART_LSR);
    } while (!(status & UART_LSR_THRE));

    uart_write_reg(base, OMAP3630_UART_THR, c);

    return true;
}

//==========================================================================
// Fetch a character from the device input buffer

static unsigned char
omap3630_serial_getc_interrupt(serial_channel *chan)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    CYG_WORD32 c;

    // Read data
    c = uart_read_reg(base, OMAP3630_UART_RHR);

    return (unsigned char)(c & 0xFF);
}

//==========================================================================

static unsigned char
omap3630_serial_getc_polled(serial_channel *chan)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    cyg_uint32 status;
    cyg_uint32 c;

    do {
        status = uart_read_reg(base, OMAP3630_UART_LSR);
    } while (!(status & UART_LSR_DR));

    c = uart_read_reg(base, OMAP3630_UART_RHR);

    return (unsigned char) (c & 0xFF);
}

//==========================================================================
// Set up the device characteristics; baud rate, etc.

static Cyg_ErrNo
omap3630_serial_set_config(serial_channel *chan, cyg_uint32 key,
                       const void *xbuf, cyg_uint32 *len)
{

    switch (key) {
    case CYG_IO_SET_CONFIG_SERIAL_INFO:
      {
        cyg_serial_info_t *config = (cyg_serial_info_t *)xbuf;
        if ( *len < sizeof(cyg_serial_info_t) ) {
            return -EINVAL;
        }
        *len = sizeof(cyg_serial_info_t);
        if ( true != omap3630_serial_config_port(chan, config, false) )
            return -EINVAL;
      }
      break;
#ifdef CYGOPT_IO_SERIAL_FLOW_CONTROL_HW

    case CYG_IO_SET_CONFIG_SERIAL_HW_RX_FLOW_THROTTLE:
      {
	  omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
	  bool throttle = *(int *)xbuf;
	  if ( *len != sizeof(int) ) {
	      return -EINVAL;
	  }
          
          if( throttle )
	  {
	      // force nRTS to high
	      uart_write_reg(omap3630_chan->base, 
		      OMAP3630_UART_MCR, 
		      omap3630_chan->mcr & ~UART_MCR_RTS );   
	  }
	  else
	  {
	      // force nRTS to low
	      uart_write_reg(omap3630_chan->base, 
		      OMAP3630_UART_MCR, 
		      omap3630_chan->mcr | UART_MCR_RTS );   
	  }
      }
      break;

    case CYG_IO_SET_CONFIG_SERIAL_HW_FLOW_CONFIG:
      {
        Cyg_ErrNo result = ENOERR;

        // If the client is asking for DSR/DTR, refuse to do it.
        if (0 != (chan->config.flags & (CYGNUM_SERIAL_FLOW_DSRDTR_RX | CYGNUM_SERIAL_FLOW_DSRDTR_TX)))
        {
            chan->config.flags &= ~(CYGNUM_SERIAL_FLOW_DSRDTR_RX | CYGNUM_SERIAL_FLOW_DSRDTR_TX);
            result = -ENOSUPP;
        }

        return result;
      }

#endif

    default:
        return -EINVAL;
    }
    return ENOERR;
}

//==========================================================================
// Enable the transmitter on the device

static void
omap3630_serial_start_xmit(serial_channel *chan)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;

    // enable TX interrupts
    omap3630_chan->ier |= UART_IER_THRI;
    uart_write_reg(base, OMAP3630_UART_IER, omap3630_chan->ier);
}

//==========================================================================
// Disable the transmitter on the device

static void
omap3630_serial_stop_xmit(serial_channel *chan)
{
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;

    if (omap3630_chan->ier & UART_IER_THRI) {
	omap3630_chan->ier &= ~UART_IER_THRI;
	uart_write_reg(base, OMAP3630_UART_IER, omap3630_chan->ier);
    }
}

//==========================================================================
// Serial I/O - low level interrupt handler (ISR)
//
// This ISR does rather more than other serial driver ISRs. Normally,
// the ISR just masks the interrupt vector and schedules the DSR,
// which then handles all IO. However, if the processor is running out
// of external RAM it is too slow to handle higher baud rates using
// that technique. Something that is exacerbated by the lack of FIFOs
// in the USART hardware.
//
// Instead, this ISR receives any incoming data into a circular
// buffer, essentially providing the FIFO lacking in the
// hardware. Transmission is still offloaded to the DSR. Only TX
// interrupts are masked while this is done to prevent an interrupt
// loop, and to avoid blocking RX interrupts.

static cyg_uint32
omap3630_serial_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
    serial_channel * const chan = (serial_channel *) data;
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
  
    cyg_drv_interrupt_mask(omap3630_chan->int_num);
    cyg_drv_interrupt_acknowledge(omap3630_chan->int_num);

    return CYG_ISR_CALL_DSR;  // Cause DSR to be run
}

//==========================================================================
// Serial I/O - high level interrupt handler (DSR)

static void
omap3630_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    serial_channel * const chan = (serial_channel *) data;
    omap3630_serial_info * const omap3630_chan = (omap3630_serial_info *) chan->dev_priv;
    const CYG_ADDRWORD base = omap3630_chan->base;
    cyg_uint32 iir, lsr, msr;
    unsigned char c = 0;

    iir = uart_read_reg(base, OMAP3630_UART_IIR);

    if (iir & UART_IIR_NO_INT)
	goto exit;

    lsr = uart_read_reg(base, OMAP3630_UART_LSR);
    if (iir & UART_IER_RLSI) 
    {
	if (lsr & UART_LSR_DR)
	    c = uart_read_reg(base, OMAP3630_UART_RHR);

	if (lsr & UART_LSR_BRK_ERROR_BITS) 
	{
	    // an error was detected
#ifdef CYGOPT_IO_SERIAL_SUPPORT_LINE_STATUS
	    cyg_serial_line_status_t stat;
	    stat.value = 1;
	    if (lsr & UART_LSR_OE) {
		stat.which = CYGNUM_SERIAL_STATUS_OVERRUNERR;
		(chan->callbacks->indicate_status)(chan, &stat );
	    } else if (lsr & UART_LSR_PE) {
		stat.which = CYGNUM_SERIAL_STATUS_PARITYERR;
		(chan->callbacks->indicate_status)(chan, &stat );
	    } else if (lsr & UART_LSR_FE) {
		stat.which = CYGNUM_SERIAL_STATUS_FRAMEERR;
		(chan->callbacks->indicate_status)(chan, &stat );
	    } else if (lsr & UART_LSR_BI) {
		stat.which = CYGNUM_SERIAL_STATUS_BREAK;
		(chan->callbacks->indicate_status)(chan, &stat );
	    }
#endif
	}
	else {
	    if(c)
		(chan->callbacks->rcv_char)(chan, c );
	}
    } 

    /*                                                             
     * Without auto-CTS, the
     * transmitter sends any data present in the transmit FIFO and a receiver overrun error
     * may result.
     */
    msr = uart_read_reg(base, OMAP3630_UART_MSR); 

    //nCTS input changed state. clear on a read
    if (msr & UART_MSR_DCTS)
    {
	if (omap3630_chan->hw_stopped) 
	{
	    omap3630_chan->hw_stopped = 0;
	    omap3630_serial_start_xmit(chan);
	} else {
	    omap3630_chan->hw_stopped = 1;
	    omap3630_serial_stop_xmit(chan);
	}
    } 

    if( !omap3630_chan->hw_stopped
	    && (iir & UART_IIR_THRI) 
	    && (lsr & UART_LSR_THRE) )
	(chan->callbacks->xmt_char)(chan);

exit:
    cyg_drv_interrupt_unmask(omap3630_chan->int_num);
}

//==========================================================================
#endif // CYGPKG_IO_SERIAL_CORTEXM_OMAP3630

// end of omap3630_serial.c
