//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================
/*
 * \file spi_parrot5.c
 *
 * SPI dev for P5
 * 
 * \copyright Parrot SA
 */

#include <pkgconf/io_spi_arm_parrot5.h>
#include <cyg/io/spi.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_io.h> // HAL_WRITE_UINT32
#include <cyg/hal/hal_diag.h> // hal_delay_us()
#include <cyg/hal/p5dev.h> // SELECT_PIN
#include <string.h> // bzero
#include <assert.h>
#include "spi_parrot5.h"


static cyg_spi_bus p5_spi_bus;

struct p5_spi_device {
    cyg_spi_bus * spi_bus;
    cyg_uint32 tx_count;
    cyg_uint32 rx_count;
    cyg_uint8* tx_data;
    cyg_uint8* rx_data;
    cyg_bool drop_cs;
    cyg_bool cs_asserted;
    cyg_uint32 speed;
} hal_p5_spi CYG_SPI_DEVICE_ON_BUS(0);

static cyg_sem_t sem_spi_interrupt;
static cyg_handle_t spi_int_handle;
static cyg_interrupt intr;
int spi_base;
int spi_int_nb = 0;

/********** function def **********/
static void p5_spi_transaction_begin(cyg_spi_device* device);
static void p5_spi_transaction_transfer(cyg_spi_device* device, cyg_bool polled, cyg_uint32 count,
            const cyg_uint8* tx_data, cyg_uint8* rx_data, cyg_bool drop_cs);
static void p5_spi_transaction_tick(cyg_spi_device* device, cyg_bool polled, cyg_uint32 count);
static void p5_spi_transaction_end(cyg_spi_device* device);
static int p5_spi_get_config(cyg_spi_device* device, cyg_uint32 key, void* buf, cyg_uint32* len);
static int p5_spi_set_config(cyg_spi_device* device, cyg_uint32 key, const void* buf, cyg_uint32* len);

static cyg_uint32 SPI_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void SPI_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);

/*****************************************************************************/

cyg_spi_device* spi_get_device(void)
{
        return (void *) &hal_p5_spi;
}

static cyg_uint32 SPI_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
    cyg_drv_interrupt_mask(SPI_INTERRUPT);
    cyg_drv_interrupt_acknowledge(SPI_INTERRUPT);
    return CYG_ISR_CALL_DSR;
}

static void receive_and_send_data(void)
{
    int reg;
    int i = 0;
    
    // receive data
    reg = spi_read_reg(SPI_STATUS);
    while ((reg & SPI_STATUS_RXEMPTY) == 0 && hal_p5_spi.rx_count != 0 && i++ < 0x10) {
        reg = spi_read_reg(SPI_DATA);
        reg &= 0xFF;
        if (hal_p5_spi.rx_data != NULL)
            *(hal_p5_spi.rx_data++) = reg;
        hal_p5_spi.rx_count--;
        reg = spi_read_reg(SPI_STATUS);
    }

    // send data
    reg = spi_read_reg(SPI_STATUS);
    i = 0;
    while ((reg & SPI_STATUS_TXFULL) == 0 && \
                    (reg & SPI_STATUS_RXFULL) == 0 && hal_p5_spi.tx_count != 0 && i++ < 0x10) {
        int add; // end of frame or not

        if (hal_p5_spi.tx_count == 1 && hal_p5_spi.drop_cs) {
            add = 0x100;
            while(!(spi_read_reg(SPI_STATUS) & SPI_STATUS_TXEMPTY));
            hal_delay_us(1);
            hal_p5_spi.cs_asserted = 0;
        } else {
            add = 0x0;
        }
        if (hal_p5_spi.tx_data != NULL) {
            spi_write_reg(SPI_DATA, *(hal_p5_spi.tx_data++) | add);
        }
        else {
            spi_write_reg(SPI_DATA, 0 | add);
        }
        hal_p5_spi.tx_count--;
        reg = spi_read_reg(SPI_STATUS);
    }
}

static void SPI_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    spi_int_nb++;
    receive_and_send_data();

    if (hal_p5_spi.tx_count == 0 && hal_p5_spi.rx_count == 0) {
        // all interrupts done
        cyg_drv_interrupt_mask(SPI_INTERRUPT);
        cyg_semaphore_post(&sem_spi_interrupt);
    } else {
        cyg_drv_interrupt_unmask(SPI_INTERRUPT);
    }
}

void spi_init(void)
{
#ifdef CYGOPT_IO_SPI_ARM_P5DEV_OUTPUT_EXTERNAL
    SELECT_PIN(SPI_CLK);
    SELECT_PIN(SPI_SS);
    SELECT_PIN(SPI_MOSI);
    SELECT_PIN(SPI_MISO);
#else // FLASH SPI
    SELECT_PIN(SPI_CLK_);
    SELECT_PIN(SPI_SS_);
    SELECT_PIN(SPI_MOSI_);
    SELECT_PIN(SPI_MISO_);
#endif
    
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_CEN, SYS_CEN_SPICLK);

    cyg_semaphore_init(&sem_spi_interrupt, 0);

    // init hal_p5_spi
    bzero((char*)&hal_p5_spi, sizeof(hal_p5_spi));
    
    hal_p5_spi.spi_bus = &p5_spi_bus;
    CYG_SPI_BUS_COMMON_INIT(hal_p5_spi.spi_bus); // init the internal mutex
    p5_spi_bus.spi_transaction_begin = p5_spi_transaction_begin;
    p5_spi_bus.spi_transaction_transfer = p5_spi_transaction_transfer;
    p5_spi_bus.spi_transaction_tick = p5_spi_transaction_tick;
    p5_spi_bus.spi_transaction_end = p5_spi_transaction_end;
    p5_spi_bus.spi_get_config = p5_spi_get_config;
    p5_spi_bus.spi_set_config = p5_spi_set_config;

    // init P5 SPI
    spi_sel(_MASTER);
    hal_p5_spi.speed = 100000;
    spi_program(SPI_CTRL_MSTR,0,SPI_REG_SPEED_VALUE(hal_p5_spi.speed),0x2,0x2);
    
    // interrupt init
    cyg_drv_interrupt_create(
        SPI_INTERRUPT,
        4,
        (cyg_addrword_t) 0,
        (cyg_ISR_t*) &SPI_ISR,
        (cyg_DSR_t*) &SPI_DSR,
        &spi_int_handle,
        &intr);
    cyg_drv_interrupt_attach(spi_int_handle);

    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_CDIS, SYS_CEN_SPICLK);
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_CEN, SYS_CEN_SPICLK);
}

static void p5_spi_transaction_begin(cyg_spi_device* device)
{
}

static void p5_spi_transaction_transfer(cyg_spi_device* device, cyg_bool polled, cyg_uint32 count,
                const cyg_uint8* tx_data, cyg_uint8* rx_data, cyg_bool drop_cs)
{
    struct p5_spi_device * p5_device = (struct p5_spi_device*) device;
    cyg_spi_bus * bus = device->spi_bus;
    int add; // end of frame or not
    int reg;
    
    p5_device->tx_count = count;
    p5_device->rx_count = count;
    p5_device->tx_data = (cyg_uint8 *)tx_data;
    p5_device->rx_data = rx_data;
    p5_device->drop_cs = drop_cs;

    if (count != 0 && p5_device->cs_asserted == 0) {
        p5_device->cs_asserted = 1;
    }
    
    if (polled) {
        cyg_drv_interrupt_mask(SPI_INTERRUPT);
        while (p5_device->tx_count != 0 || p5_device->rx_count != 0)
            receive_and_send_data();
    } else {
        cyg_drv_interrupt_unmask(SPI_INTERRUPT);
        
        if (count == 0)
                (*(bus->spi_transaction_end))(device);

        // send one data to start receiving interrupts
        p5_device->tx_count--;
        
        if (count == 1 && drop_cs) {
            add = 0x100;
            p5_device->cs_asserted = 0;
        } else {
            add = 0;
        }
        reg = spi_read_reg(SPI_STATUS);
        
        if (p5_device->tx_data == NULL) {
            spi_write_reg(SPI_DATA, add);
        } else {
            spi_write_reg(SPI_DATA, *(hal_p5_spi.tx_data++)|add);
        }
        // wait for all interrupts
        cyg_semaphore_wait(&sem_spi_interrupt);
        
    }
}

static void p5_spi_transaction_tick(cyg_spi_device* device, cyg_bool polled, cyg_uint32 count)
{
}

static void p5_spi_transaction_end(cyg_spi_device* device)
{
  struct p5_spi_device * p5_device = (struct p5_spi_device*) device;
  //cyg_spi_bus * bus = device->spi_bus;

    // On P5 we can't release CS without sending one data
    // use cyg_spi_transaction_transfer(..., true) on your last transfer
    assert(p5_device->cs_asserted == 0);
}

int p5_spi_get_config(cyg_spi_device* device, cyg_uint32 key, void* buf, cyg_uint32* len)
{
    struct p5_spi_device * p5_device = (struct p5_spi_device*) device;
    
    switch (key) {
        case CYG_IO_GET_CONFIG_SPI_CLOCKRATE:
            if (*len < sizeof(int)) {
                return -EINVAL;
            }
            *len = sizeof(int);
            *(int*) buf = p5_device->speed;
            break;
        default:
            return -EINVAL;
    }
    return ENOERR;
}

int p5_spi_set_config(cyg_spi_device* device, cyg_uint32 key, const void* buf, cyg_uint32* len)
{
    struct p5_spi_device * p5_device = (struct p5_spi_device*) device;
    
    switch (key) {
        cyg_uint32 div;

        case CYG_IO_SET_CONFIG_SPI_CLOCKRATE:
            if (*len < sizeof(int)) {
                return -EINVAL;
            }
            *len = sizeof(int);
            p5_device->speed = *(int*) buf;

            for (div = 1; div <= SPI_SPEED_DIVMAX; div++) {
                cyg_uint32 spi_clk_hz = SPI_GET_FREQ_BY_DIV(div);
                if (spi_clk_hz <= p5_device->speed) {
                    p5_device->speed = spi_clk_hz;
                    break;
                }
            }
            if (div > SPI_SPEED_DIVMAX) {
                assert(0); // unreacheable, the requested frequency is too slow
                p5_device->speed = SPI_GET_FREQ_BY_DIV(SPI_SPEED_DIVMAX);
                div = SPI_SPEED_DIVMAX;
            }
            spi_program(SPI_CTRL_MSTR , 0, div, 0x2, 0x2);
            break;
        default:
            return -EINVAL;
    }
    return ENOERR;
}
