#ifndef SPI_PARROT5_H
#define SPI_PARROT5_H

#define PARROT5_SPI_MASTER          0xD00A0000
#define PARROT5_SPI_SLAVE           0xD00A0000
#define SPI_INTERRUPT 13

////////////////////////////////////////////////////
// Parrot proprietary SPI registers
////////////////////////////////////////////////////

#define SPI_CTRL        0x000       // SPI Control Register
#define SPI_SPEED       0x004       // SPI Speed Register
#define SPI_STATUS      0x008       // SPI Status Register 
#define SPI_DATA        0x040       // SPI Data registers first address (0x40..0x7F)

// Registers bitwise definitions

// Control register
#define SPI_CTRL_STREAM         (1 << 8)    // Interrupt occurence (half fifo or empty)
#define SPI_CTRL_MSTR           (1 << 9)    // Master / Slave selection

// Status register
#define SPI_STATUS_TXFULL       1           // Tx FIFO is full
#define SPI_STATUS_TXEMPTY      (1 << 1)    // Tx FIFO is empty
#define SPI_STATUS_RXFULL       (1 << 2)    // Rx FIFO is full
#define SPI_STATUS_RXEMPTY      (1 << 3)    // Rx FIFO is empty

// Data Buffer register
#define SPI_DATA_TXLASTBYTE     (1 << 8)    // Tx Last Byte
#define SPI_DATA_RXFLAG         (1 << 8)    // Rx Flag
#define SPI_DATA_RXEMPTY        (1 << 9)    // RxFifo Empty flag

// Speed register
// We assume that the ARM processor is running at 208MHz
#define SPI_CLOCK              104000000
#define SPI_SPEED_26M          SPI_CLOCK / (2*26000000) - 1
#define SPI_SPEED_10M          SPI_CLOCK / (2*10000000) - 1
#define SPI_SPEED_5M           SPI_CLOCK / (2*5000000) - 1
#define SPI_SPEED_1M           SPI_CLOCK / (2*1000000) - 1       
#define SPI_SPEED_100K         SPI_CLOCK / (2*100000) - 1       
#define SPI_SPEED_DIVMAX       1023
#define SPI_REG_SPEED_VALUE(speed) (SPI_CLOCK / (2*(speed)) - 1)
#define SPI_GET_FREQ_BY_DIV(div)   (SPI_CLOCK / ((div+1)*2))

#define spi_sel(type) \
	spi_base = PARROT5_SPI ## type;

//! write to an SPI register
#define spi_write_reg( _reg_, _value_ ) \
	(*((volatile cyg_uint32 *)(spi_base+(_reg_))) = (cyg_uint32)(_value_))

//! read an SPI register
#define spi_read_reg(_reg_ ) \
	(*((volatile cyg_uint32 *)(spi_base+(_reg_))))

#define spi_program(_master_,_stream_,_speed_,_setup_,_hold_) \
	spi_write_reg(SPI_SPEED, _speed_ ); \
	spi_write_reg(SPI_CTRL, _master_|_stream_|(_setup_&0xf)|((_hold_<<4)&0xf0));

#endif
