//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================
/*
 * \file spi_parrot6i.c
 *
 * SPI dev for P6i
 * 
 * \copyright Parrot SA
 */

#include <pkgconf/io_spi_arm_parrot6i.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_io.h>
#include <assert.h>
#include <cyg/io/spi_parrot6i.h>

#define FIFO_SIZE        64

#define SPI_CTRL        0x000
#define SPI_SPEED       0x004
#define SPI_STATUS      0x008
#define SPI_DATA        0x040

#define SPI_CTRL_STREAM         (1<<8)
#define SPI_CTRL_MSTR           (1<<9)
#define SPI_CTRL_TSETUPCS(t)    ((t) & 0xF)
#define SPI_CTRL_THOLDCS(t)     (((t) & 0xF) << 4)
#define SPI_CTRL_CPOL           (1<<10)
#define SPI_CTRL_CPHA           (1<<11)
#define SPI_CTRL_LSB            (1<<12)

#define SPI_STATUS_TXFULL       (1<<0)
#define SPI_STATUS_TXEMPTY      (1<<1)
#define SPI_STATUS_RXFULL       (1<<2)
#define SPI_STATUS_RXEMPTY      (1<<3)

#define SPI_DATA_TXLASTBYTE     (1<<8)
#define SPI_DATA_RXFLAG         (1<<8)
#define SPI_DATA_RXEMPTY        (1<<9)

#define spi_write_bus_reg(_bus, _reg, _value) \
	(*((volatile cyg_uint32 *)((_bus)->base + (_reg))) = (cyg_uint32)(_value))

#define spi_read_bus_reg(_bus, _reg) \
	(*((volatile cyg_uint32 *)((_bus)->base + (_reg))))

#define min(a, b)                   \
    ({                              \
        typeof(a) const   _a = (a); \
        typeof(b) const   _b = (b); \
        _a < _b ? _a : _b;          \
     })

#define max(a, b)                   \
    ({                              \
        typeof(a) const   _a = (a); \
        typeof(b) const   _b = (b); \
        _a > _b ? _a : _b;          \
     })

void
p6i_set_spi_speed(struct p6i_spi_bus const* bus,
                  unsigned int speed)
{
    speed = pclock_get_rate(bus->clock) /
            min(speed, 39000000); /* Static Timing Analysis OK up to 39MHz. */
    speed = (max(speed >> 1, 1) - 1) & 0x3ff;
    spi_write_bus_reg(bus, SPI_SPEED, speed);
}

static
void
p6i_xfer_spi_bus(struct p6i_spi_bus const* bus,
                 cyg_uint8 const* tx_buff,
                 cyg_uint8* rx_buff,
                 size_t count,
                 cyg_bool drop)
{
    int b, byte;

    /* Wait till FIFO is empty. */
    while (! (spi_read_bus_reg(bus, SPI_STATUS) & SPI_STATUS_TXEMPTY))
        ;

    while (count) {
        size_t const cnt = min(count, (size_t) FIFO_SIZE);

        /* Send bytes first. */
        if (tx_buff) {
            int const last = (drop && count <= FIFO_SIZE) ?
                                 tx_buff[cnt - 1] | SPI_DATA_TXLASTBYTE :
                                 tx_buff[cnt - 1];

            for (b = 0; b < cnt - 1; b++)
                spi_write_bus_reg(bus, SPI_DATA, tx_buff[b]);

            spi_write_bus_reg(bus, SPI_DATA, last);

            tx_buff += cnt;
        }
        else {
            int const last = (drop && count <= FIFO_SIZE) ?
                                SPI_DATA_TXLASTBYTE : 0;

            for (b = 0; b < cnt - 1; b++)
                spi_write_bus_reg(bus, SPI_DATA, 0);

            spi_write_bus_reg(bus, SPI_DATA, last);
        }

        /* Then, receive bytes. */
        if (rx_buff) {
            for (b = 0; b < cnt; b++) {
                do {
                    byte = spi_read_bus_reg(bus, SPI_DATA);
                } while (byte & SPI_DATA_RXEMPTY);

                rx_buff[b] = byte & 0xff;
            }

            rx_buff += cnt;
        }
        else {
            for (b = 0; b < cnt; b++) {
                do {
                    byte = spi_read_bus_reg(bus, SPI_DATA);
                } while (byte & SPI_DATA_RXEMPTY);
            }
        }

        count -= cnt;
    }
}

void
p6i_xfer_spi_dev(struct p6i_spi_dev* device,
                 cyg_bool polled,
                 cyg_uint32 count,
                 cyg_uint8 const* tx_data,
                 cyg_uint8* rx_data,
                 cyg_bool drop_cs)
{
    CYG_ASSERT(polled,
               "Interrupt driven operations are not yet supported !!");
#if 0
    CYG_ASSERT(device->cs, "SPI device not selected");
#endif

    p6i_xfer_spi_bus((struct p6i_spi_bus*) p6i_spi_dev_bus(device),
                     tx_data,
                     rx_data,
                     (size_t) count,
                     drop_cs);
    device->cs = ! drop_cs;
}

void
p6i_end_spi(struct p6i_spi_dev* device)
{
    if (device->cs) {
         p6i_xfer_spi_dev(device, 1, 1, 0, 0, 1);
         device->cs = 0;
    }
}

static
void
p6i_init_spi_device(struct p6i_spi_dev* device)
{
    if (! device->init) {
        unsigned int                    ctrl;
        struct p6i_spi_bus const* const bus = (struct p6i_spi_bus*)
                                              p6i_spi_dev_bus(device);

        /* Setup control register according to device properties. */
        ctrl = SPI_CTRL_TSETUPCS(device->tsetupcs) |
               SPI_CTRL_THOLDCS(device->tholdcs) |
               SPI_CTRL_MSTR |
               (device->cpol ? SPI_CTRL_CPOL : 0) |
               (device->cpha ? SPI_CTRL_CPHA : 0) |
               (device->lsb ? SPI_CTRL_LSB : 0);
        spi_write_bus_reg(bus, SPI_CTRL, ctrl);

        /* Setup bus speed. */
        p6i_set_spi_speed(bus, device->hz);

        device->cs = 0;

        /* This is a one time configuration. */
        device->init = 1;
    }
}

void
p6i_begin_spi(struct p6i_spi_dev* device)
{
    p6i_init_spi_device(device);

    CYG_ASSERT(! device->cs, "SPI device already initialized");
    device->cs = 1;
}

static
void
p6i_init_spi_bus(struct p6i_spi_bus* bus)
{
    unsigned int speed;

    /* Enable clock to get proper rate (it would return 0 otherwise). */
    pclock_enable(bus->clock);

    /* Set master mode */
    spi_write_bus_reg(bus, SPI_CTRL, SPI_CTRL_MSTR);

    /* Compute failsafe initial clock speed (100KHz). */
    speed = pclock_get_rate(bus->clock) / 100000;
    speed = ((speed >> 1) - 1) & 0x3ff;

    /* Set speed */
    spi_write_bus_reg(bus, SPI_SPEED, speed);

    /* 
     * Work around: disable clock to reset the input flip flop (in case of false SPI_CLK_IN
     * detection) and reenable it to allow further configuration.
     */
    pclock_disable(bus->clock);
    pclock_enable(bus->clock);

    /* Set failsafe timings and master mode. */
    spi_write_bus_reg(bus,
                      SPI_CTRL,
                      SPI_CTRL_MSTR |
                      SPI_CTRL_TSETUPCS(10) |
                      SPI_CTRL_THOLDCS(10));
}

#ifdef CYGPKG_IO_SPI
static
int
cyg_set_p6i_spi_conf(cyg_spi_device* device,
                    cyg_uint32 key,
                    void const* buf,
                    cyg_uint32* len)
{
    switch (key) {
        case CYG_IO_SET_CONFIG_SPI_CLOCKRATE:
            if (*len >= sizeof(int)) {
                unsigned int speed = (unsigned int) (*(int*) buf);

                p6i_set_spi_speed((struct p6i_spi_bus*) device->spi_bus, speed);
                *len = sizeof(int);
                ((struct p6i_spi_dev*) device)->hz = speed;
                break;
            }
            else
                return -EINVAL;

        default:
            return -EINVAL;
    }

    return ENOERR;
}

static
int
cyg_get_p6i_spi_conf(cyg_spi_device* device,
                     cyg_uint32 key,
                     void* buf,
                     cyg_uint32* len)
{
    switch (key) {
        case CYG_IO_GET_CONFIG_SPI_CLOCKRATE:
            if (*len < sizeof(int)) {
                return -EINVAL;
            }

            *len = sizeof(int);
            *(int*) buf = ((struct p6i_spi_dev*) device)->hz;
            break;

        default:
            return -EINVAL;
    }

    return ENOERR;
}

static
void
cyg_tick_p6i_spi(cyg_spi_device* device, cyg_bool polled, cyg_uint32 count)
{
}

static
void
cyg_xfer_p6i_spi(cyg_spi_device* device,
                 cyg_bool polled,
                 cyg_uint32 count,
                 cyg_uint8 const* tx_data,
                 cyg_uint8* rx_data,
                 cyg_bool drop_cs)
{
    p6i_xfer_spi_dev((struct p6i_spi_dev*) device,
                     polled,
                     (size_t) count,
                     tx_data,
                     rx_data,
                     drop_cs);
}

static
void
cyg_end_p6i_spi(cyg_spi_device* device)
{
    p6i_end_spi((struct p6i_spi_dev*) device);
}

static
void
cyg_begin_p6i_spi(struct cyg_spi_device* device)
{
    p6i_begin_spi((struct p6i_spi_dev*) device);
}

static
void
cyg_init_p6i_spi(struct cyg_spi_bus* bus)
{
    bus->spi_transaction_begin = &cyg_begin_p6i_spi;
    bus->spi_transaction_transfer = &cyg_xfer_p6i_spi;
    bus->spi_transaction_tick = &cyg_tick_p6i_spi;
    bus->spi_transaction_end = &cyg_end_p6i_spi;
    bus->spi_get_config = &cyg_get_p6i_spi_conf;
    bus->spi_set_config = &cyg_set_p6i_spi_conf;

    return p6i_init_spi_bus((struct p6i_spi_bus*) bus);
}
#endif

extern struct p6i_spi_bus __P6I_SPIBUS_TAB__[], __P6I_SPIBUS_TAB_END__;

void
p6i_init_spi_driver(void)
{
    struct p6i_spi_bus*    bus;

    for (bus = &__P6I_SPIBUS_TAB__[0]; bus != &__P6I_SPIBUS_TAB_END__; bus++) {
#ifdef CYGPKG_IO_SPI
        CYG_SPI_BUS_COMMON_INIT((struct cyg_spi_bus*) bus);
        cyg_init_p6i_spi((struct cyg_spi_bus*) bus);
#else
        p6i_init_spi_bus(bus);
#endif
    }
}
