/* this is a simple hello world program */
#include <stdio.h>

#include <cyg/hal/hal_intr.h> // Interrupt names
#include <cyg/hal/hal_io.h>
#include <cyg/kernel/kapi.h>  // thread
#include <cyg/kernel/diag.h>
#include <cyg/hal/hal_arch.h>           /* CYGNUM_HAL_STACK_SIZE_TYPICAL */

#include <unistd.h>
#include <cyg/fileio/fileio.h>
#include <dirent.h>
#include <errno.h>
#include <cyg/hal/hal_diag.h> // for HAL_DELAY_US
#include <cyg/hal/hal_io.h>     // for HAL_WRITE_UINT32

#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include "vfatfs_test.h"


#define ENTRY_FREE   0
#define ENTRY_USED   1

typedef struct{
   char  used;
   char  name[MAX_PATH_CHAR];
   int   size;
   int   crc;
   }file_tab_t;

typedef struct{
   int         nb_files;
   file_tab_t  ftab[MAX_FILE_NB];
}file_db_t;

file_db_t   file_db;

/**************************************************** 
 * get_random_entry
 * --------------------------------------------------
 *
 ****************************************************/
int tf_get_random_index(void)
{
   int i_ent;
   do{
      i_ent = rand() % MAX_FILE_NB;
   }while( tf_is_full(i_ent) == FALSE );
//   return 4;
   return i_ent;
}

/**************************************************** 
 * get_random_entry
 * --------------------------------------------------
 *
 ****************************************************/
error_type_t tf_get_entry(int i_ent, char * fname, int *fsize, int *fcrc)
{
   error_type_t err = E_CR_OK;

   if(file_db.ftab[i_ent].used != ENTRY_USED)
      err = E_CR_FAILED;
   else
   {
      memcpy(fname, &(file_db.ftab[i_ent].name[0]), strlen(file_db.ftab[i_ent].name)+1 );
      *fsize = file_db.ftab[i_ent].size;
      *fcrc  = file_db.ftab[i_ent].crc;
   }

   return err;
}


/**************************************************** 
 * find_free_entry
 * --------------------------------------------------
 *
 ****************************************************/
int tf_is_full(int i_ent)
{
   if(file_db.ftab[i_ent].used == ENTRY_USED)
      return 1; 
   else
      return 0;
}


/**************************************************** 
 * find_free_entry
 * --------------------------------------------------
 *
 ****************************************************/
static int find_free_entry(void)
{
   int i_ent;
   for(i_ent=0; i_ent<MAX_FILE_NB; i_ent++)
   {
      if(file_db.ftab[i_ent].used == ENTRY_FREE)
         return i_ent; 
   }
   return MAX_FILE_NB+1;
}


/**************************************************** 
 * tf_init
 * --------------------------------------------------
 *
 ****************************************************/
void  tf_init(void)
{
   memset(&file_db, 0, sizeof(file_db_t));
}

/**************************************************** 
 * tf_addfile
 * --------------------------------------------------
 *
 ****************************************************/
static error_type_t  tf_chkfile_exist(char* fname, int *n);

error_type_t  tf_addfile(char* fname, int fsize, int fcrc)
{
   error_type_t err = E_CR_OK;
   int free_entry;
   free_entry = find_free_entry();

   if (free_entry > MAX_FILE_NB)
   {
      // il faut liberer un entree
      diag_printf("[I] No more free entries\n");
      err = E_CR_NO_FREE_ENTRY;
   }
   else
   {
      err = tf_chkfile_exist(fname, &free_entry);
      // if entry doesn't exist 
      if( err == E_CR_NO_ENTRY  )
      {
         file_db.ftab[free_entry].used = ENTRY_USED;
         memcpy(&(file_db.ftab[free_entry].name[0]), fname, strlen(fname)+1 );
         file_db.ftab[free_entry].size = fsize;
         file_db.ftab[free_entry].crc = fcrc;
         err = E_CR_OK;
      }
      else if( err == E_CR_OK  )
      {
         file_db.ftab[free_entry].size = fsize;
         file_db.ftab[free_entry].crc = fcrc;
         err = E_CR_OK;
      }
   }

   return err;
}

/**************************************************** 
 * tf_delfile
 * --------------------------------------------------
 *
 ****************************************************/
void  tf_delfile(int i_ent)
{
      file_db.ftab[i_ent].used = ENTRY_FREE;
      memset(&(file_db.ftab[i_ent].name[0]), 0, MAX_PATH_CHAR);
      file_db.ftab[i_ent].crc = 0;
}

/**************************************************** 
 * tf_chkfile
 * --------------------------------------------------
 *
 ****************************************************/
error_type_t  tf_chkfile(char* fname, int fsize)
{
   int i_ent = 0;
   error_type_t err;
   err = E_CR_WRONG_FILE; 

   do
   {
      if( (file_db.ftab[i_ent].used == 0x1)
        &&( 0 == strcmp(fname, &(file_db.ftab[i_ent].name[0]) ) ) )
      {
         if(file_db.ftab[i_ent].size != fsize)
         {
            err = E_CR_WRONG_SIZE;
            diag_printf("[I] file %s found SIZE-wrong\n");
         }
         else
         {
            err = E_CR_OK;
         }
      }

      i_ent++;

   }while( (i_ent < MAX_FILE_NB) && (err !=E_CR_OK) );

   if(err != E_CR_OK)
                  diag_printf("[I] file not found %s\n");

   return err;
}


/**************************************************** 
 * tf_chkfile_exist
 * --------------------------------------------------
 *
 ****************************************************/
error_type_t  tf_chkfile_exist(char* fname, int *n_ent)
{
   int i_ent = 0;
   error_type_t err = E_CR_NO_ENTRY; 

   do
   {
      if( (file_db.ftab[i_ent].used == 0x1)
        &&( 0 == strcmp(fname, &(file_db.ftab[i_ent].name[0]) ) ) )
      {
         err = E_CR_OK;
         diag_printf("[W] file already exists %s\n");
         *n_ent = i_ent;
         break;
      }
      i_ent++;

   }while( (i_ent < MAX_FILE_NB) && (err !=E_CR_OK) );

   return err;
}


/****************************************************
 * tf_ls_check
 * --------------------------------------------------
 * Check that all files are contained in the file db
 ****************************************************/
int tf_ls_check(char * path)
{
   DIR * dir;
   struct dirent * plop;
   struct stat sStat;
   int err;
   int n, i;
   int nb_file_loc = 0;
   char file_name[1024+1];
   int nb_err = 0;

   n = strlen(path);

   dir = opendir(path);

//   diag_printf("*------------------- %s ----------------*\n", path);

   if (dir == NULL) {
      diag_printf("=>rm(%s): error on open dir\n", path);
      return -1;
   }

   while((plop = readdir(dir)) != NULL) 
   {
      i = strlen(plop->d_name);

      if (n+i > (255+1)) 
      {
         closedir(dir);
         return -1;
      }

      if( 0 == strcmp(path, "/") ) // do not suppress root dir
         sprintf(file_name, "%s%s", path, plop->d_name);
      else
         sprintf(file_name, "%s/%s", path, plop->d_name);

      err = stat(file_name, &sStat); // FIXME ADD path

      if(S_ISDIR(sStat.st_mode)) 
      {
         if (plop->d_name[0] != '.')
         {
            tf_ls_check(file_name);
         }
      } 
      else 
      {
         nb_file_loc++;
         err = tf_chkfile(file_name, sStat.st_size);
         if(err != E_CR_OK)
            nb_err++;

      }
   }

   err = closedir(dir);

   if(  (0 != strcmp(path, "/")) // do not suppress root dir
      &&( nb_file_loc == 0)     )
   {
      err = stat(path, &sStat); // FIXME ADD path
      if(S_ISDIR(sStat.st_mode)) 
      {
         rm(path);
      }
   }

   if (err < 0) {
      diag_printf("=> rm : error closing dir\n");
      return -1;
   }
//   diag_printf("*------------------- End of %s ----------------*\n", path);

   return nb_err;
}



