/* this is a simple hello world program */
#include <stdio.h>

#include <cyg/hal/hal_intr.h> // Interrupt names
#include <cyg/hal/hal_io.h>
#include <cyg/kernel/kapi.h>  // thread
#include <cyg/kernel/diag.h>
#include <cyg/hal/hal_arch.h>           /* CYGNUM_HAL_STACK_SIZE_TYPICAL */

#include <unistd.h>
#include <cyg/fileio/fileio.h>
#include <dirent.h>
#include <errno.h>
#include <cyg/hal/hal_diag.h> // for HAL_DELAY_US
#include <cyg/hal/hal_io.h>     // for HAL_WRITE_UINT32

#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include "vfatfs_test.h"

#define NTHREADS 3
#define STACKSIZE ( CYGNUM_HAL_STACK_SIZE_TYPICAL + 4096 )

typedef enum{
   PRINT_BOTH = 0,
   PRINT_MO,
   PRINT_CLUSTER
} print_disk_usage_t;

/* STATICS */
static cyg_handle_t thread[NTHREADS];
static cyg_thread thread_obj[NTHREADS];
static char stack[NTHREADS][STACKSIZE];

/* to fread and fwrite */
static char file_buf[MAX_FILE_SIZE];

/*  a mutex to protect calls the file_db */
cyg_mutex_t mtx_file_db;


int max_file_nb;
int del_file_nb;
int disk_max_size;




/**************************************************** 
 * print_disk_usage
 * --------------------------------------------------
 * print disk free/total memory
 ****************************************************/
void print_disk_usage(print_disk_usage_t print)
{
   struct cyg_fs_disk_usage du;
   cyg_uint32 len;
   cyg_uint32 disk_free;
   cyg_uint32 disk_size;
   int time;
   int ret;

   /* Check Disk usage now that files have been erased */
   diag_printf("FAT DISK_USAGE ...(it can take few seconds)");
   len = sizeof(du);
   time = cyg_current_time( );
   ret = cyg_fs_getinfo("/", FS_INFO_DISK_USAGE, &du, len);
   time = cyg_current_time( ) - time ;
   diag_printf(" --> it took %d ms\n", time*10);

   if( (print == PRINT_MO)||(print == PRINT_BOTH) )
   {
      disk_size = (du.total_clusters * (du.cluster_size>>9) ) >> 11;
      disk_free = (du.free_clusters  * (du.cluster_size>>9) ) >> 11;
      diag_printf("               %d Mo / %d Mo \n"     , disk_free, disk_size );
   }

   if( (print == PRINT_CLUSTER)||(print == PRINT_BOTH) )
   {
      diag_printf("               %d clust / %d clust\n", du.free_clusters, du.total_clusters );
   }
}


/**************************************************** 
 * delete_files
 * --------------------------------------------------
 * delete files
 ****************************************************/
static char create_file_loc_name[MAX_PATH_CHAR];

error_type_t delete_files(int nb_files)
{
   error_type_t   err = E_CR_OK;
   int            i_delfile;
   int            i_ent;
   int            loc_size;
   int            loc_crc;
   int            time;

   for(i_delfile=0; i_delfile<nb_files; i_delfile++)
   {
      i_ent = tf_get_random_index( );
      tf_get_entry(i_ent, &(create_file_loc_name[0]), &loc_size, &loc_crc);
      time = cyg_current_time( );
      err = rm(create_file_loc_name);
      time = cyg_current_time( ) - time ;
      diag_printf("[i] %s deleted in %dms\n", create_file_loc_name, time*10);

      if(err == E_CR_OK) tf_delfile(i_ent);
      else return err;
   }

   return err;
}



/**************************************************** 
 * vfatfs_check_tree
 * --------------------------------------------------
 * check the whole tree
 ****************************************************/
static char vfatfs_check_file_loc_name[MAX_PATH_CHAR];

int vfatfs_check_file( int *nb_tests, int *nb_error )
{
   FILE * f;
   struct stat sStat;
   int   err;
   int   loc_size = 0;
   int   loc_crc = 0;
   int   i_ent;
   int   time;
   int   crc_read;

   (*nb_tests)++;

   memset(file_buf, 0, MAX_FILE_SIZE);

   /* File checking  */
   i_ent = tf_get_random_index( );
   tf_get_entry(i_ent, &(vfatfs_check_file_loc_name[0]), &loc_size, &loc_crc); 

   err = stat( vfatfs_check_file_loc_name, &sStat);

   if(S_ISDIR(sStat.st_mode))
   {
      err = E_CR_WRONG_FILE;
      diag_printf("------------- file type error :%s is a directory\n", vfatfs_check_file_loc_name);
      (*nb_error)++;
      return err;
   }
   else 
   {
      if(sStat.st_size != loc_size )
      {
         err = E_CR_WRONG_SIZE;
         diag_printf("------------- file_db size = %d but ls size = %d\n", loc_size, sStat.st_size );
         (*nb_error)++;
         return err;
      }
   }

   f = fopen( vfatfs_check_file_loc_name, "r");
   assert(f);

   time = cyg_current_time( );
   fread(file_buf, 1, loc_size, f);
   time = cyg_current_time( ) - time ;

   crc_read = checksum(file_buf, loc_size);
   fclose(f);

   if (crc_read != loc_crc) 
   {
      diag_printf("------------- crc error: 0x%x != 0x%x \n", loc_crc);
      (*nb_error)++;
   } 
   else 
   {
      diag_printf("------------- file: %s\n", vfatfs_check_file_loc_name);
      diag_printf("------------- size: %d bytes read in %dms - %d Ko/s\n", loc_size, time*10, (loc_size)/(time*10));
      diag_printf("------------- crc : 0x%x \n", loc_crc);
   }

   return err;
}


/**************************************************** 
 * vfatfs_check_tree
 * --------------------------------------------------
 * check the whole tree
 ****************************************************/
static char vfatfs_check_tree_loc_name[MAX_PATH_CHAR];

int vfatfs_check_tree( int *nb_tests, int *nb_error )
{
   FILE  * f;
   int   nb_err;
   int   nb_files;
   int   i_ent;
   int  loc_size;
   int  loc_crc;

   error_type_t err = E_CR_OK;
   nb_err   = 0;
   nb_files = 0;

   (*nb_tests)++;

   for(i_ent=0; i_ent < MAX_FILE_NB; i_ent++)
   {
      if( tf_is_full(i_ent)  == TRUE )
      {
         tf_get_entry(i_ent, &(vfatfs_check_tree_loc_name[0]), &loc_size, &loc_crc);
         nb_files ++;
         f = fopen(vfatfs_check_tree_loc_name, "r");
         if(f == NULL)
         {
            diag_printf("[E] Can't Open %s.\n", vfatfs_check_tree_loc_name);
            nb_err++;
         }
         else
         {
            fclose(f);
            f = NULL;
         }
      }// end if( tf_is_full(i_ent)  == TRUE )
   }// for(i_ent=0; i_ent < MAX_FILE_NB; i_ent++)

   diag_printf("------------------- Checking if the %d file_db entries exists in the disk\n", nb_files);
   (*nb_error) +=nb_err;

   nb_files = 0;
   nb_err = 0;
   nb_err = tf_ls_check("/");
   diag_printf("------------------- Checking if all files in the disk are listed in file_db\n");
   (*nb_error) +=nb_err;


   return err;
}



/**************************************************** 
 * vfatfs_make_copy
 * --------------------------------------------------
 * creates a file and make a copy
 ****************************************************/
error_type_t vfatfs_make_copy( char * name, int size, char * name_dst)
{
   error_type_t err = E_CR_OK;
   FILE  * f_src;
   FILE  * f_dst;
   int   ret;
   int   time;
   int   crc;
   char  pattern;
   char  * pt_data;
   char buffer[4];


   /* Creation of the source file */
   pattern = rand( ) % 0xFF;
   pt_data = (cyg_uint8 *)malloc( size ); 
   memset(pt_data, pattern, size);
   crc = checksum(pt_data, size);

   f_src = fopen(name, "w");
   if ( f_src == NULL )
   {
      diag_printf("[I] fopen failed, memory may be full \n");
      return E_CR_FAILED;
   }
   else
   {
      time = cyg_current_time( );
      ret = fwrite(pt_data, 1, size, f_src);
      time = cyg_current_time( ) - time ;

      if (ret != size)
      {
         diag_printf("[I] fwrite failed, memory may be full \n");
         return E_CR_FAILED;
      }
      else
      {
         diag_printf("- %d Ko/s - %dms - ", (size)/(time*10), (time*10));
      }

      fclose(f_src);
   }

   free(pt_data);

   f_src = fopen(name, "r");
   if ( f_src == NULL )
   {
      diag_printf("[I] fopen failed, %s \n", name);
      return E_CR_FAILED;
   }

   f_dst = fopen(name_dst, "w");
   if ( f_dst == NULL )
   {
      diag_printf("[I] fopen failed, %s \n", name_dst);
      return E_CR_FAILED;
   }

   while ((ret = fread(buffer, 1, sizeof(buffer), f_src)) != 0)
   {
      if ( ret == 0 )
      {
         diag_printf("[I] Erreur de lecture = %x \n", ret);
         return E_CR_FAILED;
      } 
     ret = fwrite(buffer, 1, sizeof(buffer), f_dst);
      if ( ret == 0 )
      {
         diag_printf("[I] Erreur d'ecriture = %x \n", ret);
         return E_CR_FAILED;
      }

   }
      fclose(f_src);
      fclose(f_dst);

   return err;
}

/**************************************************** 
 * vfatfs_fill_it
 * --------------------------------------------------
 * creates files until the disk is full
 ****************************************************/
error_type_t vfatfs_fill_it( char * name, int size)
{
   error_type_t err;
   FILE  * f;
   int   ret;
   int   time;
   int   crc;
   char  pattern;
   char  * pt_data;

   pattern = rand( ) % 0xFF;
   pt_data = (cyg_uint8 *)malloc( size ); 
   memset(pt_data, pattern, size);
   crc = checksum(pt_data, size);

   f = fopen(name, "w");
   if ( f == NULL )
   {
      diag_printf("[I] fopen failed, memory may be full \n");
      err = E_CR_FAILED;
   }
   else
   {
      time = cyg_current_time( );
      ret = fwrite(pt_data, 1, size, f);
      time = cyg_current_time( ) - time ;

      if (ret != size)
      {
         diag_printf("[I] fwrite failed, memory may be full \n");
         err = E_CR_FAILED;
      }
      else
      {
         err = tf_addfile(name, size, crc);
         diag_printf("- %d Ko/s - %dms - ", (size)/(time*10), (time*10));
      }

      fclose(f);
   }

   free(pt_data);

   return err;
}



/**************************************************** 
 * create_file
 * --------------------------------------------------
 * creates files. If fopen or fwrite fails, it gonna delete
 * few files to get space to go on
 ****************************************************/
static char create_file_loc_name[MAX_PATH_CHAR];

error_type_t create_file( char * name, int size)
{
   error_type_t err;
   FILE  * f;
   int   ret;
   int   time;
   int   crc;
   char  pattern;
   char  * pt_data;

   pattern = rand( ) % 0xFF;
   pt_data = (cyg_uint8 *)malloc( size ); 

   memset(pt_data, pattern, size);

   crc = checksum(pt_data, size);

   f = fopen(name, "w");
   if ( f == NULL )
   {
      diag_printf("[I] fopen failed, memory may be full \n");
      diag_printf("<----- Deleting %d files \n", del_file_nb);

      /* deletes files to make room          */
      err = delete_files(del_file_nb);
      if(err != E_CR_OK) return err;

      f = fopen(name, "w");

      if ( f == NULL )
      {
         diag_printf("[E] fopen failed again ---- ERROR!! \n");
         return E_CR_FAILED;
      }
   }

   assert(f);

   time = cyg_current_time( );
   ret = fwrite(pt_data, 1, size, f);
   time = cyg_current_time( ) - time ;

   if (ret != size)
   {
      /*-------------------------------------*/
      /* if fwrite failed                    */
      /*-------------------------------------*/
      diag_printf("[i] fwrite failed, written only 0x%x bytes, memory may be full \n", ret);
      // Check disk usage to be sure 
      print_disk_usage(PRINT_CLUSTER);

      /* delete the truncated file           */
      /*-------------------------------------*/
      fclose(f);
      err = rm(name);

      if(err != E_CR_OK) return err;

      diag_printf("<----- Deleting %d files \n", del_file_nb);

      /* deletes files to make room          */
      err = delete_files(del_file_nb);
      if(err != E_CR_OK) return err;

      // Check disk usage to be sure 
      print_disk_usage(PRINT_CLUSTER);

      /*-------------------------------------------*/
      /* now there should be enough space to go on */
      /*-------------------------------------------*/
      f = fopen(name, "w");
      assert(f);
      time = cyg_current_time( );
      ret = fwrite(pt_data, 1, size, f);
      time = cyg_current_time( ) - time ;

      if ( ret != size )  
      {
         diag_printf("[E] fwrite failed again  ---- ERROR!! \n");
         fclose(f);
         return E_CR_FAILED;
      }
      fclose(f);
   }
   assert(ret == size);


   err = tf_addfile(name, size, crc);
   fclose(f);

   if (err == E_CR_NO_FREE_ENTRY)
   {
      /* deletes files to make room          */
      err = delete_files(del_file_nb);
      if(err != E_CR_OK) return err;

      diag_printf("- %d Ko/s - %dms - ", (size)/(time*10), (time*10));
      err = tf_addfile(name, size, crc);
      assert (err == E_CR_OK); //otherwise ... something i can't understand
   }
   else
   {
      diag_printf("- %d Ko/s - %dms - ", (size)/(time*10), (time*10));
   }

   free(pt_data);

   return err;
}


/**************************************************** 
 * vfatfs_test_mktree
 * --------------------------------------------------
 * This function creates a dir tree and removes it
 ****************************************************/
static char mktree_str[1024];
static char mktree_dir_name[255+1];
static char mktree_file_name[255+1];

void vfatfs_mktree(int nb_loop)
{
   error_type_t err = E_CR_OK;
   int nb_files;
   int file_size;
   int i_loop = 0;
   nb_files  = 0;

   do{
      int time;

      /* protected code section */
      cyg_mutex_lock(&mtx_file_db); 

      mk_dirname_c(mktree_dir_name);
      time = cyg_current_time( );
      mkdir(mktree_dir_name,0);
      time = cyg_current_time( ) - time ;

      diag_printf("[i] %dms to mkdir %s\n", time*10 , mktree_dir_name);

      diag_printf("%d/%d ", i_loop, nb_loop);
      mk_filename_c(mktree_file_name);
      sprintf( mktree_str, "%s/%s", mktree_dir_name, mktree_file_name );

      file_size = MIN_FILE_SIZE /*+ rand()% (MAX_FILE_SIZE - MIN_FILE_SIZE)*/ ;

      err = create_file(mktree_str,file_size);

      diag_printf("%dKo - %s\n", file_size/1024, mktree_str);
      /* end of protected code section */
      cyg_mutex_unlock(&mtx_file_db); 

      cyg_thread_delay(5); 
      nb_files++;
      i_loop ++;

   }while( (i_loop < nb_loop)&&(err != E_CR_FAILED) );
}



/**************************************************** 
 * SDtestMain
 * --------------------------------------------------
 * Main test function
 ****************************************************/
static char volume_id[255+1];

static void SDtest_thread1(CYG_ADDRESS data)
{
   int n;
   int nb_files;
   int time;

   diag_printf("*--- Starting Thd: Creating the file tree ---*\n");

   nb_files = 0;
   n = mount("/dev/sdcard", "/", "fatfs");

   if (n != 0) {
      diag_printf("mount error\n");
      assert( n == 0);
   }
   else 
   {
      //cyg_uint32 crc;
      struct cyg_fs_disk_usage du;
      cyg_uint32 len;
      cyg_uint32 disk_free;
      cyg_uint32 disk_size;
      int ret;

      //len = sizeof(crc);
      //ret = cyg_fs_getinfo("/", FS_INFO_DISK_CHECKSUM, &crc, len);
      len = sizeof(volume_id);
      ret = cyg_fs_getinfo("/", FS_INFO_VOLUME_ID, volume_id, len);
      if(ret == ENOERR)
         diag_printf("FAT DISK VOLUME ID: %s\n", volume_id);
      else
         diag_printf("FAT DISK VOLUME ID: none\n");

      /* Getting Disk Usage Info */
      len = sizeof(du);
      diag_printf("FAT DISK USAGE ...(it can take few seconds)");
      time = cyg_current_time( );
      ret = cyg_fs_getinfo("/", FS_INFO_DISK_USAGE, &du, len);
      time = cyg_current_time( ) - time ;
      diag_printf(" --> it took %d ms\n", time*10);

      /* compute SDCard parameters to get software configuration */
      max_file_nb = ( (du.total_clusters * du.cluster_size) / MIN_FILE_SIZE ) +10;
      del_file_nb = (max_file_nb *2)/10; // delete 20% of max number of files
      disk_size = (du.total_clusters * (du.cluster_size>>9) ) >> 11;
      disk_free = (du.free_clusters  * (du.cluster_size>>9) ) >> 11;

      diag_printf("               %d Mo / %d Mo \n"     , disk_free, disk_size );
      diag_printf("               %d clust / %d clust\n", du.free_clusters, du.total_clusters );

      diag_printf("------------------------------------------------------------------------\n" );
      diag_printf("| This test permits to test standards an SDCard VFAT FS ----------------\n");
      diag_printf("| PREREQUIS :\n");
      diag_printf("| file size will be between [%d Mo;%d Mo] \n",MIN_FILE_SIZE>>20, MAX_FILE_SIZE>>20 );
      diag_printf("| %d files are going to be created\n", max_file_nb );
      diag_printf("| %d files will be deleted when disk will be full\n", del_file_nb );
      diag_printf("| for this test to be significative memory must be at least %d Mo\n", (max_file_nb-10) * (MIN_FILE_SIZE >> 20) );
      diag_printf("| total disk memory is %d Mo \n", disk_size );
      diag_printf("------------------------------------------------------------------------\n" );

      //diag_printf("FAT CRC 0x%x\n", crc);

      assert(ret == ENOERR);

      //SDMain_TU( );

      /* Removing all files and dirs */
      nb_files = 0;
      time = cyg_current_time( );
      nb_files = ls_recurs("/", RECURS_ON, PRINT_OFF);
      time = cyg_current_time( ) - time ;
      diag_printf( "[I] - %d files found in %dms \n",nb_files, time*10);

      if( nb_files > 2) // '.' and '..' dirs
      {
         diag_printf( "[I] - SDCard is not empty\n", nb_files);
         time = cyg_current_time( );
         rm_dir("/");
         time = cyg_current_time( ) - time ;
         diag_printf( "[I] - All files have been erased in %dms \n", time*10);

         /* Check Disk usage now that files have been erased */
         print_disk_usage(PRINT_BOTH);
      }
      else
      {
         diag_printf( "[I] - SDCard is empty\n", nb_files);
      }

      if ( disk_max_size < (max_file_nb*(MIN_FILE_SIZE>>20)) )
      {
         vfatfs_mktree(max_file_nb);

//         a call to this function creates titi.txt, fills it and copy it to toto.txt"
//         vfatfs_make_copy( "titi.txt", (40*1024), "toto.txt");
      }

      // locking the mutex to block the other threads
      cyg_mutex_lock(&mtx_file_db);

      /* Check Disk usage now that test is over */
      print_disk_usage(PRINT_BOTH);

      time = cyg_current_time( );
      umount("/");
      time = cyg_current_time( ) - time ;
      diag_printf("<================== SDCard umounted in %ds  ==>\n", time/100);
   }

   diag_printf("** END VFAT TEST : SDtestMain **\n");
}

/**************************************************** 
 * SDtestMain
 * --------------------------------------------------
 * Main test function
 ****************************************************/
static void SDtest_thread2(CYG_ADDRESS data)
{
   int time;
   int nb_tests;
   int nb_error;

   nb_error = 0;
   nb_tests = 0;

   diag_printf("*--- Starting Thd: Checking the file tree ---*\n");
   cyg_thread_delay(THD_CHECK_TREE_PERIOD);

   for(;;)
   {
      cyg_thread_delay(THD_CHECK_TREE_PERIOD);
      cyg_mutex_lock(&mtx_file_db);
      diag_printf("<================== Checking Tree ================================>\n");
      time = cyg_current_time( );
      vfatfs_check_tree(&nb_tests, &nb_error);
      time = cyg_current_time( ) - time ;
      diag_printf("------------------- %d errors /%d tests. -->\n", nb_error, nb_tests);
      diag_printf("<================== Tree checked in %dms containinig %d files ==>\n", 
                  time*10, ls_recurs("/", RECURS_ON, PRINT_OFF) );
      cyg_mutex_unlock(&mtx_file_db);
   }
}

/**************************************************** 
 * SDtestMain
 * --------------------------------------------------
 * Main test function
 ****************************************************/
static void SDtest_thread3(CYG_ADDRESS data)
{
   int time;
   int nb_tests;
   int nb_error;

   nb_error = 0;
   nb_tests = 0;

   diag_printf("*--- Starting Thd: Checking files ---*\n");
   cyg_thread_delay(THD_CHECK_FILE_PERIOD+THD_CHECK_FILE_PERIOD/2);

   for(;;)
   {
      cyg_thread_delay(THD_CHECK_FILE_PERIOD);
      cyg_mutex_lock(&mtx_file_db);
      diag_printf("<================== Checking File ========>\n");
      time = cyg_current_time( );
      vfatfs_check_file(&nb_tests, &nb_error );
      time = cyg_current_time( ) - time ;
      diag_printf("------------------- %d errors /%d tests. -->\n", nb_error, nb_tests);
      diag_printf("<================== File checked in %dms ==>\n", time*10);
      cyg_mutex_unlock(&mtx_file_db);
   }
}


/**************************************************** 
 * cyg_user_start
 * --------------------------------------------------
 *
 ****************************************************/
void cyg_user_start(void)
{

   cyg_mutex_init(&mtx_file_db);

   cyg_thread_create(20, SDtest_thread1, (cyg_addrword_t) 0, "test SD",
         (void *)stack[0], STACKSIZE, &thread[0], &thread_obj[0]);

   cyg_thread_create(19, SDtest_thread2, (cyg_addrword_t) 1, "Tree Check",
         (void *)stack[1], STACKSIZE, &thread[1], &thread_obj[1]);

   cyg_thread_create(19, SDtest_thread3, (cyg_addrword_t) 2, "File Check",
         (void *)stack[2], STACKSIZE, &thread[2], &thread_obj[2]);

   tf_init( );

   cyg_thread_resume(thread[0]);
   cyg_thread_resume(thread[1]);
   cyg_thread_resume(thread[2]);
}
