/* this is a simple hello world program */
#include <stdio.h>

#include <cyg/hal/hal_intr.h> // Interrupt names
#include <cyg/hal/hal_io.h>
#include <cyg/kernel/kapi.h>  // thread
#include <cyg/kernel/diag.h>
#include <cyg/hal/hal_arch.h>           /* CYGNUM_HAL_STACK_SIZE_TYPICAL */

#include <unistd.h>
#include <cyg/fileio/fileio.h>
#include <dirent.h>
#include <errno.h>
#include <cyg/hal/hal_diag.h> // for HAL_DELAY_US
#include <cyg/hal/hal_io.h>     // for HAL_WRITE_UINT32

#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include "vfatfs_test.h"


/****************************************************
 * mk_dirname
 * --------------------------------------------------
 * generate a dirname
 ****************************************************/
char * mk_dirname(int i) 
{
  static char buf[255+1];
  sprintf( buf, "/dir%02x", i );
  return buf;
}

/****************************************************
 * mk_dirname
 * --------------------------------------------------
 * Generate a filename
 ****************************************************/
char * mk_filename(int i) 
{
  static char buf[255+1];
  sprintf( buf, "testfile_%05x.bin", i );
  return buf;
}



/****************************************************
 * mk_filename_c
 * --------------------------------------------------
 * Generate a complex filename
 ****************************************************/
static char pattern_fname[255+1];

void mk_filename_c(char * buf) 
{
   int i_char;
   int fn_length; // because of the const string

   fn_length = 4*(1 + rand()%4);
   i_char = 0;

//   while(i_char < fn_length)
   {
      pattern_fname[i_char] = 'a' + ( rand() % ('z' - 'a') ) ; i_char++;
//      pattern_fname[i_char] = '0' + ( rand() % ('9' - '0') ) ; i_char++;
//      pattern_fname[i_char] = 'A' + ( rand() % ('Z' - 'A') ) ; i_char++;
//      pattern_fname[i_char] = '#' + ( rand() % ('&' - '#') ) ; i_char++;
   }

   pattern_fname[i_char] = '\0';
   sprintf( buf, "f_%s.bin", pattern_fname  );

}

/****************************************************
 * mk_dirname_c
 * --------------------------------------------------
 * generate a complex dirname
 ****************************************************/
static char pattern_dname[255+1];

void mk_dirname_c(char * str) 
{
   int i_char;
   i_char = 0;

   pattern_dname[i_char] = 'a' + ( rand() % ('z' - 'a') ); i_char++;
   pattern_dname[i_char] = '#' + ( rand() % ('&' - '#') ); i_char++;
   pattern_dname[i_char] = '0' + ( rand() % ('9' - '0') ); i_char++;
   pattern_dname[i_char] = 'A' + ( rand() % ('Z' - 'A') ); i_char++;
   pattern_dname[i_char] = '\0';

  sprintf( str, "/dir%s", pattern_dname );
}





/****************************************************
 * space
 * --------------------------------------------------
 *
 ****************************************************/
void space(int i, int n)
{
   for (;i<=n;i++) {
      diag_printf(" ");
   }
}

/****************************************************
 * checksum
 * --------------------------------------------------
 *
 ****************************************************/

int checksum(char * pt_data, int data_size)
{
   int idx = 0;
   int crc = 0xA;

//   return crc;

   while(idx < data_size)
   {
      crc += pt_data[idx];
      idx ++;
   }
   return crc;

}

/****************************************************
 * rm
 * --------------------------------------------------
 *
 ****************************************************/
error_type_t rm(char * fullname)
{
   int ret;

   ret = remove(fullname);

   if (ret < 0) 
   {
         diag_printf("=> rm : Error removing file:%s\n", fullname);
         return E_CR_FAILED;
   }
/*   else
   {
         diag_printf("=> Removing file:%s\n", fullname);
   }
*/
   return E_CR_OK;
}

/****************************************************
 * ls
 * --------------------------------------------------
 * List a directory
 ****************************************************/
int ls(char * path)
{
   DIR * dir;
   struct dirent * plop;
   struct stat sStat;
   int ret;
   int n, i;
   char file_name[255+1];

   n = strlen(path);

   dir = opendir(path);

   if (dir == NULL) 
   {
      diag_printf("=>ls(%s): error on open dir\n", path);
      return -1;
   }

   while((plop = readdir(dir)) != NULL) 
   {
      i = strlen(plop->d_name);

      if (n+i > (255+1)) 
      {
         closedir(dir);
         return -1;
      }

      memcpy(file_name, path, n);
      file_name[n] = '/';
      memcpy(file_name+n+1, plop->d_name, i);
      file_name[n+i+1] = '\0';

      ret = stat(file_name, &sStat); // FIXME ADD path

      if (ret < 0) 
      {
         diag_printf("=> ls : error on stat ");
         printf(plop->d_name);
         diag_printf("\n");
      }

      if(S_ISDIR(sStat.st_mode)) 
      {
         space(diag_printf("<DIR>"), 8);
      } 
      else 
      {
         if((sStat.st_mode & S_IWUSR) == S_IWUSR) 
         {
            space(diag_printf(" RW"), 8);
         } 
         else 
         {
            space(diag_printf(" RO"), 8);
         }
      }
      diag_printf("%ld  ", sStat.st_size);
      diag_printf("%s\n",plop->d_name);
   }//while

   ret = closedir(dir);

   if (ret < 0) 
   {
      diag_printf("=> ls : error closing dir\n");
      return -1;
   }

   return 0;
}


/****************************************************
 * ls_recurs
 * --------------------------------------------------
 * List Recursively from a directory
 ****************************************************/
int ls_recurs(char * path, int rec, int print)
{
   DIR * dir;
   struct dirent * plop;
   struct stat sStat;
   int ret;
   int n, i;
   int nb_files = 0;
   char file_name[256+1];

   n   = strlen(path);
   dir = opendir(path);

   if(print == PRINT_ON)
      diag_printf("*------------------- %s -----------------------*\n", path);

   if (dir == NULL) 
   {
      diag_printf("=>ls(%s): error on open dir\n", path);
      return -1;
   }

   while((plop = readdir(dir)) != NULL) 
   {
      i = strlen(plop->d_name);

      if (n+i > (255+1)) 
      {
         closedir(dir);
         return -1;
      }

      sprintf(file_name, "%s/%s", path, plop->d_name);

      ret = stat(file_name, &sStat); // FIXME ADD path

      if (ret < 0) {
         diag_printf("=> ls : error on stat ");
         printf(plop->d_name);
         diag_printf("\n");
      }
      else
      {
         nb_files++;
      }

      if(S_ISDIR(sStat.st_mode)) 
      {
         if(print == PRINT_ON)
         {
            space(diag_printf("<DIR>"), 8);
            diag_printf("%ld  ", sStat.st_size);
            diag_printf("%s\n",file_name);
         }

         if ( (plop->d_name[0] != '.')
            &&(rec == RECURS_ON)   )
         {
            nb_files += ls_recurs(file_name, rec, print);
         }
      }// if ISDIR
      else 
      {
         if(print == PRINT_ON)
         {
            if((sStat.st_mode & S_IWUSR) == S_IWUSR) 
            {
               space(diag_printf(" RW"), 8);
               diag_printf("%ld  ", sStat.st_size);
               diag_printf("%s\n",file_name);
            } 
            else 
            {
               space(diag_printf(" RO"), 8);
               diag_printf("%ld  ", sStat.st_size);
               diag_printf("%s\n",file_name);
            }
         }
      }//else

   }//while

   ret = closedir(dir);

   if (ret < 0) {
      diag_printf("=> ls : error closing dir\n");
      return -1;
   }

   if(print == PRINT_ON)
   {
      diag_printf("*------------------- End of %s ----------------*\n", path);
   }

   return nb_files;
}


/****************************************************
 * rm_dir
 * --------------------------------------------------
 * Remove a directory
 ****************************************************/
error_type_t rm_dir(char * path)
{
   DIR * dir;
   struct dirent * plop;
   struct stat sStat;
   
   int ret;
   int n, i;
   char file_name[255+1];
   error_type_t err = E_CR_OK;

   n   = strlen(path);
   dir = opendir(path);

//   diag_printf("*------------------- %s ----------------*\n", path);

   if (dir == NULL) 
   {
      diag_printf("=>rm(%s): error on open dir\n", path);
      return E_CR_FAILED;
   }

   while((plop = readdir(dir)) != NULL) 
   {
      i = strlen(plop->d_name);

      if (n+i > (255+1)) 
      {
         closedir(dir);
         return E_CR_FAILED;
      }

      if( 0 == strcmp(path, "/") ) // do not suppress root dir
         sprintf(file_name, "%s%s", path, plop->d_name);
      else
         sprintf(file_name, "%s/%s", path, plop->d_name);

      ret = stat(file_name, &sStat); // FIXME ADD path

      if(S_ISDIR(sStat.st_mode)) 
      {
         if (plop->d_name[0] != '.') // do not suppress '.' and '..' dirs
            err = rm_dir(file_name);
      } 
      else 
      {
         err = rm(file_name);
      }
   }//while

   ret = closedir(dir);

   if ( (ret < 0) || (err != E_CR_OK) )
   {
      diag_printf("=> rm : error closing dir\n");
      return E_CR_FAILED;
   }


   if( 0 != strcmp(path, "/") ) // do not suppress root dir
      err = rm(path);

//   diag_printf("*------------------- End of %s ----------------*\n", path);

   return err;
}



/****************************************************
 * test_fatfs_open
 * --------------------------------------------------
 *
 ****************************************************/
int test_fatfs_open(char ** tab_name, int i_tab)
{
      FILE * f;
      int nb_entries;
      if( (strlen(tab_name[i_tab]) % 13 ) == 0 )
            nb_entries = strlen(tab_name[i_tab])/13 + 1;
      else
            nb_entries = strlen(tab_name[i_tab])/13 + 2;

      diag_printf("OPEN-%3d-%02d-%s\n", strlen(tab_name[i_tab]), nb_entries, tab_name[i_tab]);
      f = fopen(tab_name[i_tab], "w") ;
      if (f == NULL)
            return 1;
      fclose(f);

      return 0;
}

