#ifndef CYGONCE_HAL_PLATFORM_SETUP_H
#define CYGONCE_HAL_PLATFORM_SETUP_H
//=============================================================================
//
//      hal_platform_setup.h
//
//      Platform specific support for HAL (assembly code)
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    michael anburaj <michaelanburaj@hotmail.com>
// Contributors: michael anburaj <michaelanburaj@hotmail.com>
// Date:         2003-08-01
// Purpose:      ARM9/SMDK2412 platform specific support routines
// Description: 
// Usage:        #include <cyg/hal/hal_platform_setup.h>
//     Only used by "vectors.S"         
//
//####DESCRIPTIONEND####
//
//=============================================================================

#include <pkgconf/system.h>           // System-wide configuration info
#include CYGBLD_HAL_VARIANT_H         // Variant specific configuration
#include CYGBLD_HAL_PLATFORM_H        // Platform specific configuration
#include CYGHWR_MEMORY_LAYOUT_H
#include <cyg/hal/hal_mmu.h>          // MMU definitions
#include <cyg/hal/s3c2412x.h>         // Platform specific hardware definitions
#include CYGNUM_HAL_MEMORY_CONFIG_H   // Variant-specific memory configuration

#if (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK   == 156000000) //26Mhz * 6
#define M_MDIV      16
#define M_PDIV      2
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 169000000) //26Mhz * 6.5
#define M_MDIV      31
#define M_PDIV      4
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 182000000) //26Mhz * 7
#define M_MDIV      13
#define M_PDIV      1
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 195000000) //26Mhz * 7.5
#define M_MDIV      22
#define M_PDIV      2
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 208000000) //26Mhz * 8

#if (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_SMDK2412)
//Quartz of the SamsungDev Board is 13Mhz
	#define M_MDIV      42
	#define M_PDIV      1
	#define M_SDIV      1
#else
	#define M_MDIV      32
	#define M_PDIV      3
	#define M_SDIV      1
#endif

#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 221000000) //26Mhz * 8.5
#define M_MDIV      26
#define M_PDIV      2
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 234000000) //26Mhz * 9
#define M_MDIV      19
#define M_PDIV      1
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 243000000) //26Mhz * 9.34
#define M_MDIV      6
#define M_PDIV      1
#define M_SDIV      0
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 247000000) //26Mhz * 9.5
#define M_MDIV      30
#define M_PDIV      2
#define M_SDIV      1
#elif (CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK == 260000000) //26Mhz * 10
#define M_MDIV      22
#define M_PDIV      1
#define M_SDIV      1
#else
#error CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK not set to the right value
#endif

#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_ROMRAM)
#define PLATFORM_SETUP1 _platform_setup1
#define CYGHWR_HAL_ARM_HAS_MMU
#define CYGSEM_HAL_ROM_RESET_USES_JUMP

// This macro represents the initial startup code for the platform        
        .macro  _platform_setup1
#ifndef CYG_HAL_STARTUP_RAM
        ldr    r0,=WTCON            // watch dog disable 
        ldr    r1,=0x0         
        str    r1,[r0]

        // We must set ratios, set memctl, then change FCLK
        ldr     r0,=CLKDIVN     // Set ratios 1:2:4 for FCLK:HCLK:PCLK
#if defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV00) || \
    defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV02) || \
    defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_MSZ_DEV00)
        ldr     r1,=(CYGNUM_HAL_ARM_SMDK2412_UART_CLOCK_DIVIDER << 8) | (0<<6) | (1<<2) | (1<<0) //PCLK = HCLK / 2, HCLK = ARMCLK / 2
#else
        ldr     r1,=(CYGNUM_HAL_ARM_SMDK2412_UART_CLOCK_DIVIDER << 8) | (1<<6) | (1<<2) | (1<<0) //PCLK = HCLK / 2, HCLK = ARMCLK / 2, USB clk divide by 2
#endif
        str     r1,[r0]

        // Configure MPLL
        ldr    r0,=MPLLCON
        ldr    r1,= (M_MDIV<<12) | (M_PDIV<<4) | M_SDIV
        str    r1,[r0]

        // Configure UPLL
        ldr    r0,=UPLLCON
#if defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV00) || \
    defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV02) || \
    defined(CYGNUM_HAL_ARM_SMDK2412_FLAVOR_MSZ_DEV00)
        ldr    r1,= (40 << 12) | ( 11 << 4) | 1
#else
        ldr    r1,= (40 << 12) | ( 11 << 4)
#endif
        str    r1,[r0]

        // Configure Clock source
        ldr    r0,=CLKSRC
        ldr    r1,=0xF330
        str    r1,[r0]

        ldr    r0,=INTMSK
        ldr    r1,=0xffffffff       // all interrupt disable
        str    r1,[r0]

        ldr    r0,=INTSUBMSK
        ldr    r1,=0xffff          // all sub interrupt disable
        str    r1,[r0]

        // Disable and clear caches
        mrc  p15,0,r0,c1,c0,0
        bic  r0,r0,#0x1000              // disable ICache
        bic  r0,r0,#0x000f              // disable DCache, write buffer,
                                        // MMU and alignment faults
        mcr  p15,0,r0,c1,c0,0
        nop
        nop
        mov  r0,#0
        mcr  p15,0,r0,c7,c6,0           // clear data cache

#if 0
        mrc  p15,0,r0,c15,c1,0          // disable streaming
        orr  r0,r0,#0x80
        mcr  p15,0,r0,c15,c1,0
#endif

        // To reduce PLL lock time, adjust the LOCKTIME register. 
        ldr    r0,=LOCKTIME
        ldr    r1,=0xffffff
        str    r1,[r0]

        // MMU_SetAsyncBusMode  //Must select, since we're setting HDIVN=1
#define R1_iA   (1<<31)
#define R1_nF   (1<<30)
        mrc     p15,0,r0,c1,c0,0
        orr     r0,r0,#(R1_nF|R1_iA)
        mcr     p15,0,r0,c1,c0,0

		//Enable NAND FLash Controller
        ldr     r0,=NFCONT
        ldr     r1,=0x100C1
        str     r1,[r0]

        // Set Mem Clock = HCLK
		ldr		r0,=SSMCCR
		ldr		r1,=3
		str		r1,[r0]

        // Configure Bank priorities
        ldr		r0,=EBI_PR
        ldr		r1,=0x0
        str		r1,[r0]

        // Configure Bank
        ldr		r0,=EBI_BANKCFG
        ldr		r1,=0x40
        str		r1,[r0]

        // Set memory control registers
		ldr		r0,=SMBCR_x + SSMC_BANK6
#if ((defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV00) || \
     (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_AVEA_DEV02) || \
     (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_MSZ_DEV00)  || \
     (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_KEA_DEV00)  || \
	 (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_SMDK2412)   || \
	 (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_KANAWA)     || \
	 (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_PUTMAN_DEV00))
		ldr		r1,=0x303020 //Memory width = 32bits
#elif (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_MOGO_DEV00)
		ldr		r1,=0x303010 //Memory width = 16bits
#else
#error Flavor NOT set for _platform_setup1 - file: hal_platform_setup.c
#endif
		str		r1,[r0]

        // Set memory control registers
//		ldr		r0,=SMBCR_x + SSMC_BANK4
//		ldr		r1,=0x303020
//		str		r1,[r0]

		//Initialise SDRAM
		//1 Wait 200us
		ldr		r8,=0x5140 //2 auto refresh cycles
888:
		subs	r8,r8,#1
		bne		888b
		//2
		ldr		r0,=BANKCFG
		ldr		r1,=BANKCFG_VALUE
		str		r1,[r0]
		//3
		ldr		r0,=BANKCON1
		ldr		r1,=BANKCON1_VALUE
		str		r1,[r0]
		
		ldr		r0,=BANKCON2
		ldr		r1,=BANKCON2_VALUE
		str		r1,[r0]

		ldr		r0,=BANKCON3
		ldr		r1,=BANKCON3_VALUE
		str		r1,[r0]
		//4
		ldr		r0,=BANKCON1
		ldr		r1,=BANKCON1_VALUE | 0x1
		str		r1,[r0]
		//5
		ldr		r0,=REFRESH
		ldr		r1,=0xff
		str		r1,[r0]
		//6
		ldr		r8,=0x100 //2 auto refresh cycles
999:
		subs	r8,r8,#1
		bne		999b
		//7
		ldr		r0,=BANKCON1
		ldr		r1,=BANKCON1_VALUE | 0x2
		str		r1,[r0]
		//8
		ldr		r0,=REFRESH
		ldr		r1,=REFRESH_VALUE
		str		r1,[r0]
		//9
#if (defined CYGNUM_HAL_ARM_SMDK2412_FLAVOR_SMDK2412)
		ldr		r0,=BANKCON1
		ldr		r1,=BANKCON1_VALUE | 0x3
		str		r1,[r0]
#endif
		//10
		ldr		r0,=BANKCON1
		ldr		r1,=BANKCON1_VALUE
		str		r1,[r0]

#endif /* !CYG_HAL_STARTUP_RAM */

        // Set up a stack [for calling C code]
        ldr     r1,=__startup_stack
        ldr     r2,=CYGMEM_REGION_ram_PHYS
        orr     sp,r1,r2

		//memset to 0 mmu table
		ldr		r0,=CYGMEM_REGION_ram_PHYS + 0x4000
		add		r1,r0,#0x4000
		ldr		r2,=0
111:
		str		r2,[r0],#4
		cmp		r1,r0
		bne		111b
        
        //configure mmu
        bl      hal_mmu_init

        // Enable MMU
        ldr     r2,=10f
#ifdef CYG_HAL_STARTUP_ROMRAM
        ldr     r1,=__exception_handlers
        ldr		r9,=SMDK2412_FLASH_VIRT_BASE
        sub     r1,r2,r1
        add     r2,r9,r1        // r9 has ROM offset
#endif
        ldr     r1,=MMU_Control_Init|MMU_Control_M
		mov		r8,pc
        mcr     MMU_CP,0,r1,MMU_Control,c0
        mov     pc,r2    // Change address spaces
        nop
        nop
        nop

10:
#ifdef CYG_HAL_STARTUP_ROMRAM
        mov     r0,r9                     // Relocate FLASH/ROM to RAM
        ldr     r1,=__exception_handlers  // ram base & length
        ldr     r2,=__rom_data_end
20:     ldr     r3,[r0],#4
        str     r3,[r1],#4
        cmp     r1,r2
        bne     20b
        ldr     r0,=30f
        mov     pc,r0
        nop
        nop
        nop
        nop
30:             
#endif
        .endm
        
#else // defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_ROMRAM)
#define PLATFORM_SETUP1
#endif

//-----------------------------------------------------------------------------
// end of hal_platform_setup.h
#endif // CYGONCE_HAL_PLATFORM_SETUP_H
