#ifndef CYGONCE_SMDK2412_H
#define CYGONCE_SMDK2412_H

//=============================================================================
//
//      s3c2410x.h
//
//      Platform specific support (register layout, etc)
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    michael anburaj <michaelanburaj@hotmail.com>
// Contributors: michael anburaj <michaelanburaj@hotmail.com>
// Date:         2003-08-01
// Purpose:      SMDK2410 platform specific support definitions
// Description: 
// Usage:        #include <cyg/hal/s3c2410x.h>
//
//####DESCRIPTIONEND####
//
//=============================================================================

#include <pkgconf/hal_arm_arm9_smdk2412.h>
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H


// Memory layout details needed by conversion macro
#define SMDK2412_FLASH_PHYS_BASE         0x00000000
#define SMDK2412_FLASH_VIRT_BASE         0x80000000

// Internal clocks
#define FCLK	CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK
#define HCLK	CYGNUM_HAL_ARM_SMDK2412_BUS_CLOCK
#define PCLK	CYGNUM_HAL_ARM_SMDK2412_PERIPHERAL_CLOCK
#define UARTCLK	(CYGNUM_HAL_ARM_SMDK2412_CPU_CLOCK / (CYGNUM_HAL_ARM_SMDK2412_UART_CLOCK_DIVIDER + 1))

//2 EBI
//=====
#define EBI_PR			0x48800000 //Bus Priority decision
#define EBI_BANKCFG		0x48800004 //Bank configuration register

//3 MOBILE DRAM CONTROLLER
//========================
//Memory control 
#define BANKCFG			0x48000000  //Configuration
#define BANKCON1		0x48000004  //control
#define BANKCON2		0x48000008  //timing control 
#define BANKCON3		0x4800000c  //(E)MRS
#define REFRESH			0x48000010  //Refresh control
#define DRAM_TIMEOUT	0x48000014  //Timeout control

//Memory Control BANKCFG
#define RASBW0    (0x00<<17)  //Bit width of RAS address of Bank 0 (11 bit)
#define RASBW1    (0x00<<14)  //Bit width of RAS address of Bank 1 (11 bit)
#define CASBW0    (0x00<<11)  //Bit width of CAS address of Bank 0 ( 8 bit)
#define CASBW1    (0x00<<8)   //Bit width of CAS address of Bank 1 ( 8 bit)
#define ADDRCFG0  (0x00<<6)   //Memory address configuration (BA, RAS, CAS)
#define ADDRCFG1  (0x00<<4)   //Memory address configuration (BA, RAS, CAS)
#define MEMCFG    (0x00<<2)   //External Memory Configuration (SDR)
#define MEMBW     (0x0)       //External Memory DATA bux width (32 bits)

//Memory Control BANKCON1
#define BUSY      (0x1<<31)   //if set in busy mode
#define WBUF      (0x1<<6)    //Write buffer control (enable)
#define AP        (0x0<<5)    //Autoprecharge control (enable auto pre-charge)
#define PWRDN     (0x0<<4)    //Power down control (does not support)
#define INIT      (0x00)      //DRAM initialization control (Normal operation)

//DRAM timing control register
#define tRAS      (0xF<<20)   //Row active time (16-clock)
#define tRC       (0xF<<16)   //Row cycle time (16-clock)
#define CASLAT    (0x3<<4)    //CAS Latency Control (3-clock)
#define tRCD      (0x3<<2)    //RAS to CAS delay (4-clock)
#define tRP       (0x3)       //Row pre-charge time (4-clock)

//Mobile DRAM (E)MRS Register
#define BAE       (0x0<<30)   //Bank address for EMRS (0)
#define DS        (0x0<<21)   //Drive Strength for EMRS (0)
#define PASR      (0x0<<16)   //Partial Array Self Refresh for EMRS (0)
#define BA        (0x0<<14)   //Bank address for MRS (0)
#define ECASLAT   (0x3<<4)    //CAS Latency Control (3-clock)

//Mobile DRAM refresh control register
#define REFCYC    (0x0020)    //DRAM refresh cycle 

//Write Buffer Timeout control register
#define INIT_TIMEOUT   (0x0000)    //Write buffer timeout delay time

//4 SSMC
//======
//Offset for accessing BANK
#define SSMC_BANK0 (0x00)     //Offset bank0
#define SSMC_BANK1 (0x20)     //Offset bank1
#define SSMC_BANK2 (0x40)     //Offset bank2
#define SSMC_BANK3 (0x60)     //Offset bank3
#define SSMC_BANK4 (0x80)     //Offset bank4
#define SSMC_BANK5 (0xA0)     //Offset bank5
#define SSMC_BANK6 (0xC0)     //Offset bank6
#define SSMC_BANK7 (0xE0)     //Offset bank7

//Bank idle cycle control register 0-7 (base address)
#define SMBIDCYR_x (0x4F000000)       //Bank_x idle cycle control register
#define IDCY      (0xF)       //Idle or turnaround cycles (Turnaround time = IDCY x HCLK period)

//Bank read wait state control registers 0-7 
#define SMBWSTRDR_x (SMBIDCYR_x+0x4)   //Bank_x read wait state control register
#define WSTRD      (0x1F)     //Wait state time = WSTRD x HCLK period

//Bank write wait state control registers 0-7 
#define SMBWSTWRR_x (SMBIDCYR_x+0x8)   //Bank_x write wait state control register
#define WSTRD      (0x1F)     //Wait state time = WSTRD x HCLK period

//Bank output enable assertion delay control registers 0-7
#define SMBWSTOENR_x (SMBIDCYR_x+0xC)   //Bank_x output enable assertion delay control register
#define WSTOEN      (0x00)     //Output enable assertion delay from chip select assertion

//Bank write enable assertion delay control registers 0-7
#define SMBWSTWENR_x (SMBIDCYR_x+0x10)   //Bank_x write enable assertion delay control register
#define WSTWEN      (0x01)     //Write enable assertion delay from chip select assertion

//Bank control registers 0-7
#define SMBCR_x (SMBIDCYR_x+0x14)   //Bank_x control register

//Bank burst read wait delay control registers 0-7
#define SMBSR_x (SMBIDCYR_x+0x18)   //Bank_x status register

//Bank status registers 0-7
#define SMBWSTBRDR_x (SMBIDCYR_x+0x1C)   //Bank_x burst read wait delay control register
#define WSTBRD       (0x1F)    //Burst read wait state

#define SSMCSR			0x4F000200  //SSMCC Status Register
#define SSMCCR			0x4F000204  //SSMCC Control Register

//5 NAND FLASH CONTROLLER
//=======================
#define NFCONF			0x4e000000  //Configuration
#define NFCONT			0x4e000004  //Control
#define NFCMMD			0x4e000008  //Command
#define NFADDR			0x4e00000c  //Address
#define NFDATA			0x4e000010  //Data
#define NFMECCD0		0x4e000014  //1st and 2nd main ECC data
#define NFMECCD1		0x4e000018  //3rd and 4th main ECC data
#define NFSECCD			0x4e00001c  //Spare ECC read
#define NFSBLK			0x4e000020  //Programmable start block address 
#define NFEBLK			0x4e000024  //Programmable end block address 
#define NFSTAT			0x4e000028  //NAND status
#define NFECCERR0		0x4e00002c  //ECC error status0
#define NFECCERR1		0x4e000030  //ECC error status1
#define NFMECC0			0x4e000034  //Generated ECC status0
#define NFMECC1			0x4e000038  //Generated ECC status1
#define NFSECC			0x4e00003c  //Generated Spare area ECC status
#define NFMLCBITPT		0x4e000040  //4-bit ECC error bit pattern

//6 SYSTEM CONTROLLER
//===================
#define LOCKTIME   0x4c000000  //PLL lock time counter
#define MPLLCON    0x4c000004  //MPLL Control
#define UPLLCON    0x4c000008  //UPLL Control
#define CLKCON     0x4c00000c  //Clock generator control
#define CLKDIVN    0x4c000014  //Clock divider control
#define OSCSET     0x4c000018  //Oscillator stabilization time counter
#define CLKSRC     0x4c00001c  //Clock source control
#define PWRMODECON 0x4c000020  //Power management mode setting
#define PWRCFG     0x4c000024  //Power management configuration
#define ENDIAN     0x4c00002c  //System endian control
#define SWRSTCON   0x4c000030  //S/W reset control
#define RSTCON     0x4c000034  //Reset control
#define INFORM0    0x4c000070  //User defined inforamtion register 0
#define INFORM1    0x4c000074  //User defined inforamtion register 1
#define INFORM2    0x4c000078  //User defined inforamtion register 2
#define INFORM3    0x4c00007c  //User defined inforamtion register 3

//7 DMA CONTROLLER
//================
//Offset for accessing DMA
#define DMA_0       0x00
#define DMA_1       0x40
#define DMA_2       0x80
#define DMA_3       0xc0

#define DISRC_x     0x4b000000  //Initial source
#define DISRCC_x    0x4b000004  //Initial source control
#define DIDST_x     0x4b000008  //Initial Destination
#define DIDSTC_x    0x4b00000c  //Initial Destination control
#define DCON_x      0x4b000010  //Control
#define DSTAT_x     0x4b000014  //Status
#define DCSRC_x     0x4b000018  //Current source
#define DCDST_x     0x4b00001c  //Current destination
#define DMASKTRIG_x 0x4b000020  //Mask trigger
#define DMAREQSEL_x 0x4b000024  //Request Selection Register

//8 I/O PORT
//==========

//Offset for accessing PORT A to H
#define PORT_A      0x00 //This port doesn't have GPBDN_x and GPBSLPCON_x
#define PORT_B      0x10
#define PORT_C      0x20
#define PORT_D      0x30
#define PORT_E      0x40
#define PORT_F      0x50 //This port doesn't have GPBSLPCON
#define PORT_G      0x60
#define PORT_H      0x70

#define GP_x_CON    0x56000000  //Port x control
#define GP_x_DAT    0x56000004  //Port x data
#define GP_x_DN     0x56000008  //Port x Pull-down disable
#define GP_x_SLPCON 0x5600000c  //Port x Sleep mode configuration

#define MISCCR      0x56000090  //Miscellaneous control
#define DCLKCON     0x56000094  //DCLK0/1 control
#define EXTINT0     0x56000098  //External interrupt control register 0
#define EXTINT1     0x5600009c  //External interrupt control register 1
#define EXTINT2     0x560000A0  //External interrupt control register 2
#define EINTFLT2    0x560000Ac  //External interrupt filter control register 2
#define EINTFLT3    0x4c6000B0  //External interrupt filter control register 3
#define EINTMASK    0x560000b4  //External interrupt mask
#define EINTPEND    0x560000b8  //External interrupt pending
#define GSTATUS0    0x560000bc  //External pin status
#define GSTATUS1    0x560000c0  //Chip ID(0x32410002)
#define GSTATUS2    0x560000c4  //Reset type
#define GSTATUS3    0x560000c8  //Saved data0(32-bit) before entering POWER_OFF mode 
#define GSTATUS4    0x560000cc  //Saved data0(32-bit) before entering POWER_OFF mode 
#define GSTATUS5    0x560000d0  //Saved data0(32-bit) before entering POWER_OFF mode 
#define MSTCON      0x560000d4  //Memory stop Control
#define MSLCON      0x560000d8  //Memory sleep Control
#define DSC0        0x560000dc  //strength Control register 0
#define DSC1        0x560000e0  //strength Control register 1

//9 PWM TIMER
//===========
#define TCFG0  0x51000000  //Timer 0 configuration
#define TCFG1  0x51000004  //Timer 1 configuration
#define TCON   0x51000008  //Timer control

#define TCNTB_x		0x5100000c  //Timer count buffer		0 - 3
#define TCMPB_x		0x51000010  //Timer compare buffer		0 - 3
#define TCNTO_x		0x51000014  //Timer count observation	0 - 3

#define PWM_TIMER_OFFSET	0x0C

#define TCNTB4 0x5100003c  //Timer count buffer 4
#define TCNTO4 0x51000040  //Timer count observation 4

//10 UART
//=======
//Offset for accessing UART
#define UART0       0x0000
#define UART1       0x4000
#define UART2       0x8000

#define ULCON_x     0x50000000  //Line control
#define UCON_x      0x50000004  //Control
#define UFCON_x     0x50000008  //FIFO control
#define UMCON_x     0x5000000c  //Modem control
#define UTRSTAT_x   0x50000010  //Tx/Rx status
#define UERSTAT_x   0x50000014  //Rx error status
#define UFSTAT_x    0x50000018  //FIFO status
#define UMSTAT_x    0x5000001c  //Modem status
#define UTXH_x      0x50000020  //Tansmission Hold
#define URXH_x      0x50000024  //Receive buffer
#define UBRDIV_x    0x50000028  //Baud rate divisor
#define UDIVSLOT_x  0x5000002C  //To have a more accurate baudrate

#define OFS_ULCON     (ULCON_x-ULCON_x)    //UART Line control
#define OFS_UCON      (UCON_x-ULCON_x)     //UART Control
#define OFS_UFCON     (UFCON_x-ULCON_x)    //UART FIFO control
#define OFS_UMCON     (UMCON_x-ULCON_x)    //UART Modem control
#define OFS_UTRSTAT   (UTRSTAT_x-ULCON_x)  //UART Tx/Rx status
#define OFS_UERSTAT   (UERSTAT_x-ULCON_x)  //UART Rx error status
#define OFS_UFSTAT    (UFSTAT_x-ULCON_x)   //UART FIFO status
#define OFS_UMSTAT    (UMSTAT_x-ULCON_x)   //UART Modem status
#define OFS_UTXH      (UTXH_x-ULCON_x)     //UART Transmission Hold
#define OFS_URXH      (URXH_x-ULCON_x)     //UART Receive buffer
#define OFS_UBRDIV    (UBRDIV_x-ULCON_x)   //UART Baud rate divisor
#define OFS_UDIVSLOT  (UDIVSLOT_x-ULCON_x) //UART Baud rate divisor

#define TX_FIFO_FULL    0x4000				//Mask
#define RX_DATA_PRESENT 0x7F				//Mask if at least one bit is set there is data to read

//11 USB HOST CONTROLLER
//====================== NOT IMPLEMENTED

//12 USB DEVICE CONTROLLER
//====================== NOT CHECKED
#define FUNC_ADDR_REG     0x52000140  //Function address
#define PWR_REG           0x52000144  //Power management
#define EP_INT_REG        0x52000148  //EP Interrupt pending and clear
#define USB_INT_REG       0x52000158  //USB Interrupt pending and clear
#define EP_INT_EN_REG     0x5200015c  //Interrupt enable
#define USB_INT_EN_REG    0x5200016c
#define FRAME_NUM1_REG    0x52000170  //Frame number lower byte
#define FRAME_NUM2_REG    0x52000174  //Frame number higher byte
#define INDEX_REG         0x52000178  //Register index
#define MAXP_REG          0x52000180  //Endpoint max packet
#define EP0_CSR           0x52000184  //Endpoint 0 status
#define IN_CSR1_REG       0x52000184  //In endpoint control status
#define IN_CSR2_REG       0x52000188
#define OUT_CSR1_REG      0x52000190  //Out endpoint control status
#define OUT_CSR2_REG      0x52000194
#define OUT_FIFO_CNT1_REG 0x52000198  //Endpoint out write count
#define OUT_FIFO_CNT2_REG 0x5200019c
#define EP0_FIFO          0x520001c0  //Endpoint 0 FIFO
#define EP1_FIFO          0x520001c4  //Endpoint 1 FIFO
#define EP2_FIFO          0x520001c8  //Endpoint 2 FIFO
#define EP3_FIFO          0x520001cc  //Endpoint 3 FIFO
#define EP4_FIFO          0x520001d0  //Endpoint 4 FIFO
#define EP1_DMA_CON       0x52000200  //EP1 DMA interface control
#define EP1_DMA_UNIT      0x52000204  //EP1 DMA Tx unit counter
#define EP1_DMA_FIFO      0x52000208  //EP1 DMA Tx FIFO counter
#define EP1_DMA_TTC_L     0x5200020c  //EP1 DMA total Tx counter
#define EP1_DMA_TTC_M     0x52000210
#define EP1_DMA_TTC_H     0x52000214
#define EP2_DMA_CON       0x52000218  //EP2 DMA interface control
#define EP2_DMA_UNIT      0x5200021c  //EP2 DMA Tx unit counter
#define EP2_DMA_FIFO      0x52000220  //EP2 DMA Tx FIFO counter
#define EP2_DMA_TTC_L     0x52000224  //EP2 DMA total Tx counter
#define EP2_DMA_TTC_M     0x52000228
#define EP2_DMA_TTC_H     0x5200022c
#define EP3_DMA_CON       0x52000240  //EP3 DMA interface control
#define EP3_DMA_UNIT      0x52000244  //EP3 DMA Tx unit counter
#define EP3_DMA_FIFO      0x52000248  //EP3 DMA Tx FIFO counter
#define EP3_DMA_TTC_L     0x5200024c  //EP3 DMA total Tx counter
#define EP3_DMA_TTC_M     0x52000250
#define EP3_DMA_TTC_H     0x52000254
#define EP4_DMA_CON       0x52000258  //EP4 DMA interface control
#define EP4_DMA_UNIT      0x5200025c  //EP4 DMA Tx unit counter
#define EP4_DMA_FIFO      0x52000260  //EP4 DMA Tx FIFO counter
#define EP4_DMA_TTC_L     0x52000264  //EP4 DMA total Tx counter
#define EP4_DMA_TTC_M     0x52000268
#define EP4_DMA_TTC_H     0x5200026c


//13 INTERRUPT CONTROLLER
//=======================
#define SRCPND     0x4a000000  //Interrupt request status
#define INTMOD     0x4a000004  //Interrupt mode control
#define INTMSK     0x4a000008  //Interrupt mask control
#define PRIORITY   0x4a00000c  //IRQ priority control
#define INTPND     0x4a000010  //Interrupt request status

// PENDING BIT
#define BIT_EINT0      (0x1)
#define BIT_EINT1      (0x1<<1)
#define BIT_EINT2      (0x1<<2)
#define BIT_EINT3      (0x1<<3)
#define BIT_EINT4_7    (0x1<<4)
#define BIT_EINT8_23   (0x1<<5)
#define BIT_NOTUSED6   (0x1<<6)
#define BIT_nBAT_FLT   (0x1<<7)
#define BIT_TICK       (0x1<<8)
#define BIT_WDT        (0x1<<9)
#define BIT_TIMER0     (0x1<<10)
#define BIT_TIMER1     (0x1<<11)
#define BIT_TIMER2     (0x1<<12)
#define BIT_TIMER3     (0x1<<13)
#define BIT_TIMER4     (0x1<<14)
#define BIT_UART2      (0x1<<15)
#define BIT_LCD        (0x1<<16)
#define BIT_DMA0       (0x1<<17)
#define BIT_DMA1       (0x1<<18)
#define BIT_DMA2       (0x1<<19)
#define BIT_DMA3       (0x1<<20)
#define BIT_SDI        (0x1<<21)
#define BIT_SPI0       (0x1<<22)
#define BIT_UART1      (0x1<<23)
#define BIT_NAND       (0x1<<24)
#define BIT_USBD       (0x1<<25)
#define BIT_USBH       (0x1<<26)
#define BIT_IIC        (0x1<<27)
#define BIT_UART0      (0x1<<28)
#define BIT_SPI1       (0x1<<29)
#define BIT_RTC        (0x1<<30)
#define BIT_ADC        (0x1<<31)
#define BIT_ALLMSK     (0xffffffff)

#define INTOFFSET  0x4a000014  //Interrupt request source offset
#define SUBSRCPND  0x4a000018  //Sub source pending
#define INTSUBMSK  0x4a00001c  //Interrupt sub mask

#define BIT_SUB_ALLMSK  (0x7ff)
#define BIT_SUB_SPI1    (0x1<<15)
#define BIT_SUB_CF      (0x1<<14)
#define BIT_SUB_SDI     (0x1<<13)
#define BIT_SUB_SPI1_TO (0x1<<12)
#define BIT_SUB_SPI0_TO (0x1<<11)
#define BIT_SUB_ADC     (0x1<<10)
#define BIT_SUB_TC      (0x1<<9)
#define BIT_SUB_ERR2    (0x1<<8)
#define BIT_SUB_TXD2    (0x1<<7)
#define BIT_SUB_RXD2    (0x1<<6)
#define BIT_SUB_ERR1    (0x1<<5)
#define BIT_SUB_TXD1    (0x1<<4)
#define BIT_SUB_RXD1    (0x1<<3)
#define BIT_SUB_ERR0    (0x1<<2)
#define BIT_SUB_TXD0    (0x1<<1)
#define BIT_SUB_RXD0    (0x1<<0)

//14 LCD CONTROLLER
//=================
#define LCDCON1    0x4d000000  //LCD control 1
#define LCDCON2    0x4d000004  //LCD control 2
#define LCDCON3    0x4d000008  //LCD control 3
#define LCDCON4    0x4d00000c  //LCD control 4
#define LCDCON5    0x4d000010  //LCD control 5
#define LCDSADDR1  0x4d000014  //STN/TFT Frame buffer start address 1
#define LCDSADDR2  0x4d000018  //STN/TFT Frame buffer start address 2
#define LCDSADDR3  0x4d00001c  //STN/TFT Virtual screen address set
#define TPAL       0x4d000020  //Temporary palette register
#define LCDINTPND  0x4d000024  //LCD interrupt pending register
#define LCDSRCPND  0x4d000028  //LCD interrupt source pending register
#define LCDINTMSK  0x4d00002c  //LCD interrupt source masked register
#define TCONSEL    0x4d000030  //LPC3600 Control modes
#define LCDCON6    0x4d000034  //LCD control 6
#define LCDCON7    0x4d000038  //LCD control 7
#define LCDCON8    0x4d00003c  //LCD control 8
#define LCDCON9    0x4d000040  //LCD control 9
#define REDLUT0    0x4d000044  //Red lookup table[31:0]
#define REDLUT1    0x4d000048  //Red lookup table[63:32]
#define REDLUT2    0x4d00004c  //Red lookup table[95:64]
#define REDLUT3    0x4d000050  //Red lookup table[127:96]
#define REDLUT4    0x4d000054  //Red lookup table[159:128]
#define REDLUT5    0x4d000058  //Red lookup table[192:160]
#define REDLUT6    0x4d00005c  //Red lookup table[223:192]
#define GREENLUT0  0x4d000060  //Green lookup table[31:0]
#define GREENLUT1  0x4d000064  //Green lookup table[63:32]
#define GREENLUT2  0x4d000068  //Green lookup table[95:64]
#define GREENLUT3  0x4d00006c  //Green lookup table[127:96]
#define GREENLUT4  0x4d000070  //Green lookup table[159:128]
#define GREENLUT5  0x4d000074  //Green lookup table[192:160]
#define GREENLUT6  0x4d000078  //Green lookup table[223:192]
#define GREENLUT7  0x4d00007c  //Green lookup table[255:224]
#define GREENLUT8  0x4d000080  //Green lookup table[287:256]
#define GREENLUT9  0x4d000084  //Green lookup table[319:288]
#define GREENLUT10 0x4d000088  //Green lookup table[351:320]
#define GREENLUT11 0x4d00008c  //Green lookup table[383:352]
#define GREENLUT12 0x4d000090  //Green lookup table[415:384]
#define GREENLUT13 0x4d000094  //Green lookup table[447:416]
#define BLUELUT0   0x4d000098  //Blue lookup table[31:0]
#define BLUELUT1   0x4d00009c  //Blue lookup table[63:32]
#define BLUELUT2   0x4d0000a0  //Blue lookup table[95:64]
#define BLUELUT3   0x4d0000a4  //Blue lookup table[127:96]
#define BLUELUT4   0x4d0000a8  //Blue lookup table[159:128]
#define BLUELUT5   0x4d0000ac  //Blue lookup table[192:160]
#define BLUELUT6   0x4d0000b0  //Blue lookup table[223:192]
#define PALETTE    0x4d000400  //Palette start address


#define FRCPAT0    0x4d0000b4  //FRC Pattern Register 0
//...ALL other pattern exist to access pattern 24 : FRCPAT0 + (24 * 4)
#define FRCPAT63   0x4d0001b0  //FRC Pattern Register 63

//15 ADC & TOUCH SCREEN INTERFACE
//===============================
#define ADCCON    0x58000000  //ADC control
#define ADCTSC    0x58000004  //ADC touch screen control
#define ADCDLY    0x58000008  //ADC start or Interval Delay
#define ADCDAT0   0x5800000c  //ADC conversion data 0
#define ADCDAT1   0x58000010  //ADC conversion data 1

//16 REAL TIME CLOCK (RTC)
//========================
#define RTCCON    0x57000040  //RTC control
#define TICNT0    0x57000044  //Tick time count register 0
#define TICNT1    0x5700004C  //Tick time count register 1
#define RTCALM    0x57000050  //RTC alarm control
#define ALMSEC    0x57000054  //Alarm second
#define ALMMIN    0x57000058  //Alarm minute
#define ALMHOUR   0x5700005c  //Alarm Hour
#define ALMDATE   0x57000060  //Alarm date
#define ALMMON    0x57000064  //Alarm month
#define ALMYEAR   0x57000068  //Alarm year
#define BCDSEC    0x57000070  //BCD second 0..59
#define BCDMIN    0x57000074  //BCD minute 0..59
#define BCDHOUR   0x57000078  //BCD hour 0..24
#define BCDDATE   0x5700007c  //BCD date 1 .. 31
#define BCDDAY    0x57000080  //BCD day in the week 1 = monday, 7 = sunday
#define BCDMON    0x57000084  //BCD month 1..12
#define BCDYEAR   0x57000088  //BCD year 0..99 (2000 -> 2099)

//17 WATCH DOG TIMER
//==================
#define WTCON   0x53000000  //Watch-dog timer mode
#define WTDAT   0x53000004  //Watch-dog timer data
#define WTCNT   0x53000008  //Watch-dog timer count

//18 MMC/SD/SDIO CONTROLLER
//=========================
#define SDICON     0x5a000000  //SDI control
#define SDIPRE     0x5a000004  //SDI baud rate prescaler
#define SDICARG    0x5a000008  //SDI command argument
#define SDICCON    0x5a00000c  //SDI command control
#define SDICSTA    0x5a000010  //SDI command status
#define SDIRSP0    0x5a000014  //SDI response 0
#define SDIRSP1    0x5a000018  //SDI response 1
#define SDIRSP2    0x5a00001c  //SDI response 2
#define SDIRSP3    0x5a000020  //SDI response 3
#define SDIDTIMER  0x5a000024  //SDI data/busy timer
#define SDIBSIZE   0x5a000028  //SDI block size
#define SDIDCON    0x5a00002c  //SDI data control
#define SDIDCNT    0x5a000030  //SDI data remain counter
#define SDIDSTA    0x5a000034  //SDI data status
#define SDIFSTA    0x5a000038  //SDI FIFO status
#define SDIIMSK    0x5a00003c  //SDI interrupt mask
#define SDIDAT     0x5a000040  //SDI data

//19 IIC-BUS INTERFACE
//====================
#define IICCON  0x54000000  //IIC control
#define IICSTAT 0x54000004  //IIC status
#define IICADD  0x54000008  //IIC address
#define IICDS   0x5400000c  //IIC data shift
#define IICLC	0x54000010	//IIC Multi-Master line control register

//20 IIS-BUS INTERFACE
//====================
#define IISCON  0x55000000  //IIS Control
#define IISMOD  0x55000004  //IIS Mode
#define IISFIC  0x55000008  //IIS interface FIFO control
#define IISPSR  0x5500000c  //IIS interface clock divider control
#define IISTXD  0x55000010  //IIS interface transmit data register
#define IISRXD  0x55000014  //IIS interface receive data register

//21 SPI INTERFACE
//================
#define SPCON_x    0x59000000  //SPI control
#define SPSTA_x    0x59000004  //SPI status
#define SPPIN_x    0x59000008  //SPI pin control
#define SPPRE_x    0x5900000c  //SPI baud rate prescaler
#define SPTDAT_x   0x59000010  //SPI Tx data
#define SPRDAT_x   0x59000014  //SPI Rx data
#define SPTXFIFO_x 0x59000018  //SPI Channel x Tx FIFO
#define SPRXFIFO_x 0x5900001C  //SPI Channel x Rx FIFO
#define SPRDATB_x  0x59000020  //SPI Channel x Rx pre-data
#define SPFIC_x    0x59000024  //SPI Channel x FIFO interrupt and DMA Control

#define SPI_0      0x00000000
#define SPI_1      0x00000100

//22 ATA CONTROLLER
//================= NOT IMPLEMENTED

#endif // CYGONCE_SMDK2412_H
//-----------------------------------------------------------------------------
// end of s3c2412x.h
