/*=============================================================================
//
//      hal_diag.c
//
//      HAL diagnostic output code
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2003, 2004, eCosCentric Ltd.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):   nickg, eCosCentric
// Contributors:nickg, eCosCentric
// Date:        2004-05-07
// Purpose:     HAL diagnostic output
// Description: Implementations of HAL diagnostic output support.
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

#include <pkgconf/hal.h>
#include CYGBLD_HAL_VARIANT_H           // Variant specific configuration
#include CYGBLD_HAL_PLATFORM_H          // Platform specific configuration
#include <pkgconf/hal_arm_arm9_vpb926ejs.h>

#include <cyg/infra/cyg_type.h>         // base types
#include <cyg/infra/cyg_trac.h>         // tracing macros
#include <cyg/infra/cyg_ass.h>          // assertion macros

#include <cyg/hal/hal_arch.h>           // basic machine info
#include <cyg/hal/hal_intr.h>           // interrupt macros
#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_if.h>             // interface API
#include <cyg/hal/hal_misc.h>           // Helper functions
#include <cyg/hal/vpb926ejs.h>         // platform definitions

//-----------------------------------------------------------------------------
typedef struct {
    cyg_uint8 *base;
    cyg_int32  msec_timeout;
    int        isr_vector;
    cyg_int32  baud_rate;
    int        int_state;
} channel_data_t;

static channel_data_t vpb926ejs_ser_channels[] = {
    { (cyg_uint8 *)VPB926EJS_UART0, 1000, CYGNUM_HAL_INTERRUPT_UART0, CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD },
    { (cyg_uint8 *)VPB926EJS_UART1, 1000, CYGNUM_HAL_INTERRUPT_UART1, CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD },
    { (cyg_uint8 *)VPB926EJS_UART2, 1000, CYGNUM_HAL_INTERRUPT_UART2, CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD },
};

//-----------------------------------------------------------------------------

static void
cyg_hal_plf_serial_init_channel(void *__ch_data)
{
    channel_data_t *chan = ((channel_data_t*)__ch_data);
    cyg_uint8 *base = chan->base;
    unsigned int temp;
    unsigned int divider;
    unsigned int remainder;                                                
    unsigned int fraction;
    unsigned int ref_clk;

    // Disable everything
    HAL_WRITE_UINT32(base+_UART_CR, 0);

    // Set baud rate
    // IBRD = UART_CLK / (16 * BPS)
    // FBRD = ROUND((64 * MOD(UART_CLK,(16 * BPS))) / (16 * BPS))
    temp      = 16 * chan->baud_rate;
    ref_clk   = 24000000;
    divider   = ref_clk / temp;
    remainder = ref_clk % temp;
    temp      = (8 * remainder) / chan->baud_rate;
    fraction  = (temp >> 1) + (temp & 1);

    HAL_WRITE_UINT32(base+_UART_IBRD, divider);
    HAL_WRITE_UINT32(base+_UART_FBRD, fraction);

    // Set 8bit, 1 stop, no parity
    HAL_WRITE_UINT32(base+_UART_LCRH, _UART_LCRH_WLEN_8 | _UART_LCRH_FEN);

    // Enable UART
    HAL_WRITE_UINT32(base+_UART_CR, _UART_CR_UARTEN | _UART_CR_TXE | _UART_CR_RXE);
}

void
cyg_hal_plf_serial_putc(void *__ch_data, char c)
{
    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    cyg_uint32 status;

    do {
        HAL_READ_UINT32(base+_UART_FR, status);
    } while ((status & _UART_FR_TXFF) != 0);

    HAL_WRITE_UINT32(base+_UART_DR, c);
}

static cyg_bool
cyg_hal_plf_serial_getc_nonblock(void *__ch_data, cyg_uint8 *ch)
{
    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    cyg_uint32 status;
    cyg_uint32 c;

    HAL_READ_UINT32(base+_UART_FR, status);
    if ((status & _UART_FR_RXFE) != 0)
        return false;

    HAL_READ_UINT32(base+_UART_DR, c);
    *ch = c;

    return true;
}

cyg_uint8
cyg_hal_plf_serial_getc(void *__ch_data)
{
    cyg_uint8 ch;

    while(!cyg_hal_plf_serial_getc_nonblock(__ch_data, &ch));
    return ch;
}

static void
cyg_hal_plf_serial_write(void *__ch_data, const cyg_uint8 *__buf, 
                         cyg_uint32 __len)
{
    while(__len-- > 0)
        cyg_hal_plf_serial_putc(__ch_data, *__buf++);
}

static void
cyg_hal_plf_serial_read(void *__ch_data, cyg_uint8 *__buf, cyg_uint32 __len)
{
    while(__len-- > 0)
        *__buf++ = cyg_hal_plf_serial_getc(__ch_data);
}

cyg_bool
cyg_hal_plf_serial_getc_timeout(void *__ch_data, cyg_uint8 *ch)
{
    int delay_count;
    channel_data_t *chan = (channel_data_t*)__ch_data;
    cyg_bool res;

    delay_count = chan->msec_timeout * 10; // delay in .1 ms steps

    for(;;) {
        res = cyg_hal_plf_serial_getc_nonblock(__ch_data, ch);
        if (res || 0 == delay_count--)
            break;
        
        CYGACC_CALL_IF_DELAY_US(100);
    }
    return res;
}

static int
cyg_hal_plf_serial_control(void *__ch_data, __comm_control_cmd_t __func, ...)
{
    channel_data_t *chan = (channel_data_t*)__ch_data;
    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    int ret = 0;
    cyg_uint32 imsc;

    HAL_READ_UINT32( base+_UART_IMSC, imsc );
    switch (__func) {
    case __COMMCTL_IRQ_ENABLE:
        chan -> int_state = 1;
        imsc |= (_UART_IMSC_RXIM |
                 _UART_IMSC_RTIM |
                 _UART_IMSC_BEIM |
                 _UART_IMSC_OEIM |
                 _UART_IMSC_FEIM |
                 _UART_IMSC_PEIM );
        HAL_WRITE_UINT32(base+_UART_IMSC, imsc );
        HAL_INTERRUPT_UNMASK(chan->isr_vector);
        break;
    case __COMMCTL_IRQ_DISABLE:
        ret = chan->int_state;
        chan->int_state = 0;
        imsc &= ~(_UART_IMSC_RXIM |
                  _UART_IMSC_RTIM |
                  _UART_IMSC_BEIM |
                  _UART_IMSC_OEIM |
                  _UART_IMSC_FEIM |
                  _UART_IMSC_PEIM );
        HAL_WRITE_UINT32(base+_UART_IMSC, imsc );
        HAL_INTERRUPT_MASK(chan->isr_vector);
        break;
    case __COMMCTL_GETBAUD:
        ret = chan->baud_rate;
        break;
    case __COMMCTL_SETBAUD:
    {
	cyg_uint32 b;
        va_list ap;

        va_start(ap, __func);
        b = va_arg(ap, cyg_int32);
        va_end(ap);

        chan->baud_rate = b;
        cyg_hal_plf_serial_init_channel(chan);
        ret = 1;
        break;
    }
    case __COMMCTL_DBG_ISR_VECTOR:
        ret = chan->isr_vector;
        break;
    case __COMMCTL_SET_TIMEOUT:
    {
        va_list ap;

        va_start(ap, __func);

        ret = chan->msec_timeout;
        chan->msec_timeout = va_arg(ap, cyg_uint32);

        va_end(ap);
    }        
    default:
        break;
    }
    return ret;
}

static int
cyg_hal_plf_serial_isr(void *__ch_data, int *__ctrlc, 
                       CYG_ADDRWORD __vector, CYG_ADDRWORD __data)
{
    int res = 0;
    channel_data_t *chan = (channel_data_t *)__ch_data;
    cyg_uint8 *base = ((channel_data_t *)__ch_data)->base;
    cyg_uint32 status;
    cyg_uint32 _c;
    char c;

    cyg_drv_interrupt_acknowledge(chan->isr_vector);
    *__ctrlc = 0;
    HAL_READ_UINT32(base+_UART_FR, status);
    if ((status & _UART_FR_RXFE) == 0) {
        HAL_READ_UINT32(base+_UART_DR, _c);
        c = (char)_c;
        if( cyg_hal_is_break( &c , 1 ) )
            *__ctrlc = 1;
        res = CYG_ISR_HANDLED;
    }
    return res;
}

static void
cyg_hal_plf_serial_init(void)
{
    hal_virtual_comm_table_t *comm;
    int cur = CYGACC_CALL_IF_SET_CONSOLE_COMM(CYGNUM_CALL_IF_SET_COMM_ID_QUERY_CURRENT);
    int i;

    for (i = 0;  i < CYG_NELEM(vpb926ejs_ser_channels);  i++) {
        // Init channels
        cyg_hal_plf_serial_init_channel(&vpb926ejs_ser_channels[i]);
        // Setup procs in the vector table
        CYGACC_CALL_IF_SET_CONSOLE_COMM(i);
        comm = CYGACC_CALL_IF_CONSOLE_PROCS();
        CYGACC_COMM_IF_CH_DATA_SET(*comm, &vpb926ejs_ser_channels[i]);
        CYGACC_COMM_IF_WRITE_SET(*comm, cyg_hal_plf_serial_write);
        CYGACC_COMM_IF_READ_SET(*comm, cyg_hal_plf_serial_read);
        CYGACC_COMM_IF_PUTC_SET(*comm, cyg_hal_plf_serial_putc);
        CYGACC_COMM_IF_GETC_SET(*comm, cyg_hal_plf_serial_getc);
        CYGACC_COMM_IF_CONTROL_SET(*comm, cyg_hal_plf_serial_control);
        CYGACC_COMM_IF_DBG_ISR_SET(*comm, cyg_hal_plf_serial_isr);
        CYGACC_COMM_IF_GETC_TIMEOUT_SET(*comm, cyg_hal_plf_serial_getc_timeout);
    }

    // Restore original console
    CYGACC_CALL_IF_SET_CONSOLE_COMM(cur);
}

void
cyg_hal_plf_comms_init(void)
{
    static int initialized = 0;

    if (initialized)
        return;

    initialized = 1;

    cyg_hal_plf_serial_init();
}

/*---------------------------------------------------------------------------*/
/* End of hal_diag.c */
