#include <stdio.h>
#include <assert.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/domain.h>
#include <sys/protosw.h>
#include <sys/queue.h>
#include <sys/sysctl.h>
//#include <malloc.h>
#include <sys/mbuf.h>
#include <sys/socket.h>
#include <sys/socketvar.h>

#include <net/if.h>
#include <net/ethernet.h>

struct rifpcb {
	struct socket *so;
	struct ifnet *ifp;
	int proto;
};

static struct rifpcb  static_pcb;

/*
 * Nominal space allocated to a raw ip socket.
 */
#define	RIFSNDQ		8192
#define	RIFRCVQ		8192

u_long	rif_sendspace = RIFSNDQ;
u_long	rif_recvspace = RIFRCVQ;

void rif_init(void)
{
}

void rif_forward(struct ether_header *eh, struct ifnet *ifp, struct mbuf *m)  {
	int proto = ntohs(eh->ether_type);
	if (ifp==static_pcb.ifp && (static_pcb.proto==0 || proto==static_pcb.proto)) {
		struct ether_header *hh;
		struct mbuf *mm=m_copy(m,0, M_COPYALL);
		if (mm==NULL) return;

		if (static_pcb.proto==0) {
			M_PREPEND(mm, sizeof(*eh), M_DONTWAIT);
			hh=mtod(mm, struct ether_header*);
			*hh=*eh;
		}
		// pass the list to the socket and wake it up
		sbappend(&static_pcb.so->so_rcv, mm);
		sorwakeup(static_pcb.so);
	}
}

void rif_input(struct mbuf *m, int off)
{
	assert(0);
}

int rif_output(struct mbuf *m, struct socket *so, u_long dst)
{
	int rc=EINVAL;
	struct rifpcb *pcb=(struct rifpcb *)so->so_pcb;

	if (pcb!=NULL && pcb->ifp!=NULL) {
		int s = splimp();
		int error;
		IFQ_ENQUEUE(&pcb->ifp->if_snd, m, error);
		(*pcb->ifp->if_start)(pcb->ifp);
		splx(s);
		rc=ENOERR;
	}
	return rc;
}

int rif_ctloutput(struct socket *so, struct sockopt *sopt)
{
	return ENOERR;
}

void rif_ctlinput(int cmd, struct sockaddr *sa, void *vip)
{
}

static int rif_pcballoc(struct socket *so) {
	struct rifpcb *pcb=&static_pcb;
	int rc=ENOMEM;

	if (pcb->ifp==NULL) {
		so->so_pcb=(void *)pcb;
		pcb->ifp = NULL;
		pcb->proto = 0;
		pcb->so=so;
		rc=ENOERR;
	}
	return rc;
}

static void rif_pcbdetach(struct socket *so) {
	if (so->so_pcb!=NULL) {
		static_pcb.so=NULL;
		static_pcb.ifp=NULL;
		so->so_pcb=NULL;
	}
}

static int rif_attach(struct socket *so, int proto, struct proc *p)
{
	struct rifpcb *inp;
	int error, s;

	inp = (struct rifpcb*)so->so_pcb;
	if (inp)
		return EINVAL;

	error = soreserve(so, rif_sendspace, rif_recvspace);
	if (error)
		return error;
	s = splnet();
	error = rif_pcballoc(so);
	splx(s);
	return error;
}

static int rif_detach(struct socket *so)
{
	rif_pcbdetach(so);
	return ENOERR;
}

static int rif_abort(struct socket *so)
{
	soisdisconnected(so);
	return rif_detach(so);
}

int rif_connect(struct socket *so, struct sockaddr *nam, struct proc *p)
{
	soisconnected(so);
	return ENOERR;
}

static int rif_disconnect(struct socket *so)
{
	soisdisconnected(so);
	return ENOERR;
}

static int rif_bind(struct socket *so, struct sockaddr *nam, struct proc *p)
{
	char *ifname=(char*)nam;
	register struct ifnet *ifp = ifnet.tqh_first;
	char workbuf[32];
	int l;

	if (ifname==NULL)
		return EINVAL;

	while (ifp!=NULL) {
		snprintf(workbuf, sizeof(workbuf), "%s%d", ifp->if_name, ifp->if_unit);
		l=strlen(workbuf);
		if (strncmp(workbuf, ifname, l)==0 && (ifname[l]==0 || ifname[l]==':')) {
			struct rifpcb *pcb=(struct rifpcb *)so->so_pcb;

			if (pcb!=NULL) {
				pcb->ifp=ifp;

				if (ifname[l]==':') 
					sscanf(ifname+l+1, "%d", &pcb->proto);
				else
					pcb->proto=0;

				// printf(" raw interface socket bound to %s%d protocol %04X\n", pcb->ifp->if_name, pcb->ifp->if_unit, pcb->proto);
				return ENOERR;
			}
		}
		ifp=ifp->if_link.tqe_next;
	}
	return EINVAL;
}

static int rif_shutdown(struct socket *so)
{
	socantsendmore(so);
	return ENOERR;
}

static int rif_send(struct socket *so, int flags, struct mbuf *m, struct sockaddr *nam, struct mbuf *control, struct proc *p)
{
	return rif_output(m, so, 0);
}


static int rif_control(
	struct socket *so,
	u_long cmd,
	caddr_t data,
	register struct ifnet *ifp,
	struct proc *p)
{
	int rc=EPERM;
	struct rifpcb *pcb=(struct rifpcb *)so->so_pcb;
	if (pcb!=NULL) {
		rc=(*pcb->ifp->if_ioctl)(pcb->ifp, cmd, data);
	}
	return rc;
}

static int rif_setpeeraddr(
	struct socket *so,
	struct sockaddr **nam)
{
	return EPERM;
}

static int rif_setsockaddr(
	struct socket *so,
	struct sockaddr **nam)
{
	return EPERM;
}

struct pr_usrreqs rif_usrreqs = {
	rif_abort, pru_accept_notsupp, rif_attach, rif_bind, rif_connect,
	pru_connect2_notsupp, rif_control, rif_detach, rif_disconnect,
	pru_listen_notsupp, rif_setpeeraddr, pru_rcvd_notsupp,
	pru_rcvoob_notsupp, rif_send, pru_sense_null, rif_shutdown,
	rif_setsockaddr, sosend, soreceive, sopoll
};

extern	struct domain rifdomain;

struct protosw rifsw[] = {
	/* raw wildcard */
{ SOCK_RAW,	&rifdomain,	0,		PR_CONNREQUIRED|PR_IMPLOPCL, 
  rif_input,	0,		0,		rif_ctloutput,
  0,
  rif_init,	0,		0,		0,
  &rif_usrreqs
},
};

struct domain rifdomain =
    { AF_RIF, "raw interface", 0, 0, 0, 
      (struct protosw *)rifsw,
      (struct protosw *)&rifsw[sizeof(rifsw)/sizeof(rifsw[0])], 0,
     0 
    };

DOMAIN_SET(rif);
