//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  mem_layout.h
 * @brief Virtual memory layout
 *
 * Memory protection external symbols declaration. These symbols are used to
 * describe link-time memory layout.
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-06-28
 * ============================================================================
 */
#ifndef _MEMPROT_MEM_LAYOUT_H
#define _MEMPROT_MEM_LAYOUT_H

#ifndef __ASSEMBLER__
#include <cyg/infra/cyg_type.h>
#include <stddef.h>
#endif

#include <pkgconf/memprot.h>
#include <cyg/hal/hal_mmu.h>

#ifndef __ASSEMBLER__
extern char CYG_LABEL_NAME (__free_ram_start) [];
extern char CYG_LABEL_NAME (__heap1_end) [];
#endif

//! Virtual process memory areas
#define DECLARE_VP_AREAS(_n_) \
extern char __memprot_area_start_text_##_n_, __memprot_area_end_text_##_n_, \
       __memprot_area_start_ro_##_n_, __memprot_area_end_ro_##_n_, \
       __memprot_area_start_rw_##_n_, __memprot_area_end_rw_##_n_, \
       __memprot_area_start_data_##_n_, __memprot_area_end_data_##_n_, \
       __memprot_area_start_bss_##_n_, __memprot_area_end_bss_##_n_, \
       __memprot_area_start_heap_##_n_, __memprot_area_end_heap_##_n_, \
       __heap_top_##_n_, \
       __memprot_area_start_stacks_##_n_, __memprot_area_end_stacks_##_n_, \
       __rom_ro_start_##_n_, __rom_ro_end_##_n_, \
       __rom_data_start_##_n_, __rom_data_end_##_n_

extern char __memprot_area_start_text_ecos, __memprot_area_end_text_ecos;
extern char __memprot_area_start_data_ecos, __memprot_area_end_data_ecos;
extern char __memprot_area_start_heap_ecos, __memprot_area_end_heap_ecos;
extern char __memprot_area_start_mmu, __memprot_area_end_mmu;
extern char __memprot_rom_end, __heap_top_ecos;

DECLARE_VP_AREAS(01);
DECLARE_VP_AREAS(02);
DECLARE_VP_AREAS(03);
DECLARE_VP_AREAS(04);
DECLARE_VP_AREAS(05);
DECLARE_VP_AREAS(06);
DECLARE_VP_AREAS(07);
DECLARE_VP_AREAS(08);
DECLARE_VP_AREAS(09);
DECLARE_VP_AREAS(10);
DECLARE_VP_AREAS(11);
DECLARE_VP_AREAS(12);
DECLARE_VP_AREAS(13);
DECLARE_VP_AREAS(14);
DECLARE_VP_AREAS(15);

#define VP_INFO(_n_) { \
  .vm_start = (CYG_ADDRESS)&__memprot_area_start_ro_##_n_,\
  .vm_end = (CYG_ADDRESS)&__memprot_area_end_stacks_##_n_,\
  .ro_start = (CYG_ADDRESS)&__memprot_area_start_ro_##_n_,\
  .ro_end = (CYG_ADDRESS)&__memprot_area_end_ro_##_n_,\
  .rw_start = (CYG_ADDRESS)&__memprot_area_start_rw_##_n_,\
  .rw_end = (CYG_ADDRESS)&__memprot_area_end_rw_##_n_,\
  .data_start = (CYG_ADDRESS)&__memprot_area_start_data_##_n_,\
  .data_end = (CYG_ADDRESS)&__memprot_area_end_data_##_n_,\
  .bss_start = (CYG_ADDRESS)&__memprot_area_start_bss_##_n_,\
  .bss_end = (CYG_ADDRESS)&__memprot_area_end_bss_##_n_,\
  .heap_start = (CYG_ADDRESS)&__memprot_area_start_heap_##_n_,\
  .heap_end = (CYG_ADDRESS)&__memprot_area_end_heap_##_n_,\
  .heap_top = (CYG_ADDRESS)&__heap_top_##_n_,\
  .stacks_start = (CYG_ADDRESS)&__memprot_area_start_stacks_##_n_,\
  .stacks_end = (CYG_ADDRESS)&__memprot_area_end_stacks_##_n_,\
  .rom_ro_start = (CYG_ADDRESS)&__rom_ro_start_##_n_,\
  .rom_ro_end = (CYG_ADDRESS)&__rom_ro_end_##_n_,\
  .rom_data_start = (CYG_ADDRESS)&__rom_data_start_##_n_,\
  .rom_data_end = (CYG_ADDRESS)&__rom_data_end_##_n_,\
}

#ifdef CYGPKG_HAL_ARM_ARM9_P5DEV // FIXME ugly_ugly_ugly !
#  define ECOS_RO_START 0x100000
#else
#  define ECOS_RO_START 0x40000
#endif


// Virtual process initializers
// eCos is represented as process 0.
#define VP_LAYOUTS { \
 { \
 .ro_start = (CYG_ADDRESS)ECOS_RO_START,\
 .ro_end = (CYG_ADDRESS)&__memprot_area_start_data_ecos,\
 .rw_start = (CYG_ADDRESS)&__memprot_area_start_data_ecos,\
 .rw_end = (CYG_ADDRESS)&__memprot_area_end_data_ecos,\
 .heap_start = (CYG_ADDRESS)&__memprot_area_start_heap_ecos,\
 .heap_end = (CYG_ADDRESS)&__memprot_area_end_heap_ecos,\
 .heap_top = (CYG_ADDRESS)&__heap_top_ecos,\
    },\
 VP_INFO(01),\
 VP_INFO(02),\
 VP_INFO(03),\
 VP_INFO(04),\
 VP_INFO(05),\
 VP_INFO(06),\
 VP_INFO(07),\
 VP_INFO(08),\
 VP_INFO(09),\
 VP_INFO(10),\
 VP_INFO(11),\
 VP_INFO(12),\
 VP_INFO(13),\
 VP_INFO(14),\
 VP_INFO(15) \
}

#endif
