//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  swi_calls.c
 * @brief System calls gateways
 *
 * Memory protection system calls gateways.
 * NOTE : Additional system calls should be appended at the end of this file.
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-26
 * ============================================================================
 */

#include <cyg/infra/diag.h>
#include <cyg/kernel/kapi.h>
#include <pkgconf/memprot.h>
#include <pkgconf/system.h>

#include "diag.h"
#include "syscall.h"
#include "thread.h"
#include "memalloc.h"

// ===========================================================================
// cyg_xxx calls
// ===========================================================================

/** A useful internal function that checks the validity of a thread handle.
 *
 * This function is called in SYSTEM mode used to ensure that a thread handle
 * is valid, i.e. appears in the current threads list.
 *
 * @param thread The thread handle to check
 * @return The checked handle, or 0 if thread is not valid
 */
static cyg_handle_t __check_thread_handle( cyg_handle_t thread ) {
  cyg_uint16 id;
  // first check that we are not going to abort in kernel mode
  if ( !__check_system_read(thread, sizeof(cyg_thread)))
    return (cyg_handle_t)0;
  // then get a (possibly wrong) thread ID
  id = ((cyg_thread *)thread)->unique_id;
  // and search for this ID in thread linked list
  return cyg_thread_find(id);
}

// cyg_thread_create() is replaced with cyg_memprot_create()
SWI_DECLARE( cyg_thread_create, __memprot_cyg_thread_create);
void __memprot_cyg_thread_create( cyg_addrword_t      sched_info,
                  cyg_thread_entry_t  *entry,
                  cyg_addrword_t      entry_data,
                  char                *name,
                  void                *stack_base,
                  cyg_ucount32        stack_size,
                  cyg_handle_t        *handle,
                  cyg_thread          *thread ) {
  if (( !__check_user_read(entry, 4))||
      ( !__check_user_write(handle, sizeof(cyg_handle_t)))) {
    INFO("bad parameters in cyg_thread_create(): entry=0x%08x handle=0x%08x\n",
     (unsigned int)entry, (unsigned int)handle);
    return;
  }
  cyg_memprot_thread_create(sched_info,entry,entry_data,name,stack_base,
                stack_size, handle, thread);
}

// these don't require privileges (but of course return garbage if misused)
/*
  cyg_handle_t cyg_thread_self(void);
  cyg_handle_t cyg_thread_idle_thread(void);
  cyg_addrword_t cyg_thread_get_stack_base(cyg_handle_t thread);
  cyg_uint32 cyg_thread_get_stack_size(cyg_handle_t thread);
  cyg_uint32 cyg_thread_measure_stack_usage(cyg_handle_t thread);
*/

SWI_DECLARE( cyg_thread_yield, cyg_thread_yield);
SWI_DECLARE( cyg_thread_delay, cyg_thread_delay);
SWI_DECLARE( cyg_thread_suspend, cyg_thread_suspend);
SWI_DECLARE( cyg_thread_release, cyg_thread_release );


SWI_DECLARE( cyg_thread_resume, __memprot_cyg_thread_resume );
void __memprot_cyg_thread_resume(cyg_handle_t thread) {
  if ( !__check_owner(thread, sizeof(thread)) ) {
    INFO("permission denied in cyg_thread_resume() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  if ( __check_thread_handle(thread) != thread ) {
    INFO("bad thread handle in cyg_thread_resume() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  cyg_thread_resume(thread);
}

SWI_DECLARE( cyg_thread_get_next, __memprot_cyg_thread_get_next );
cyg_bool __memprot_cyg_thread_get_next(cyg_handle_t *thread, cyg_uint16 *id) {
  cyg_handle_t rthread;
  if ( !__check_user_write(thread, sizeof(cyg_handle_t)) ||
       !__check_user_write(id, sizeof(cyg_uint16)) ) {
    INFO("bad parameters in cyg_thread_next() : thread=0x%08x, id=0x%08x\n",
     (unsigned int)thread, (unsigned int)id );
    return 0;
  }
  rthread = __check_thread_handle(*thread);
  if (( rthread != *thread )||
      ( ((cyg_thread *)(*thread))->unique_id != *id )) {
    INFO("invalid handle in cyg_thread_next() : *thread=0x%08x, *id=0x%08x\n",
     (unsigned int)*thread, (unsigned int)*id );
    return 0;
  }
  return cyg_thread_get_next(thread,id);
}

SWI_DECLARE( cyg_thread_get_info, __memprot_cyg_thread_get_info );
cyg_bool __memprot_cyg_thread_get_info( cyg_handle_t thread,
                    cyg_uint16 id,
                    cyg_thread_info *info ) {
  if ( !__check_user_write(info, sizeof(cyg_thread_info)) ||
       ( __check_thread_handle(thread) != thread )) {
    INFO("bad parameters in cyg_thread_get_info( 0x%08x, %d, 0x%08x )\n",
     (unsigned int)thread, id, (unsigned int)info );
    return 0;
  }
  return cyg_thread_get_info( thread, id, info);
}

// cyg_uint16 cyg_thread_get_id(cyg_handle_t thread)

SWI_DECLARE( cyg_thread_find, cyg_thread_find );

// cyg_thread_exit() is replaced with cyg_memprot_thread_exit()
SWI_DECLARE( cyg_thread_exit, cyg_memprot_thread_exit );

SWI_DECLARE( cyg_thread_kill, __memprot_cyg_thread_kill );
void __memprot_cyg_thread_kill(cyg_handle_t thread) {
  if ( !__check_owner(thread, sizeof(thread)) ) {
    INFO("permission denied in cyg_thread_kill() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  if ( __check_thread_handle(thread) != thread ) {
    INFO("bad thread handle in cyg_thread_kill() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  cyg_thread_kill(thread);
}

// cyg_thread_delete() is replaced with cyg_memprot_thread_delete()
SWI_DECLARE( cyg_thread_delete, __memprot_cyg_thread_delete );
void __memprot_cyg_thread_delete(cyg_handle_t thread) {
  if ( !__check_owner((void *)thread, sizeof(thread)) ) {
    INFO("permission denied in cyg_thread_delete() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  if ( __check_thread_handle(thread) != thread ) {
    INFO("bad thread handle in cyg_thread_delete() : thread=0x%08x\n",
     (unsigned int)thread);
    return;
  }
  cyg_memprot_thread_delete(thread);
}


// these are seldom used
/*
  cyg_priority_t cyg_thread_get_priority(cyg_handle_t thread);
  cyg_priority_t cyg_thread_get_current_priority(cyg_handle_t thread);
  void cyg_thread_set_priority(cyg_handle_t thread, cyg_priority_t priority);
*/

// these shouldn't be used in user mode
/*
  cyg_ucount32 cyg_thread_new_data_index(void);
  void cyg_thread_free_data_index(cyg_ucount32 index);
*/

// these can be called from user mode, except for a few CYG_ASSERTS
/*
  cyg_addrword_t cyg_thread_get_data(cyg_ucount32 index);
  cyg_addrword_t* cyg_thread_get_data_ptr(cyg_ucount32 index);
*/

SWI_DECLARE( cyg_thread_set_data, __memprot_cyg_thread_set_data );
void __memprot_cyg_thread_set_data(cyg_ucount32 index, cyg_addrword_t data) {
  if ( index < CYGNUM_KERNEL_THREADS_DATA_MAX )
    cyg_thread_set_data( index, data );
}

// these shouldn't be used in user mode
/*
  cyg_bool_t cyg_thread_add_destructor(cyg_thread_destructor_fn fn, cyg_addrword_t data);
  cyg_bool_t cyg_thread_rem_destructor(cyg_thread_destructor_fn fn, cyg_addrword_t data);
*/


// void cyg_mutex_init(cyg_mutex_t* mutex);
// void cyg_mutex_destroy(cyg_mutex_t* mutex);

SWI_DECLARE( cyg_mutex_lock, __memprot_cyg_mutex_lock );
cyg_bool_t __memprot_cyg_mutex_lock(cyg_mutex_t* mutex) {
  if ( !__check_user_write(mutex, sizeof(cyg_mutex_t)) ) {
    INFO("permission denied in cyg_mutex_lock() : mutex=0x%08x\n",
     (unsigned int)mutex);
    return (cyg_bool_t)0;
  }
  return cyg_mutex_lock(mutex);
}

SWI_DECLARE( cyg_mutex_trylock, __memprot_cyg_mutex_trylock );
cyg_bool_t __memprot_cyg_mutex_trylock(cyg_mutex_t* mutex) {
  if ( !__check_user_write(mutex, sizeof(cyg_mutex_t)) ) {
    INFO("permission denied in cyg_mutex_trylock() : mutex=0x%08x\n",
     (unsigned int)mutex);
    return (cyg_bool_t)0;
  }
  return cyg_mutex_trylock(mutex);
}

SWI_DECLARE( cyg_mutex_unlock, __memprot_cyg_mutex_unlock );
void __memprot_cyg_mutex_unlock(cyg_mutex_t* mutex) {
  if ( !__check_user_write(mutex, sizeof(cyg_mutex_t)) ) {
    INFO("permission denied in cyg_mutex_unlock() : mutex=0x%08x\n",
     (unsigned int)mutex);
    return;
  }
  cyg_mutex_unlock(mutex);
}

// These are seldom used :

// void cyg_mutex_release(cyg_mutex_t* mutex);
// void cyg_mutex_set_ceiling(cyg_mutex_t* mutex, cyg_priority_t priority);
// void cyg_mutex_set_protocol(cyg_mutex_t* mutex, enum cyg_mutex_protocol p);


// TODO !!
SWI_DECLARE( cyg_thread_set_priority, cyg_thread_set_priority );
SWI_DECLARE( cyg_mbox_create, cyg_mbox_create );
SWI_DECLARE( cyg_mbox_delete, cyg_mbox_delete );
SWI_DECLARE( cyg_mbox_get, cyg_mbox_get );
SWI_DECLARE( cyg_mbox_timed_get, cyg_mbox_timed_get );
SWI_DECLARE( cyg_mbox_tryget, cyg_mbox_tryget );
SWI_DECLARE( cyg_mbox_peek, cyg_mbox_peek );
SWI_DECLARE( cyg_mbox_peek_item, cyg_mbox_peek_item );
SWI_DECLARE( cyg_mbox_put, cyg_mbox_put );
SWI_DECLARE( cyg_mbox_timed_put, cyg_mbox_timed_put );
SWI_DECLARE( cyg_mbox_tryput, cyg_mbox_tryput );
SWI_DECLARE( cyg_mbox_waiting_to_get, cyg_mbox_waiting_to_get );
SWI_DECLARE( cyg_mbox_waiting_to_put, cyg_mbox_waiting_to_put );
SWI_DECLARE( cyg_semaphore_init, cyg_semaphore_init );
SWI_DECLARE( cyg_semaphore_destroy, cyg_semaphore_destroy );
SWI_DECLARE( cyg_semaphore_wait, cyg_semaphore_wait );
SWI_DECLARE( cyg_semaphore_timed_wait, cyg_semaphore_timed_wait );
SWI_DECLARE( cyg_semaphore_trywait, cyg_semaphore_trywait );
SWI_DECLARE( cyg_semaphore_post, cyg_semaphore_post );
SWI_DECLARE( cyg_semaphore_peek, cyg_semaphore_peek );
SWI_DECLARE( cyg_flag_init, cyg_flag_init );
SWI_DECLARE( cyg_flag_destroy, cyg_flag_destroy );
SWI_DECLARE( cyg_flag_setbits, cyg_flag_setbits );
SWI_DECLARE( cyg_flag_maskbits, cyg_flag_maskbits );
SWI_DECLARE( cyg_flag_wait, cyg_flag_wait );
SWI_DECLARE( cyg_flag_timed_wait, cyg_flag_timed_wait );
SWI_DECLARE( cyg_flag_poll, cyg_flag_poll );
SWI_DECLARE( cyg_flag_peek, cyg_flag_peek );
SWI_DECLARE( cyg_flag_waiting, cyg_flag_waiting );
SWI_DECLARE( cyg_cond_init, cyg_cond_init );
SWI_DECLARE( cyg_cond_destroy, cyg_cond_destroy );
SWI_DECLARE( cyg_cond_wait, cyg_cond_wait );
SWI_DECLARE( cyg_cond_signal, cyg_cond_signal );
SWI_DECLARE( cyg_cond_broadcast, cyg_cond_broadcast );
SWI_DECLARE( cyg_cond_timed_wait, cyg_cond_timed_wait );
SWI_DECLARE( cyg_io_lookup, cyg_io_lookup );
SWI_DECLARE( cyg_io_write, cyg_io_write );
SWI_DECLARE( cyg_io_read, cyg_io_read );
SWI_DECLARE( cyg_io_bwrite, cyg_io_bwrite );
SWI_DECLARE( cyg_io_bread, cyg_io_bread );
SWI_DECLARE( cyg_io_select, cyg_io_select );
SWI_DECLARE( cyg_io_get_config, cyg_io_get_config );
SWI_DECLARE( cyg_io_get_config_word, cyg_io_get_config_word );
SWI_DECLARE( cyg_io_set_config, cyg_io_set_config );
SWI_DECLARE( cyg_io_set_config_word, cyg_io_set_config_word );
SWI_DECLARE( cyg_alarm_create, cyg_alarm_create );
SWI_DECLARE( cyg_alarm_delete, cyg_alarm_delete );
SWI_DECLARE( cyg_alarm_initialize, cyg_alarm_initialize );
SWI_DECLARE( cyg_alarm_enable, cyg_alarm_enable );
SWI_DECLARE( cyg_alarm_disable, cyg_alarm_disable );
SWI_DECLARE( cyg_clock_create, cyg_clock_create );
SWI_DECLARE( cyg_clock_delete, cyg_clock_delete );
SWI_DECLARE( cyg_clock_to_counter, cyg_clock_to_counter );
SWI_DECLARE( cyg_clock_set_resolution, cyg_clock_set_resolution );
SWI_DECLARE( cyg_clock_get_resolution, cyg_clock_get_resolution );
SWI_DECLARE( cyg_real_time_clock, cyg_real_time_clock );
SWI_DECLARE( cyg_current_time, cyg_current_time );
SWI_DECLARE( cyg_counter_create, cyg_counter_create );
SWI_DECLARE( cyg_counter_delete, cyg_counter_delete );
SWI_DECLARE( cyg_counter_current_value, cyg_counter_current_value );
SWI_DECLARE( cyg_counter_set_value, cyg_counter_set_value );
SWI_DECLARE( cyg_counter_tick, cyg_counter_tick );

// ===========================================================================
// libc calls
// ===========================================================================

// TODO !!

SWI_DECLARE( fclose, fclose );
SWI_DECLARE( fflush, fflush );
SWI_DECLARE( fopen, fopen );
SWI_DECLARE( freopen, freopen );
SWI_DECLARE( ferror, ferror );
SWI_DECLARE( remove, remove );
SWI_DECLARE( clearerr, clearerr );
SWI_DECLARE( fileno, fileno );
SWI_DECLARE( setvbuf, setvbuf );
SWI_DECLARE( snprintf, snprintf );
SWI_DECLARE( sprintf, sprintf );
SWI_DECLARE( sscanf, sscanf );
SWI_DECLARE( feof, feof );
SWI_DECLARE( fgetpos, fgetpos );
SWI_DECLARE( fseek, fseek );
SWI_DECLARE( fsetpos, fsetpos );
SWI_DECLARE( ftell, ftell );
SWI_DECLARE( rewind, rewind );
SWI_DECLARE( perror, perror );
SWI_DECLARE( puts, puts );
SWI_DECLARE( setbuf, setbuf );
SWI_DECLARE( vfprintf, vfprintf );
SWI_DECLARE( vprintf, vprintf );
SWI_DECLARE( vscanf, vscanf );
SWI_DECLARE( vsprintf, vsprintf );
SWI_DECLARE( ungetc, ungetc );
SWI_DECLARE( vsnprintf, vsnprintf );
SWI_DECLARE( vsscanf, vsscanf );
SWI_DECLARE( fgetc, fgetc );
SWI_DECLARE( getc, getc );
SWI_DECLARE( getchar, getchar );
SWI_DECLARE( fgets, fgets );
SWI_DECLARE( fread, fread );
SWI_DECLARE( fscanf, fscanf );
SWI_DECLARE( gets, gets );
SWI_DECLARE( scanf, scanf );
SWI_DECLARE( vfscanf, vfscanf );
SWI_DECLARE( fnprintf, fnprintf );
SWI_DECLARE( fprintf, fprintf );
SWI_DECLARE( fputc, fputc );
SWI_DECLARE( putc, putc );
SWI_DECLARE( putchar, putchar );
SWI_DECLARE( fputs, fputs );
SWI_DECLARE( fwrite, fwrite );
SWI_DECLARE( printf, printf );
SWI_DECLARE( vfnprintf, vfnprintf );
SWI_DECLARE( isalnum, isalnum );
SWI_DECLARE( isalpha, isalpha );
SWI_DECLARE( iscntrl, iscntrl );
SWI_DECLARE( isdigit, isdigit );
SWI_DECLARE( isgraph, isgraph );
SWI_DECLARE( islower, islower );
SWI_DECLARE( isprint, isprint );
SWI_DECLARE( ispunct, ispunct );
SWI_DECLARE( isspace, isspace );
SWI_DECLARE( isupper, isupper );
SWI_DECLARE( isxdigit, isxdigit );
SWI_DECLARE( tolower, tolower );
SWI_DECLARE( toupper, toupper );
SWI_DECLARE( localeconv, localeconv );
SWI_DECLARE( setlocale, setlocale );
SWI_DECLARE( longjmp, longjmp );
SWI_DECLARE( raise, raise );
SWI_DECLARE( signal, signal );
SWI_DECLARE( _exit, _exit );
SWI_DECLARE( abort, abort );
SWI_DECLARE( atexit, atexit );
SWI_DECLARE( exit, exit );
SWI_DECLARE( abs, abs );
SWI_DECLARE( labs, labs );
SWI_DECLARE( atof, atof );
SWI_DECLARE( atoi, atoi );
SWI_DECLARE( atol, atol );
SWI_DECLARE( bsearch, bsearch );
SWI_DECLARE( div, div );
SWI_DECLARE( ldiv, ldiv );
SWI_DECLARE( getenv, getenv );
SWI_DECLARE( qsort, qsort );
SWI_DECLARE( rand, rand );
SWI_DECLARE( rand_r, rand_r );
SWI_DECLARE( srand, srand );
SWI_DECLARE( strtod, strtod );
SWI_DECLARE( strtol, strtol );
SWI_DECLARE( strtoul, strtoul );
SWI_DECLARE( system, system );
SWI_DECLARE( memchr, memchr );
SWI_DECLARE( memcmp, memcmp );
SWI_DECLARE( memmove, memmove );
SWI_DECLARE( strcat, strcat );
SWI_DECLARE( strchr, strchr );
SWI_DECLARE( strcmp, strcmp );
SWI_DECLARE( strcoll, strcoll );
SWI_DECLARE( strcpy, strcpy );
SWI_DECLARE( strcspn, strcspn );
SWI_DECLARE( strlen, strlen );
SWI_DECLARE( strncat, strncat );
SWI_DECLARE( strncmp, strncmp );
SWI_DECLARE( strncpy, strncpy );
SWI_DECLARE( strpbrk, strpbrk );
SWI_DECLARE( strrchr, strrchr );
SWI_DECLARE( strspn, strspn );
SWI_DECLARE( strstr, strstr );
SWI_DECLARE( strtok, strtok );
SWI_DECLARE( strtok_r, strtok_r );
SWI_DECLARE( strxfrm, strxfrm );
SWI_DECLARE( strdup, strdup );
SWI_DECLARE( asctime, asctime );
SWI_DECLARE( asctime_r, asctime_r );
SWI_DECLARE( clock, clock );
SWI_DECLARE( ctime, ctime );
SWI_DECLARE( ctime_r, ctime_r );
SWI_DECLARE( difftime, difftime );
SWI_DECLARE( gmtime, gmtime );
SWI_DECLARE( localtime, localtime );
SWI_DECLARE( localtime_r, localtime_r );
SWI_DECLARE( mktime, mktime );
SWI_DECLARE( strftime, strftime );
SWI_DECLARE( time, time );
SWI_DECLARE( strerror, strerror );
SWI_DECLARE( cyg_error_get_errno_p, cyg_error_get_errno_p );
SWI_DECLARE( dup, dup );
SWI_DECLARE( dup2, dup2 );
SWI_DECLARE( access, access );
SWI_DECLARE( chdir, chdir );
SWI_DECLARE( mount, mount );
SWI_DECLARE( umount, umount );
SWI_DECLARE( creat, creat );
SWI_DECLARE( getcwd, getcwd );
SWI_DECLARE( link, link );
SWI_DECLARE( mkdir, mkdir );
SWI_DECLARE( open, open );
SWI_DECLARE( pathconf, pathconf );
SWI_DECLARE( rename, rename );
SWI_DECLARE( rmdir, rmdir );
SWI_DECLARE( stat, stat );
SWI_DECLARE( unlink, unlink );
SWI_DECLARE( close, close );
SWI_DECLARE( fcntl, fcntl );
SWI_DECLARE( fpathconf, fpathconf );
SWI_DECLARE( fstat, fstat );
SWI_DECLARE( fsync, fsync );
SWI_DECLARE( ioctl, ioctl );
SWI_DECLARE( isatty, isatty );
SWI_DECLARE( lseek, lseek );
SWI_DECLARE( read, read );
SWI_DECLARE( write, write );
SWI_DECLARE( select, select );
SWI_DECLARE( closedir, closedir );
SWI_DECLARE( opendir, opendir );
SWI_DECLARE( readdir, readdir );
SWI_DECLARE( readdir_r, readdir_r );
SWI_DECLARE( rewinddir, rewinddir );
SWI_DECLARE( accept, accept );
SWI_DECLARE( bind, bind );
SWI_DECLARE( connect, connect );
SWI_DECLARE( getpeername, getpeername );
SWI_DECLARE( getsockname, getsockname );
SWI_DECLARE( getsockopt, getsockopt );
SWI_DECLARE( listen, listen );
SWI_DECLARE( poll, poll );
SWI_DECLARE( recv, recv );
SWI_DECLARE( recvfrom, recvfrom );
SWI_DECLARE( recvmsg, recvmsg );
SWI_DECLARE( send, send );
SWI_DECLARE( sendmsg, sendmsg );
SWI_DECLARE( sendto, sendto );
SWI_DECLARE( setsockopt, setsockopt );
SWI_DECLARE( shutdown, shutdown );
SWI_DECLARE( socket, socket );
SWI_DECLARE( getprotobyname, getprotobyname );
SWI_DECLARE( getprotobynumber, getprotobynumber );
SWI_DECLARE( getservbyname, getservbyname );
SWI_DECLARE( getservbyport, getservbyport );
SWI_DECLARE( gethostbyaddr, gethostbyaddr );
SWI_DECLARE( gethostbyname, gethostbyname );
SWI_DECLARE( tftp_get, tftp_get );
SWI_DECLARE( tftp_put, tftp_put );
SWI_DECLARE( tftpd_start, tftpd_start );
SWI_DECLARE( tftpd_stop, tftpd_stop );
SWI_DECLARE( mq_close, mq_close );
SWI_DECLARE( mq_getattr, mq_getattr );
SWI_DECLARE( mq_notify, mq_notify );
SWI_DECLARE( mq_open, mq_open );
SWI_DECLARE( mq_receive, mq_receive );
SWI_DECLARE( mq_send, mq_send );
SWI_DECLARE( mq_setattr, mq_setattr );
SWI_DECLARE( mq_unlink, mq_unlink );
SWI_DECLARE( sem_close, sem_close );
SWI_DECLARE( sem_destroy, sem_destroy );
SWI_DECLARE( sem_getvalue, sem_getvalue );
SWI_DECLARE( sem_init, sem_init );
SWI_DECLARE( sem_open, sem_open );
SWI_DECLARE( sem_post, sem_post );
SWI_DECLARE( sem_trywait, sem_trywait );
SWI_DECLARE( sem_unlink, sem_unlink );
SWI_DECLARE( sem_wait, sem_wait );
SWI_DECLARE( clock_getres, clock_getres );
SWI_DECLARE( clock_gettime, clock_gettime );
SWI_DECLARE( clock_settime, clock_settime );
SWI_DECLARE( nanosleep, nanosleep );
SWI_DECLARE( timer_create, timer_create );
SWI_DECLARE( timer_delete, timer_delete );
SWI_DECLARE( timer_getoverrun, timer_getoverrun );
SWI_DECLARE( timer_gettime, timer_gettime );
SWI_DECLARE( timer_settime, timer_settime );
SWI_DECLARE( alarm, alarm );
SWI_DECLARE( kill, kill );
SWI_DECLARE( pause, pause );
SWI_DECLARE( pthread_kill, pthread_kill );
SWI_DECLARE( pthread_sigmask, pthread_sigmask );
SWI_DECLARE( sigpending, sigpending );
SWI_DECLARE( sigaction, sigaction );
SWI_DECLARE( sigaddset, sigaddset );
SWI_DECLARE( sigdelset, sigdelset );
SWI_DECLARE( sigemptyset, sigemptyset );
SWI_DECLARE( sigfillset, sigfillset );
SWI_DECLARE( sigismember, sigismember );
SWI_DECLARE( siglongjmp, siglongjmp );
SWI_DECLARE( sigprocmask, sigprocmask );
SWI_DECLARE( sigqueue, sigqueue );
SWI_DECLARE( sigsuspend, sigsuspend );
SWI_DECLARE( sigtimedwait, sigtimedwait );
SWI_DECLARE( sigwait, sigwait );
SWI_DECLARE( sigwaitinfo, sigwaitinfo );
SWI_DECLARE( sleep, sleep );
SWI_DECLARE( sched_get_priority_max, sched_get_priority_max );
SWI_DECLARE( sched_get_priority_min, sched_get_priority_min );
SWI_DECLARE( sched_getparam, sched_getparam );
SWI_DECLARE( sched_getscheduler, sched_getscheduler );
SWI_DECLARE( sched_rr_get_interval, sched_rr_get_interval );
SWI_DECLARE( sched_setparam, sched_setparam );
SWI_DECLARE( sched_setscheduler, sched_setscheduler );
SWI_DECLARE( sched_yield, sched_yield );
SWI_DECLARE( getegid, getegid );
SWI_DECLARE( geteuid, geteuid );
SWI_DECLARE( getgid, getgid );
SWI_DECLARE( getgroups, getgroups );
SWI_DECLARE( getpgrp, getpgrp );
SWI_DECLARE( getpid, getpid );
SWI_DECLARE( getppid, getppid );
SWI_DECLARE( getuid, getuid );
SWI_DECLARE( setgid, setgid );
SWI_DECLARE( setpgid, setpgid );
SWI_DECLARE( setsid, setsid );
SWI_DECLARE( setuid, setuid );
SWI_DECLARE( sysconf, sysconf );
SWI_DECLARE( htonl, htonl );
SWI_DECLARE( htons, htons );
SWI_DECLARE( inet_ntoa, inet_ntoa );
SWI_DECLARE( ntohs, ntohs );
SWI_DECLARE( gettimeofday, gettimeofday );
SWI_DECLARE( statMode, statMode );

// Miscellanous calls from various packages

SWI_DECLARE( cyg_ppp_options_init, cyg_ppp_options_init );
SWI_DECLARE( cyg_ppp_up, cyg_ppp_up );
SWI_DECLARE( cyg_ppp_down, cyg_ppp_down );
SWI_DECLARE( cyg_ppp_wait_up, cyg_ppp_wait_up );
SWI_DECLARE( cyg_ppp_wait_down, cyg_ppp_wait_down );
SWI_DECLARE( cyg_ppp_chat, cyg_ppp_chat );

SWI_DECLARE( init_all_network_interfaces, init_all_network_interfaces );

SWI_DECLARE( tftp_client_get, tftp_client_get );
SWI_DECLARE( tftp_client_put, tftp_client_put );

SWI_DECLARE( cyg_dns_res_start, cyg_dns_res_start );
SWI_DECLARE( cyg_dns_res_init, cyg_dns_res_init );
SWI_DECLARE( getdomainname, getdomainname );
SWI_DECLARE( setdomainname, setdomainname );


// memory allocation is done in user mode

#ifdef CYGOPT_MEMPROT_USER_MALLOC

#undef  SWI_DECLARE
#define SWI_DECLARE(_syscall_,_gate_)

// an *ugly* hack to know from which domain a function was called
#define GET_LR() ({ \
  cyg_uint32 _lr; \
  asm volatile ("mov %0,lr" : "=r"(_lr)); \
  _lr; \
})

SWI_DECLARE( malloc, malloc );
void * __swi_malloc( size_t size ) {
  return cyg_memprot_malloc( size, GET_LR());
}

SWI_DECLARE( free, free );
void __swi_free( void *ptr ) {
  cyg_memprot_free(ptr, GET_LR());
}

SWI_DECLARE( calloc, calloc );
void * __swi_calloc( size_t nmemb, size_t size ) {
  return cyg_memprot_calloc( nmemb, size, GET_LR());
}

SWI_DECLARE( realloc, realloc );
void * __swi_realloc( void *ptr, size_t size ) {
  return cyg_memprot_realloc( ptr, size, GET_LR());
}

#undef  SWI_DECLARE
#define SWI_DECLARE(_syscall_,_gate_) \
void __swi_##_syscall_ (void) __attribute__ ((naked)); \
void __swi_##_syscall_ (void) { asm( "swi\n\t.word " #_gate_ "\n"); }

#endif

// does any math function require system privileges ?

// ===========================================================================
// cyg_memprot_xxx calls
// ===========================================================================

SWI_DECLARE( cyg_memprot_thread_fail, cyg_memprot_thread_fail);
SWI_DECLARE( cyg_memprot_get_current_domain, cyg_memprot_get_current_domain);
SWI_DECLARE( cyg_memprot_sbrk, cyg_memprot_sbrk);

// ===========================================================================
// pthread_xxx calls
// ===========================================================================

SWI_DECLARE( pthread_attr_destroy, pthread_attr_destroy );
SWI_DECLARE( pthread_attr_getdetachstate, pthread_attr_getdetachstate );
SWI_DECLARE( pthread_attr_getinheritsched, pthread_attr_getinheritsched );
SWI_DECLARE( pthread_attr_getschedparam, pthread_attr_getschedparam );
SWI_DECLARE( pthread_attr_getschedpolicy, pthread_attr_getschedpolicy );
SWI_DECLARE( pthread_attr_getscope, pthread_attr_getscope );
SWI_DECLARE( pthread_attr_getstackaddr, pthread_attr_getstackaddr );
SWI_DECLARE( pthread_attr_getstacksize, pthread_attr_getstacksize );
SWI_DECLARE( pthread_attr_init, pthread_attr_init );
SWI_DECLARE( pthread_attr_setdetachstate, pthread_attr_setdetachstate );
SWI_DECLARE( pthread_attr_setinheritsched, pthread_attr_setinheritsched );
SWI_DECLARE( pthread_attr_setschedparam, pthread_attr_setschedparam );
SWI_DECLARE( pthread_attr_setschedpolicy, pthread_attr_setschedpolicy );
SWI_DECLARE( pthread_attr_setscope, pthread_attr_setscope );
SWI_DECLARE( pthread_attr_setstackaddr, pthread_attr_setstackaddr );
SWI_DECLARE( pthread_attr_setstacksize, pthread_attr_setstacksize );
SWI_DECLARE( pthread_cancel, pthread_cancel );
SWI_DECLARE( pthread_cleanup_pop_inner, pthread_cleanup_pop_inner );
SWI_DECLARE( pthread_cleanup_push_inner, pthread_cleanup_push_inner );
SWI_DECLARE( pthread_cond_broadcast, pthread_cond_broadcast );
SWI_DECLARE( pthread_cond_destroy, pthread_cond_destroy );
SWI_DECLARE( pthread_cond_init, pthread_cond_init );
SWI_DECLARE( pthread_cond_signal, pthread_cond_signal );
SWI_DECLARE( pthread_cond_timedwait, pthread_cond_timedwait );
SWI_DECLARE( pthread_cond_wait, pthread_cond_wait );
SWI_DECLARE( pthread_condattr_destroy, pthread_condattr_destroy );
SWI_DECLARE( pthread_condattr_init, pthread_condattr_init );

// we redefine pthread_create
SWI_DECLARE( pthread_create, cyg_memprot_pthread_create );

SWI_DECLARE( pthread_detach, pthread_detach );
SWI_DECLARE( pthread_equal, pthread_equal );
SWI_DECLARE( pthread_exit, pthread_exit );
SWI_DECLARE( pthread_getschedparam, pthread_getschedparam );
SWI_DECLARE( pthread_getspecific, pthread_getspecific );
SWI_DECLARE( pthread_join, pthread_join );
SWI_DECLARE( pthread_key_create, pthread_key_create );
SWI_DECLARE( pthread_key_delete, pthread_key_delete );
SWI_DECLARE( pthread_measure_stack_usage, pthread_measure_stack_usage );
SWI_DECLARE( pthread_mutex_destroy, pthread_mutex_destroy );
SWI_DECLARE( pthread_mutex_getprioceiling, pthread_mutex_getprioceiling );
SWI_DECLARE( pthread_mutex_init, pthread_mutex_init );
SWI_DECLARE( pthread_mutex_lock, pthread_mutex_lock );
SWI_DECLARE( pthread_mutex_setprioceiling, pthread_mutex_setprioceiling );
SWI_DECLARE( pthread_mutex_trylock, pthread_mutex_trylock );
SWI_DECLARE( pthread_mutex_unlock, pthread_mutex_unlock );
SWI_DECLARE( pthread_mutexattr_destroy, pthread_mutexattr_destroy );
SWI_DECLARE(pthread_mutexattr_getprioceiling,pthread_mutexattr_getprioceiling);
SWI_DECLARE( pthread_mutexattr_getprotocol, pthread_mutexattr_getprotocol );
SWI_DECLARE( pthread_mutexattr_init, pthread_mutexattr_init );
SWI_DECLARE(pthread_mutexattr_setprioceiling,pthread_mutexattr_setprioceiling);
SWI_DECLARE( pthread_mutexattr_setprotocol, pthread_mutexattr_setprotocol );
SWI_DECLARE( pthread_once, pthread_once );
SWI_DECLARE( pthread_self, pthread_self );
SWI_DECLARE( pthread_setcancelstate, pthread_setcancelstate );
SWI_DECLARE( pthread_setcanceltype, pthread_setcanceltype );
SWI_DECLARE( pthread_setschedparam, pthread_setschedparam );
SWI_DECLARE( pthread_setspecific, pthread_setspecific );
SWI_DECLARE( pthread_testcancel, pthread_testcancel );

// ===========================================================================
// Extra system calls
// ===========================================================================

// Add your own system calls here

SWI_DECLARE( diag_printf, diag_printf );
SWI_DECLARE( diag_write_string, diag_write_string );
SWI_DECLARE( cyg_scheduler_lock, cyg_scheduler_lock );
SWI_DECLARE( cyg_scheduler_unlock, cyg_scheduler_unlock );

SWI_DECLARE( camif_configure, camif_configure );
SWI_DECLARE( camif_start_capture, camif_start_capture );
SWI_DECLARE( camif_stop_capture, camif_stop_capture );
SWI_DECLARE( camif_next_frame, camif_next_frame );

SWI_DECLARE( aai_p4mode_setup, aai_p4mode_setup );
SWI_DECLARE( aai_p4mode_run, aai_p4mode_run );
SWI_DECLARE( aai_p4mode_run2, aai_p4mode_run2 );
SWI_DECLARE( aai_p4mode_stop, aai_p4mode_stop );
SWI_DECLARE( aai_p4mode_shutdown, aai_p4mode_shutdown );
SWI_DECLARE( aai_p4mode_run_n, aai_p4mode_run_n );

SWI_DECLARE( vai_buffer_setup, vai_buffer_setup );
SWI_DECLARE( vai_p4mode_setup, vai_p4mode_setup );
SWI_DECLARE( vai_p4mode_run, vai_p4mode_run );
SWI_DECLARE( vai_p4mode_stop, vai_p4mode_stop );


// ParrotOS system calls
// FIXME: no check is done on arguments yet

// Re-implemented calls
SWI_DECLARE( sup_start_module_stack, cyg_memprot_sup_start_module_stack );
SWI_DECLARE( sup_init_modules, cyg_memprot_sup_init_modules );
SWI_DECLARE( UART_Create_Task, cyg_memprot_UART_Create_Task );
// this function is used to regain system privileges on procedure termination
SWI_DECLARE( memprot_system, cyg_memprot_system );
// These calls are not re-implemented:
SWI_DECLARE( sup_init, sup_init );
SWI_DECLARE( sup_send_message_now, sup_send_message_now );
SWI_DECLARE( sup_stop_module, sup_stop_module );
SWI_DECLARE( sup_get_thread_handle, sup_get_thread_handle );
SWI_DECLARE( sup_undo_message, sup_undo_message );
SWI_DECLARE( sup_post_message, sup_post_message );
SWI_DECLARE( sup_get_message, sup_get_message );
SWI_DECLARE( har_read_date, har_read_date );
SWI_DECLARE( har_set_date, har_set_date );
SWI_DECLARE( sup_Disable_DSR, sup_Disable_DSR );
SWI_DECLARE( sup_Enable_DSR, sup_Enable_DSR );
SWI_DECLARE( sup_Set_DSR_Request, sup_Set_DSR_Request );
SWI_DECLARE( sup_p4mutex_lock, sup_p4mutex_lock );
SWI_DECLARE( sup_p4mutex_unlock, sup_p4mutex_unlock );
SWI_DECLARE( sup_watchdog, sup_watchdog );

SWI_DECLARE( sup_init_timers, sup_init_timers );
SWI_DECLARE( sup_set_timer, sup_set_timer );
SWI_DECLARE( sup_clear_timer, sup_clear_timer );
SWI_DECLARE( sup_clear_timers_by_callback, sup_clear_timers_by_callback );
SWI_DECLARE( sup_clear_all_timers, sup_clear_all_timers );

SWI_DECLARE( UART_Init, UART_Init );
SWI_DECLARE( UART_Read, UART_Read );
SWI_DECLARE( UART_Send, UART_Send );
SWI_DECLARE( UART_Start_Receive, UART_Start_Receive );
SWI_DECLARE( UART_Stop_Receive, UART_Stop_Receive );
SWI_DECLARE( UART_Start_Transmit, UART_Start_Transmit );
SWI_DECLARE( UART_Change_Speed, UART_Change_Speed );
SWI_DECLARE( UART_Flush, UART_Flush );

SWI_DECLARE( UARTRECORDER_Init, UARTRECORDER_Init );
SWI_DECLARE( RT_Post, RT_Post );
SWI_DECLARE( RT_Post_App_Msg, RT_Post_App_Msg );
SWI_DECLARE( RT_Post_CStr, RT_Post_CStr );
SWI_DECLARE( RT_Post_Str, RT_Post_Str );
SWI_DECLARE( RT_Post_Pack2, RT_Post_Pack2 );
SWI_DECLARE( RT_Post_Int, RT_Post_Int );
SWI_DECLARE( RT_Post_MultInt, RT_Post_MultInt );
SWI_DECLARE( RT_Post_Error, RT_Post_Error );
SWI_DECLARE( RT_Post_Debug_Error, RT_Post_Debug_Error );
SWI_DECLARE( RT_Post_Debug_ReadMem, RT_Post_Debug_ReadMem );
SWI_DECLARE( RT_Post_Debug_FunctionResult, RT_Post_Debug_FunctionResult );
SWI_DECLARE( RT_Post_Debug_RTTrace, RT_Post_Debug_RTTrace );
SWI_DECLARE( RT_Post_Printf, RT_Post_Printf );
SWI_DECLARE( RT_Post_Dialog, RT_Post_Dialog );
SWI_DECLARE( RT_Post_MultU8, RT_Post_MultU8 );
SWI_DECLARE( RT_Post_MultU16, RT_Post_MultU16 );
SWI_DECLARE( RT_Post_MultU32, RT_Post_MultU32 );
SWI_DECLARE( RT_Post_U8, RT_Post_U8 );
SWI_DECLARE( RT_Post_U16, RT_Post_U16 );
SWI_DECLARE( RT_Post_U32, RT_Post_U32 );
SWI_DECLARE( UARTRT_Init, UARTRT_Init );
SWI_DECLARE( uart_rt_send_message, uart_rt_send_message );
SWI_DECLARE( RT_Blues_Handle_Incoming, RT_Blues_Handle_Incoming );
SWI_DECLARE( RT_Tango_Handle_Incoming, RT_Tango_Handle_Incoming );
SWI_DECLARE( RT_Appli_Handle_Incoming, RT_Appli_Handle_Incoming );
SWI_DECLARE( uart_rt_flush, uart_rt_flush );

// Baseband system calls
SWI_DECLARE( Baseband_init, Baseband_init );
SWI_DECLARE( Baseband_start, Baseband_start );
SWI_DECLARE( Baseband_reset, Baseband_reset );
SWI_DECLARE( Baseband_initBDAddr, Baseband_initBDAddr );
SWI_DECLARE( Baseband_initVcoTrim, Baseband_initVcoTrim );

// Flash system calls
SWI_DECLARE( flash_init, flash_init );
SWI_DECLARE( flash_erase, flash_erase );
SWI_DECLARE( flash_program, flash_program );
SWI_DECLARE( flash_read, flash_read );
SWI_DECLARE( flash_dev_query, flash_dev_query );
SWI_DECLARE( flash_lock, flash_lock );
SWI_DECLARE( flash_unlock, flash_unlock );
SWI_DECLARE( flash_verify_addr, flash_verify_addr );
SWI_DECLARE( flash_get_limits, flash_get_limits );
SWI_DECLARE( flash_get_block_info, flash_get_block_info );
SWI_DECLARE( flash_code_overlaps, flash_code_overlaps );
SWI_DECLARE( flash_errmsg, flash_errmsg );

// PWM for P5
SWI_DECLARE( pwm_start, pwm_start );
SWI_DECLARE( pwm_stop, pwm_stop );
SWI_DECLARE( pwm_set_width, pwm_set_width );
SWI_DECLARE( pwm_set_freq, pwm_set_freq );

// BNEP
SWI_DECLARE( cyg_bnep_data_available, cyg_bnep_data_available);
SWI_DECLARE( cyg_bnep_set_sender, cyg_bnep_set_sender);

// GPIO & buttons
SWI_DECLARE( button_register, button_register );
SWI_DECLARE( gpio_config, gpio_config );
SWI_DECLARE( gpio_read, gpio_read );
SWI_DECLARE( gpio_output_clear, gpio_output_clear );
SWI_DECLARE( gpio_output_set, gpio_output_set );

// Low Power

SWI_DECLARE( parrot5_lpo_event_unregister, parrot5_lpo_event_unregister);
SWI_DECLARE( parrot5_lpo_event_register, parrot5_lpo_event_register);

// CPULoad
SWI_DECLARE( cyg_cpuload_calibrate, cyg_cpuload_calibrate );
SWI_DECLARE( cyg_cpuload_create, cyg_cpuload_create );
SWI_DECLARE( cyg_cpuload_delete, cyg_cpuload_delete );
SWI_DECLARE( cyg_cpuload_get, cyg_cpuload_get );

// Queue
SWI_DECLARE( cyg_queue_init, cyg_queue_init );
SWI_DECLARE( cyg_queue_destroy, cyg_queue_destroy );
SWI_DECLARE( cyg_queue_peek, cyg_queue_peek );
SWI_DECLARE( cyg_queue_put, cyg_queue_put );
SWI_DECLARE( cyg_queue_get, cyg_queue_get );
SWI_DECLARE( cyg_queue_tryput, cyg_queue_tryput );
SWI_DECLARE( cyg_queue_tryget, cyg_queue_tryget );
SWI_DECLARE( cyg_queue_timed_put, cyg_queue_timed_put );
SWI_DECLARE( cyg_queue_timed_get, cyg_queue_timed_get );

// Hardware timer
SWI_DECLARE( p5dev_hw_alarm_set, p5dev_hw_alarm_set );
SWI_DECLARE( p5dev_hw_alarm_clear, p5dev_hw_alarm_clear );

// FTP
SWI_DECLARE( ftp_get, ftp_get );
SWI_DECLARE( ftp_put, ftp_put );

// Clock
SWI_DECLARE( clock_us, clock_us);

// MMU
SWI_DECLARE( arm_mmu_v2p, arm_mmu_v2p);
SWI_DECLARE( arm_mmu_flush_dcache, arm_mmu_flush_dcache );

// CV
SWI_DECLARE( funct_cv_compute, funct_cv_compute );

// AMPLI STA326
SWI_DECLARE( STA326_Init, STA326_Init );
SWI_DECLARE( STA326_Mute, STA326_Mute );
SWI_DECLARE( STA326_UnMute, STA326_UnMute );
SWI_DECLARE( STA326_Write, STA326_Write );
SWI_DECLARE( STA326_Read, STA326_Read );



SWI_DECLARE( RT_Post_Color, RT_Post_Color );
SWI_DECLARE( RT_Post_Color_App_Msg, RT_Post_Color_App_Msg );
SWI_DECLARE( RT_Post_Color_CStr, RT_Post_Color_CStr );
SWI_DECLARE( RT_Post_Color_Str, RT_Post_Color_Str );
SWI_DECLARE( RT_Post_Color_Pack2, RT_Post_Color_Pack2 );
SWI_DECLARE( RT_Post_Color_Int, RT_Post_Color_Int );
SWI_DECLARE( RT_Post_Color_MultInt, RT_Post_Color_MultInt );
SWI_DECLARE( RT_Post_Color_Error, RT_Post_Color_Error );
SWI_DECLARE( RT_Post_Color_Debug_Error, RT_Post_Color_Debug_Error );
SWI_DECLARE( RT_Post_Color_Debug_ReadMem, RT_Post_Color_Debug_ReadMem );
SWI_DECLARE( RT_Post_Color_Debug_FunctionResult, RT_Post_Color_Debug_FunctionResult );
SWI_DECLARE( RT_Post_Color_Debug_RTTrace, RT_Post_Color_Debug_RTTrace );
SWI_DECLARE( RT_Post_Color_Printf, RT_Post_Color_Printf );
SWI_DECLARE( RT_Post_Color_Dialog, RT_Post_Color_Dialog );
SWI_DECLARE( RT_Post_Color_MultU8, RT_Post_Color_MultU8 );
SWI_DECLARE( RT_Post_Color_MultU16, RT_Post_Color_MultU16 );
SWI_DECLARE( RT_Post_Color_MultU32, RT_Post_Color_MultU32 );
SWI_DECLARE( RT_Post_Color_U8, RT_Post_Color_U8 );
SWI_DECLARE( RT_Post_Color_U16, RT_Post_Color_U16 );
SWI_DECLARE( RT_Post_Color_U32, RT_Post_Color_U32 );

// others ?
