//==========================================================================
//
//      network_support.c
//
//      Misc network support functions
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2003 Andrew Lunn <andrew.lunn@ascom.ch>   
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: gthomas, sorin@netappi.com ("Sorin Babeanu"), hmt, jlarmour,
//               andrew.lunn@ascom.ch
// Date:         2000-01-10
// Purpose:      
// Description:  
//              
//
//####DESCRIPTIONEND####
//
//==========================================================================

// BOOTP support

#include <pkgconf/net.h>
#undef _KERNEL
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>

#include <net/if.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <net/route.h>

#include <cyg/infra/diag.h>
#include <cyg/kernel/kapi.h>

#include <stdio.h>    // for 'sprintf()'

#include <bootp.h>
#include <network.h>
#include <arpa/inet.h>

#ifdef CYGPKG_IO_PCMCIA
#include <cyg/io/eth/netdev.h>
#endif

#ifdef CYGPKG_NET_DHCP
#include <dhcp.h>
#endif

#include <zcip.h>

#ifdef CYGPKG_NS_DNS
#include <pkgconf/ns_dns.h>
#endif

#include <pkgconf/io_eth_drivers.h> // module configury; SIMULATED_FAILURES
#include <pkgconf/net.h>            // CYGPKG_NET_FAST_THREAD_TICKLE_DEVS?

#ifdef CYGHWR_NET_DRIVER_WL0 
#ifdef CYGPKG_DEVS_80211_BROADCOM_BCM4326
extern void	bcm4326_wait_device_ready(void);
#endif
#ifdef CYGPKG_DEVS_80211_BROADCOM_BCM43XX
extern void	bcm43xx_wait_device_ready(void);
#endif
#endif

#ifdef CYGHWR_NET_DRIVER_WL0 
struct bootp wl0_bootp_data;
cyg_bool_t   wl0_up = false;
const char   wl0_name[] = "wl0";
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
struct bootp eth0_bootp_data;
cyg_bool_t   eth0_up = false;
const char   eth0_name[] = "eth0";
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
struct bootp eth1_bootp_data;
cyg_bool_t   eth1_up = false;
const char   eth1_name[] = "eth1";
#endif
#ifdef CYGOPT_NET_DRIVER_BNEP0
struct bootp bnep0_bootp_data;
cyg_bool_t   bnep0_up = false;
const char   bnep0_name[] = "bnep0";
#endif


/// DFG (30/05/2007): Add a function to initialise only ONE interface
const struct iftable_opt_S {
    const char *name;
    cyg_bool_t *up;
    struct bootp *bootp_data;
    cyg_uint8 *dhcpstate;
    struct dhcp_lease *lease;
} iftable[] = {
#ifdef CYGHWR_NET_DRIVER_WL0 
    { wl0_name, &wl0_up, &wl0_bootp_data, &wl0_dhcpstate, &wl0_lease },
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0 
    { eth0_name, &eth0_up, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease },
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1 
    { eth1_name, &eth1_up, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease },
#endif
#ifdef CYGHWR_NET_DRIVER_BNEP0 
    { bnep0_name, &bnep0_up, &bnep0_bootp_data, &bnep0_dhcpstate, &bnep0_lease },
#endif
    { NULL } // at least, one item in the table
};

// local variable used asa mutex
static volatile int in_init_all_network_interfaces = 0;

#define _string(s) #s
#define string(s) _string(s)

#ifndef CYGPKG_LIBC_STDIO
#define perror(s) NETWORK_PRINTF(#s ": %s\n", strerror(errno))
#endif

#ifdef CYGPKG_NET_NLOOP
#if CYGPKG_NET_NLOOP > 0
//  
//   Initialize loopback interface  ----------   Added by sorin@netappi.com 
//
cyg_bool_t init_loopback_interface(int lo)
{
    struct sockaddr_in *addrp;
    struct ifreq ifr;
    int s;
    int one = 1;
    struct ecos_rtentry route;
    struct in_addr netmask, gateway;

    s = socket(AF_INET, SOCK_DGRAM, 0); 
    if (s < 0) {
        perror("socket");
        return false;
    }
    if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one))) {
        perror("setsockopt");
	    close(s);
        return false;
    }

    addrp = (struct sockaddr_in *) &ifr.ifr_addr;
    memset(addrp, 0, sizeof(*addrp));
    addrp->sin_family = AF_INET;
    addrp->sin_len = sizeof(*addrp);
    addrp->sin_port = 0;
    // Make an address 127.0.<lo>.1 to manage multiple loopback ifs.
    // (There is normally only 1, so it's the standard 127.0.0.1)
    addrp->sin_addr.s_addr = htonl((0x100 * lo) + INADDR_LOOPBACK) ; 

#if CYGPKG_NET_NLOOP > 1
    // Init the one we were told to
    sprintf(ifr.ifr_name, "lo%d", lo);
#else
    strcpy(ifr.ifr_name, "lo0");
#endif    

    if (ioctl(s, SIOCSIFADDR, &ifr)) {
        perror("SIOCIFADDR");
        close(s);
        return false;
    }
    
#if 1 < CYGPKG_NET_NLOOP
    // We cheat to make different nets for multiple loopback devs
    addrp->sin_addr.s_addr = netmask.s_addr = htonl(IN_CLASSC_NET);
#else
    // 
    addrp->sin_addr.s_addr = netmask.s_addr = htonl(IN_CLASSA_NET);
#endif
    if (ioctl(s, SIOCSIFNETMASK, &ifr)) {
        perror("SIOCSIFNETMASK");
        close(s);
        return false;
    }
    ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
    if (ioctl(s, SIOCSIFFLAGS, &ifr)) {
        perror("SIOCSIFFLAGS");
        close(s);
        return false;
    }

    gateway.s_addr = htonl(INADDR_LOOPBACK);  
    memset(&route, 0, sizeof(route));
    addrp->sin_family = AF_INET;
    addrp->sin_len = sizeof(*addrp);
    addrp->sin_port = 0;
    addrp->sin_addr.s_addr = htonl((0x100 * lo) + INADDR_LOOPBACK) & netmask.s_addr;
    memcpy(&route.rt_dst, addrp, sizeof(*addrp));
    addrp->sin_addr = netmask;
    memcpy(&route.rt_genmask, addrp, sizeof(*addrp));
    addrp->sin_addr = gateway;
    memcpy(&route.rt_gateway, addrp, sizeof(*addrp));
    
    route.rt_dev = ifr.ifr_name;
    route.rt_flags = RTF_UP|RTF_GATEWAY;
    route.rt_metric = 0;

    if (ioctl(s, SIOCADDRT, &route)) {
        NETWORK_PRINTF("Route - dst: %s", inet_ntoa(((struct sockaddr_in *)&route.rt_dst)->sin_addr));
        NETWORK_PRINTF(", mask: %s", inet_ntoa(((struct sockaddr_in *)&route.rt_genmask)->sin_addr));
        NETWORK_PRINTF(", gateway: %s\n", inet_ntoa(((struct sockaddr_in *)&route.rt_gateway)->sin_addr));
        if (errno != EEXIST) {
            perror("SIOCADDRT 3");
            close(s);
            return false;
        }
    }
    close(s);
    return true;
}
#endif
#endif


//
// Internal function which builds up a fake BOOTP database for
// an interface.
//

static unsigned char *
add_tag(unsigned char *vp,
        unsigned char tag,
        void *val,
        int len)
{
    int i;
    unsigned char *xp = (unsigned char *)val;
    *vp++ = tag;
    *vp++ = len;
    for (i = 0;  i < len;  i++) {
        *vp++ = *xp++;
    }
    return vp;
}

void
build_bootp_record(struct bootp *bp,
                   const char *if_name,
                   const char *addrs_ip,
                   const char *addrs_netmask,
                   const char *addrs_broadcast,
                   const char *addrs_gateway,
                   const char *addrs_server)
{
    int i, s;
    in_addr_t addr;
    unsigned char *vp;
    unsigned char cookie[] = VM_RFC1048;
    struct ifreq ifr;

    bzero(bp, sizeof(struct bootp));
    bp->bp_op = BOOTREPLY;
    bp->bp_htype = HTYPE_ETHERNET;
    bp->bp_hlen = 6;

    // Query the hardware address
    for (i = 0;  i < bp->bp_hlen;  i++) {
        bp->bp_chaddr[i] = 0xFF;  // Unknown
    }
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s >= 0) {
        strcpy(ifr.ifr_name, if_name);
        if (ioctl(s, SIOCGIFHWADDR, &ifr) >= 0) {
            bcopy(ifr.ifr_hwaddr.sa_data, bp->bp_chaddr, bp->bp_hlen);
        }
        close(s);
    }

    // Fill in the provided IP addresses
    bp->bp_ciaddr.s_addr = inet_addr(addrs_ip);
    bp->bp_yiaddr.s_addr = inet_addr(addrs_ip);
    bp->bp_siaddr.s_addr = inet_addr(addrs_server);
    bp->bp_giaddr.s_addr = inet_addr(addrs_gateway);
    vp = &bp->bp_vend[0];
    bcopy(&cookie, vp, sizeof(cookie));
    vp += sizeof(cookie);
    addr = inet_addr(addrs_netmask);
    vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
    addr = inet_addr(addrs_broadcast);
    vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
    addr = inet_addr(addrs_gateway);
    if (addr != INADDR_NONE) // we don't have a gateway
	vp = add_tag(vp, TAG_GATEWAY, &addr, sizeof(in_addr_t));
    *vp = TAG_END;
}


//
// Enable autoip
//
#define ADDR_SIZE 16
typedef struct {
	char	ip[ADDR_SIZE];
	char	netmask[ADDR_SIZE];
	char	broadcast[ADDR_SIZE];
	char	gateway[ADDR_SIZE];
	char	server[ADDR_SIZE];
} ifconfig_addrs;
ifconfig_addrs customip_addrs;
cyg_bool_t EnableCustomIP = false;

void set_customip_addrs(char* ip_addr, char* netmask_addr, char* broadcast_addr, 
					  char* gateway_addr, char* server_addr)
{
    if (ip_addr == NULL) return;

	strncpy(customip_addrs.ip, ip_addr, ADDR_SIZE-1);
	//customip_addrs.ip[ADDR_SIZE-1] = '\0';

	if (netmask_addr) strncpy(customip_addrs.netmask ,netmask_addr,ADDR_SIZE-1);
	//customip_addrs.netmask[ADDR_SIZE-1] = '\0';

	if (broadcast_addr) strncpy(customip_addrs.broadcast, broadcast_addr,ADDR_SIZE-1);
	//customip_addrs.broadcast[ADDR_SIZE-1] = '\0';

	if (gateway_addr) strncpy(customip_addrs.gateway, gateway_addr,ADDR_SIZE-1);
	//customip_addrs.gateway[ADDR_SIZE-1] = '\0';

	if (server_addr) strncpy(customip_addrs.server, server_addr,ADDR_SIZE-1);
	//customip_addrs.server[ADDR_SIZE-1] = '\0';

	EnableCustomIP = true;
}
void reset_customip_addrs( void )
{
	EnableCustomIP = false;
}


//
// Initialize network services[s] (Loopback/DHCP Thread/DNS)
//
void
init_all_network_services(void)
{
  static int started = 0;
  if (started == 0) {
      //started++; //DO IT ALL TIME BECAUSE ROUTE IS CLEARED BY dhcp_helt()

#ifdef CYGPKG_NET_NLOOP
#if 0 < CYGPKG_NET_NLOOP
      {
          //static 
		  int loop_init = 0;
          int i;
          if ( 0 == loop_init++ )
              for ( i = 0; i < CYGPKG_NET_NLOOP; i++ )
                  init_loopback_interface( i );
      }
#endif
#endif

#ifdef CYGDAT_NS_DNS_DEFAULT_SERVER
      {
          cyg_dns_res_start(CYGDAT_NS_DNS_DEFAULT_SERVER);
      }
#endif

#ifdef CYGDAT_NS_DNS_DOMAINNAME_NAME
#define _NAME string(CYGDAT_NS_DNS_DOMAINNAME_NAME)
      {
          const char buf[] = _NAME;
          int len = strlen(_NAME);
      
          setdomainname(buf,len);
      }
#endif
  }
}


//
// Initialize network interface[s] using BOOTP/DHCP
//
void
init_all_network_interfaces(void)
{
#ifdef CYGPKG_IO_PCMCIA
    cyg_netdevtab_entry_t *t;
#endif // CYGPKG_IO_PCMCIA
#ifdef CYGOPT_NET_IPV6_ROUTING_THREAD
    int rs_wait = 40;
#endif

    cyg_scheduler_lock();
    while ( in_init_all_network_interfaces ) {
        // Another thread is doing this...
        cyg_scheduler_unlock();
        cyg_thread_delay( 10 );
        cyg_scheduler_lock();
    }
    in_init_all_network_interfaces = 1;
    cyg_scheduler_unlock();

#ifdef CYGHWR_NET_DRIVER_WL0 
#ifdef CYGPKG_DEVS_80211_BROADCOM_BCM4326
	bcm4326_wait_device_ready();
#endif
#ifdef CYGPKG_DEVS_80211_BROADCOM_BCM43XX
	//bcm43xx_wait_device_ready();
#endif

    if ( ! wl0_up ) { // Make this call idempotent
#ifdef CYGHWR_NET_DRIVER_WL0_BOOTP
        // Perform a complete initialization, using BOOTP/DHCP
        wl0_up = true;
		if (EnableCustomIP == true){
			NETWORK_PRINTF("CUSTOMIP[%s] configuring...\n",wl0_name);
				build_bootp_record(&wl0_bootp_data,
									wl0_name,
									(const char*)customip_addrs.ip,
									(const char*)customip_addrs.netmask,
									(const char*)customip_addrs.broadcast,
									(const char*)customip_addrs.gateway,
									(const char*)customip_addrs.server);
				show_bootp(wl0_name, &wl0_bootp_data);
				EnableCustomIP = false;
		}
		else {
#ifdef CYGHWR_NET_DRIVER_WL0_DHCP
        wl0_dhcpstate = 0; // Says that initialization is external to dhcp
        if (do_dhcp(wl0_name, &wl0_bootp_data, &wl0_dhcpstate, &wl0_lease)) 
#else
#ifdef CYGPKG_NET_DHCP
        wl0_dhcpstate = DHCPSTATE_BOOTP_FALLBACK;
        // so the dhcp machine does no harm if called
#endif
        if (do_bootp(wl0_name, &wl0_bootp_data)) 
#endif
        {
#ifdef CYGHWR_NET_DRIVER_WL0_BOOTP_SHOW
            show_bootp(wl0_name, &wl0_bootp_data);
#endif
        } else {
            NETWORK_PRINTF("BOOTP/DHCP failed on wl0\n");
	     wl0_up = false;
        }
		} // EnableCustomIP == false

#elif defined(CYGHWR_NET_DRIVER_WL0_ADDRS_IP)
        wl0_up = true;
        build_bootp_record(&wl0_bootp_data,
                           wl0_name,
                           string(CYGHWR_NET_DRIVER_WL0_ADDRS_IP),
                           string(CYGHWR_NET_DRIVER_WL0_ADDRS_NETMASK),
                           string(CYGHWR_NET_DRIVER_WL0_ADDRS_BROADCAST),
                           string(CYGHWR_NET_DRIVER_WL0_ADDRS_GATEWAY),
                           string(CYGHWR_NET_DRIVER_WL0_ADDRS_SERVER));
        show_bootp(wl0_name, &wl0_bootp_data);
#endif
    }
#endif // CYGHWR_NET_DRIVER_WL0
#ifdef CYGHWR_NET_DRIVER_ETH0
    if ( ! eth0_up ) { // Make this call idempotent
#ifdef CYGPKG_IO_PCMCIA
        if ((t = eth_drv_netdev("eth0")) != (cyg_netdevtab_entry_t *)NULL) {
            int tries = 0;
            while (t->status != CYG_NETDEVTAB_STATUS_AVAIL) {
                if (tries == 0) {
                    NETWORK_PRINTF("... Waiting for PCMCIA device 'eth0'\n");
                } 
                if (++tries == 5) {
                    NETWORK_PRINTF("... Giving up on PCMCIA device 'eth0'\n");
                    goto bail_eth0;
                }
                cyg_thread_delay(100);
            }
        }
#endif // CYGPKG_IO_PCMCIA
#ifdef CYGHWR_NET_DRIVER_ETH0_BOOTP
        // Perform a complete initialization, using BOOTP/DHCP
        eth0_up = true;
#ifdef CYGHWR_NET_DRIVER_ETH0_DHCP
        eth0_dhcpstate = 0; // Says that initialization is external to dhcp
        if (do_dhcp(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease)) 
#else
#ifdef CYGPKG_NET_DHCP
        eth0_dhcpstate = DHCPSTATE_BOOTP_FALLBACK;
        // so the dhcp machine does no harm if called
#endif
        if (do_bootp(eth0_name, &eth0_bootp_data)) 
#endif
        {
#ifdef CYGHWR_NET_DRIVER_ETH0_BOOTP_SHOW
            show_bootp(eth0_name, &eth0_bootp_data);
#endif
        } else {
            NETWORK_PRINTF("BOOTP/DHCP failed on eth0\n");
            eth0_up = false;
        }
#elif defined(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP)
        eth0_up = true;
        build_bootp_record(&eth0_bootp_data,
                           eth0_name,
                           string(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP),
                           string(CYGHWR_NET_DRIVER_ETH0_ADDRS_NETMASK),
                           string(CYGHWR_NET_DRIVER_ETH0_ADDRS_BROADCAST),
                           string(CYGHWR_NET_DRIVER_ETH0_ADDRS_GATEWAY),
                           string(CYGHWR_NET_DRIVER_ETH0_ADDRS_SERVER));
        show_bootp(eth0_name, &eth0_bootp_data);
#endif
#ifdef CYGPKG_IO_PCMCIA
    bail_eth0:
#endif
    }
#endif // CYGHWR_NET_DRIVER_ETH0
#ifdef CYGHWR_NET_DRIVER_ETH1
    if ( ! eth1_up ) { // Make this call idempotent
#ifdef CYGPKG_IO_PCMCIA
        if ((t = eth_drv_netdev("eth1")) != (cyg_netdevtab_entry_t *)NULL) {
            int tries = 0;
            while (t->status != CYG_NETDEVTAB_STATUS_AVAIL) {
                if (tries == 0) {
                    NETWORK_PRINTF("... Waiting for PCMCIA device 'eth1'\n");
                } 
                if (++tries == 5) {
                    NETWORK_PRINTF("... Giving up on PCMCIA device 'eth1'\n");
                    goto bail_eth1;
                }
                cyg_thread_delay(100);
            }
        }
#endif // CYGPKG_IO_PCMCIA
#ifdef CYGHWR_NET_DRIVER_ETH1_BOOTP
        // Perform a complete initialization, using BOOTP/DHCP
        eth1_up = true;
#ifdef CYGHWR_NET_DRIVER_ETH1_DHCP
        eth1_dhcpstate = 0; // Says that initialization is external to dhcp
        if (do_dhcp(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease)) 
#else
#ifdef CYGPKG_NET_DHCP
        eth1_dhcpstate = DHCPSTATE_BOOTP_FALLBACK;
        // so the dhcp machine does no harm if called
#endif
        if (do_bootp(eth1_name, &eth1_bootp_data))
#endif
        {
#ifdef CYGHWR_NET_DRIVER_ETH1_BOOTP_SHOW
            show_bootp(eth1_name, &eth1_bootp_data);
#endif
        } else {
            NETWORK_PRINTF("BOOTP/DHCP failed on eth1\n");
            eth1_up = false;
        }
#elif defined(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP)
        eth1_up = true;
        build_bootp_record(&eth1_bootp_data,
                           eth1_name,
                           string(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP),
                           string(CYGHWR_NET_DRIVER_ETH1_ADDRS_NETMASK),
                           string(CYGHWR_NET_DRIVER_ETH1_ADDRS_BROADCAST),
                           string(CYGHWR_NET_DRIVER_ETH1_ADDRS_GATEWAY),
                           string(CYGHWR_NET_DRIVER_ETH1_ADDRS_SERVER));
        show_bootp(eth1_name, &eth1_bootp_data);
#endif
#ifdef CYGPKG_IO_PCMCIA
    bail_eth1:
#endif
    }
#endif // CYGHWR_NET_DRIVER_ETH1
#ifdef CYGOPT_NET_DRIVER_BNEP0
    if ( ! bnep0_up ) { // Make this call idempotent
#ifdef CYGOPT_NET_DRIVER_BNEP0_BOOTP
        // Perform a complete initialization, using BOOTP/DHCP
        bnep0_up = true;
#ifdef CYGOPT_NET_DRIVER_BNEP0_DHCP
        bnep0_dhcpstate = 0; // Says that initialization is external to dhcp
        if (do_dhcp(bnep0_name, &bnep0_bootp_data, &bnep0_dhcpstate, &bnep0_lease)) 
#else
#ifdef CYGPKG_NET_DHCP
        bnep0_dhcpstate = DHCPSTATE_BOOTP_FALLBACK;
        // so the dhcp machine does no harm if called
#endif
        if (do_bootp(bnep0_name, &bnep0_bootp_data)) 
#endif
        {
#ifdef CYGOPT_NET_DRIVER_BNEP0_BOOTP_SHOW
            show_bootp(bnep0_name, &bnep0_bootp_data);
#endif
        } else {
            NETWORK_PRINTF("BOOTP/DHCP failed on bnep0\n");
            bnep0_up = false;
        }
#elif defined(CYGOPT_NET_DRIVER_BNEP0_ADDRS_IP)
        bnep0_up = true;
        build_bootp_record(&bnep0_bootp_data,
                           bnep0_name,
                           string(CYGOPT_NET_DRIVER_BNEP0_ADDRS_IP),
                           string(CYGOPT_NET_DRIVER_BNEP0_ADDRS_NETMASK),
                           string(CYGOPT_NET_DRIVER_BNEP0_ADDRS_BROADCAST),
                           string(CYGOPT_NET_DRIVER_BNEP0_ADDRS_GATEWAY),
                           string(CYGOPT_NET_DRIVER_BNEP0_ADDRS_SERVER));
        show_bootp(bnep0_name, &bnep0_bootp_data);
#endif
#ifdef CYGPKG_IO_PCMCIA
    bail_bnep0:
#endif
    }
#endif // CYGOPT_NET_DRIVER_BNEP0
#ifdef CYGHWR_NET_DRIVER_WL0
#ifndef CYGHWR_NET_DRIVER_WL0_MANUAL
    if (wl0_up) {
        if (!init_net(wl0_name, &wl0_bootp_data)) {
            NETWORK_PRINTF("Network initialization failed for wl0\n");
            wl0_up = false;
        }
#ifdef CYGHWR_NET_DRIVER_WL0_IPV6_PREFIX
        if (!init_net_IPv6(wl0_name, &wl0_bootp_data, 
                           string(CYGHWR_NET_DRIVER_WL0_IPV6_PREFIX))) {
            NETWORK_PRINTF("Static IPv6 network initialization failed for wl0\n");
            wl0_up = false;  // ???
        }
#endif
    }
#endif
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
#ifndef CYGHWR_NET_DRIVER_ETH0_MANUAL
    if (eth0_up) {
        if (!init_net(eth0_name, &eth0_bootp_data)) {
            NETWORK_PRINTF("Network initialization failed for eth0\n");
            eth0_up = false;
        }
#ifdef CYGHWR_NET_DRIVER_ETH0_IPV6_PREFIX
        if (!init_net_IPv6(eth0_name, &eth0_bootp_data, 
                           string(CYGHWR_NET_DRIVER_ETH0_IPV6_PREFIX))) {
            NETWORK_PRINTF("Static IPv6 network initialization failed for eth0\n");
            eth0_up = false;  // ???
        }
#endif
    }
#endif
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
#ifndef CYGHWR_NET_DRIVER_ETH1_MANUAL
    if (eth1_up) {
        if (!init_net(eth1_name, &eth1_bootp_data)) {
            NETWORK_PRINTF("Network initialization failed for eth1\n");
            eth1_up = false;
        }
#ifdef CYGHWR_NET_DRIVER_ETH1_IPV6_PREFIX
        if (!init_net_IPv6(eth1_name, &eth1_bootp_data, 
                           string(CYGHWR_NET_DRIVER_ETH1_IPV6_PREFIX))) {
            NETWORK_PRINTF("Static IPv6 network initialization failed for eth1\n");
            eth1_up = false; // ???
        }
#endif
    }
#endif
#endif
#ifdef CYGOPT_NET_DRIVER_BNEP0
#ifndef CYGOPT_NET_DRIVER_BNEP0_MANUAL
    if (bnep0_up) {
        if (!init_net(bnep0_name, &bnep0_bootp_data)) {
            NETWORK_PRINTF("Network initialization failed for bnep0\n");
            bnep0_up = false;
        }
#ifdef CYGOPT_NET_DRIVER_BNEP0_IPV6_PREFIX
        if (!init_net_IPv6(bnep0_name, &bnep0_bootp_data, 
                           string(CYGOPT_NET_DRIVER_BNEP0_IPV6_PREFIX))) {
            NETWORK_PRINTF("Static IPv6 network initialization failed for bnep0\n");
            bnep0_up = false;  // ???
        }
#endif
    }
#endif
#endif

#ifdef CYGOPT_NET_DHCP_DHCP_THREAD
    dhcp_start_dhcp_mgt_thread();
#endif

#ifdef CYGOPT_NET_IPV6_ROUTING_THREAD
    ipv6_start_routing_thread();

    // Wait for router solicit process to happen.
    while (rs_wait-- && !cyg_net_get_ipv6_advrouter(NULL)) {
        cyg_thread_delay(10);
    }
    if (rs_wait == 0 ) {
        NETWORK_PRINTF("No router solicit received\n");
    } else {
        // Give Duplicate Address Detection time to work
        cyg_thread_delay(200);
    }
#endif

    // Init loopback and DNS services
    init_all_network_services();

    // Open the monitor to other threads.
    in_init_all_network_interfaces = 0;
}

//
// Initialize network interface[s] using BOOTP/DHCP
//
int 
init_one_network_interface(const char *name, struct if_params_S *param)
{
    int i, ret;
#ifdef CYGPKG_IO_PCMCIA
    cyg_netdevtab_entry_t *t;
#endif // CYGPKG_IO_PCMCIA
#ifdef CYGOPT_NET_IPV6_ROUTING_THREAD
    int rs_wait = 40;
#endif

    // Init loopback and DNS services, just in case not already called
    init_all_network_services();

    cyg_scheduler_lock();
    while ( in_init_all_network_interfaces ) {
        // Another thread is doing this...
        cyg_scheduler_unlock();
        cyg_thread_delay( 10 );
        cyg_scheduler_lock();
    }
    in_init_all_network_interfaces = 1;
    cyg_scheduler_unlock();


    // search in iftable

    for (i=0; iftable[i].name != NULL; i++)
        if (strcmp(name, iftable[i].name) == 0)
            break;

    if (iftable[i].name == NULL) {
        // specified interface name not found
        return -1;
    }
    // no else

    if ( ! *iftable[i].up ) { // bring device up
        if ( (param->options & IF_PARAMS_OPT_MASK) != IF_PARAMS_OPT_STATIC ) {
            // Perform a complete initialization, using BOOTP/DHCP
            *iftable[i].up = true;
            if ((param->options & IF_PARAMS_OPT_MASK) == IF_PARAMS_OPT_DHCP) {
                *iftable[i].dhcpstate = 0; // Says that initialization is external to dhcp
                ret = do_dhcp(iftable[i].name, iftable[i].bootp_data, iftable[i].dhcpstate, iftable[i].lease);
            }
            else {
#ifdef CYGPKG_NET_DHCP
#ifdef CYGHWR_NET_DRIVER_WL0
                wl0_dhcpstate = DHCPSTATE_BOOTP_FALLBACK;
                // so the dhcp machine does no harm if called
#endif
#endif
                ret = do_bootp(iftable[i].name, iftable[i].bootp_data);
            }
            if (ret) {
                if (param->options & IF_PARAMS_OPT_SHOW)
                    show_bootp(iftable[i].name, iftable[i].bootp_data);
            } 
            else {
                NETWORK_PRINTF("BOOTP/DHCP failed on %s\n", iftable[i].name);
                *iftable[i].up = false;
               
                // fallback to AutoIP
                    *iftable[i].up = true;
					if ( do_zcip(iftable[i].name, iftable[i].bootp_data, false) == 0){
                    show_bootp(iftable[i].name, iftable[i].bootp_data);
                }
				else
                    *iftable[i].up = false;
            }
        }
        else { // Static IP ou AutoIP direct
            if (param->options & IF_PARAMS_OPT_AUTOIP) {
                    *iftable[i].up = true;
					if ( do_zcip(iftable[i].name, iftable[i].bootp_data, false) == 0){
                    show_bootp(iftable[i].name, iftable[i].bootp_data);
                }
				else
                    *iftable[i].up = false;
            }
            else if (param->IP) {
                *iftable[i].up = true;
                build_bootp_record(iftable[i].bootp_data,
                                   iftable[i].name,
                                   param->IP,
                                   param->NETMASK,
                                   param->BROADCAST,
                                   param->GATEWAY,
                                   param->SERVER);
                show_bootp(iftable[i].name, iftable[i].bootp_data);
            }
        }
    }

    if (*iftable[i].up) {
        if (!init_net(iftable[i].name, iftable[i].bootp_data)) {
            NETWORK_PRINTF("Network initialization failed for %s\n", iftable[i].name);
            *iftable[i].up = false;
        }
#ifdef CYGPKG_NET_INET6
        if (iftable[i].IPV6_PREFIX) {
            if (!init_net_IPv6(iftable[i].name, iftable[i].bootp_data, iftable[i].IPV6_PREFIX)) {
                NETWORK_PRINTF("Static IPv6 network initialization failed for %s\n", iftable[i].name);
                *iftable[i].up = false;  // ???
            }
        }
#endif
    }

#ifdef CYGOPT_NET_DHCP_DHCP_THREAD
    dhcp_start_dhcp_mgt_thread();
#endif

#ifdef CYGOPT_NET_IPV6_ROUTING_THREAD
    ipv6_start_routing_thread();

    // Wait for router solicit process to happen.
    while (rs_wait-- && !cyg_net_get_ipv6_advrouter(NULL)) {
        cyg_thread_delay(10);
    }
    if (rs_wait == 0 ) {
        NETWORK_PRINTF("No router solicit received\n");
    } else {
        // Give Duplicate Address Detection time to work
        cyg_thread_delay(200);
    }
#endif

    if ( (param->options & IF_PARAMS_OPT_MASK) == IF_PARAMS_OPT_STATIC ) {
        // initialize using specified addresses
        if (param->DNS != NULL) {
            cyg_dns_res_start(param->DNS);
        }

        if (param->DOMAIN != NULL) {
            int len = strlen(param->DOMAIN);
            setdomainname(param->DOMAIN,len);
        }
    }
    else {
        // initialize DNS serveur using DHCP/BOOTP received address
        /// @todo Change do_dhcp() so it does that itself
        /// currently, dhcp doesn't save domainname nor domainname server address
    }

    // Inform about the final status of interface
    param->up = *iftable[i].up;

    // Open the monitor to other threads.
    in_init_all_network_interfaces = 0;

    return 0; // assume no error here
}

//
// Get network interface configuration
// User should take care of freeing all the strings that will be allocated in structure param
//
void
get_network_interface_config(const char *name, struct if_params_S *param)
{
    int i;
    char *addr;

    for (i=0; iftable[i].name != NULL; i++)
        if (strcmp(name, iftable[i].name) == 0){
            break;
	 }

    if (iftable[i].name == NULL) {
        // specified interface name not found
        return ;
    }

    param->options = -1;
    param->IPV6_PREFIX = NULL;

    addr = inet_ntoa(iftable[i].bootp_data->bp_yiaddr);
    param->IP = strdup(addr);

    param->NETMASK = NULL;
    param->BROADCAST = NULL;

    addr = inet_ntoa(iftable[i].bootp_data->bp_giaddr);
    param->GATEWAY = strdup(addr);

    addr = inet_ntoa(iftable[i].bootp_data->bp_siaddr);
    param->SERVER = strdup(addr);

    param->DOMAIN = NULL;
    param->DNS = NULL;

    param->MAC = strdup((const char *)iftable[i].bootp_data->bp_chaddr);

}

// EOF network_support.c
