//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  dlops.c
* @brief DLFCN custom operations
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2008-01-25
*******************************************************************************
*/

#include "common.h"
#include <dlfcn/dlfcn_setup.h>

static int dl_op_map(void **base, unsigned int size);
static int dl_op_unmap(void *base, unsigned int size);

/* Redefine dlfcn ops for USER mode operation */
static const struct dlfcn_ops ops = {
    .op_map    = dl_op_map,
    .op_unmap  = dl_op_unmap,
    .op_malloc = cyg_process_malloc,
    .op_free   = cyg_process_free,
    .op_imb    = cyg_process_imb,
    .op_open   = SYSCALL(open),
    .op_read   = SYSCALL(read),
    .op_lseek  = SYSCALL(lseek),
    .op_close  = SYSCALL(close),
};

static int dl_op_map(void **base, unsigned int size)
{
    int ret;
    cyg_process *self = cyg_process_self();

    *base = (void *)(self->rw.base + MMU_PAGE_SIZE);
    size = ALIGN_TOP(size, MMU_PAGE_SIZE);

    ret = SYSCALL(cyg_process_map)(self, (CYG_ADDRESS)(*base), size);

    return ret;
}

static int dl_op_unmap(void *base, unsigned int size)
{
    int ret;
    cyg_process *self = cyg_process_self();

    size = ALIGN_TOP(size, MMU_PAGE_SIZE);
    ret = SYSCALL(cyg_process_unmap)(self, (CYG_ADDRESS)base, size);

    return ret;
}

void dlfcn_init(void)
{
    dlfcn_setup(&ops);
}
