//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  dynamo.c
* @brief Dynamo package set-up
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2007-02-04
*******************************************************************************
*/

#include <string.h>
#include <errno.h>

#include <cyg/kernel/kapi.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/infra/diag.h>
#include <cyg/infra/cyg_ass.h>

#include <pkgconf/hal.h>
#include <pkgconf/hal_arm.h>

// get memory layout
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H

#include "common.h"

#define RAM_SIZE      ALIGN_BASE((CYGOPT_DYNAMO_RAM_SIZE << 10), MMU_PAGE_SIZE)
#define NB_RAM_PAGES  (RAM_SIZE/MMU_PAGE_SIZE)

#define VIRTUAL_TOP   (CYGMEM_REGION_virtual+CYGMEM_REGION_virtual_SIZE)
#define VM_BASE       ALIGN_TOP( CYGMEM_REGION_virtual, MMU_SECTION_SIZE )
#define VM_TOP        ALIGN_BASE( VIRTUAL_TOP, MMU_SECTION_SIZE )
#define VM_SECTS      ((VM_TOP-VM_BASE)/MMU_SECTION_SIZE)
#define VM_NB_32MB    (VM_SECTS/32)

//! A global buffer to which all virtual memory will be mapped
static cyg_uint8 dynamo_ram[RAM_SIZE] __attribute__ ((aligned(MMU_PAGE_SIZE)));
static cyg_uint32 dynamo_ram_table[NB_RAM_PAGES];

//! A table for virtual memory allocation
static cyg_uint32 dynamo_vm_table[VM_NB_32MB];

//! A global lock for VM access
cyg_mutex_t vm_mutex;

void
dynamo_init(void) {
    int status;
    struct mmu_cfg mmu;
    CYG_ADDRESS ram_base, ram_top;

    // initialize physical RAM allocator
    ram_base = arm_mmu_v2p((CYG_ADDRESS)dynamo_ram);
    ram_top = ram_base + NB_RAM_PAGES*MMU_PAGE_SIZE;

    status = ram_init( ram_base, dynamo_ram_table, NB_RAM_PAGES );
    if ( status != 0 ) {
        DIAG( "could not initialize ram allocator, aborting...\n" );
        return;
    }
    DIAG( "physical ram 0x%08x-0x%08x, %d kBytes\n", ram_base, ram_top,
          (ram_top-ram_base) >> 10);

    // initialize VM allocator
    status = vm_init( VM_BASE, dynamo_vm_table, VM_NB_32MB );
    if ( status != 0 ) {
        DIAG( "could not initialize vm allocator, aborting...\n" );
        return;
    }
    DIAG( "virtual ram  0x%08x-0x%08x, %d MBytes\n",
          VM_BASE, VM_BASE+((32*VM_NB_32MB) << 20), VM_NB_32MB*32 );

     // install our own exception handlers
    dynamo_install_exception_handlers();

    // configure MMU to use RAM allocator
    mmu.ttb = NULL;
    mmu.palloc = ram_allocate;
    mmu.pfree = ram_free;
    mmu.palloc_pt = ram_allocate;
    mmu.pfree_pt = ram_free;

    arm_mmu_set_config(&mmu);

    // initialize DACR
    arm_mmu_set_dacr( 0x55555555, CYGOPT_DYNAMO_DACR_MASK);

    cyg_mutex_init(&vm_mutex);

    // platform specific tweaks
    dynamo_hal_setup();

    // initialize process manager
    cyg_process_init();

    // initialize dynamic loader library
    dlfcn_init();
}
