/**
*******************************************************************************
* @file  rtd2lxt.h
* @brief Real-Time Monitor host tools
*
* rtd2lxt header file
*
* Copyright (C) 2006 Parrot S.A.
*
* @author     gregoire.etienne@parrot.com
* @date       2007-06-22
*******************************************************************************
*/

#ifndef _RTD2LXT_H_
#define _RTD2LXT_H_


#define THREAD_ID_MAX           (1024)

#define RTMON_SUPPORTED_VERSION (10)
#define LINEBUF_MAX             (256)
#define HASHTAB_SIZE            (20011)
#define ADDR2LINE_MAX           (64)
#define NO_PKT                  ((uint32_t)-1)
#define RTMON_PEND_TICK(_pkt_)  ((_pkt_) & (1 << RTMON_PKT_FLAG_PEND_TICK))

#define RESET_PKT_INDEX()   (buf_index = 0)
#define PEEK_PKT()          (( buf_index < buf_size )? buf[buf_index]   : NO_PKT)
#define GET_PKT()           (( buf_index < buf_size )? buf[buf_index++] : NO_PKT)


// Extracted from thread.hxx
enum {                       // Thread state values  
   RUNNING    = 0,          // Thread is runnable or running
   SLEEPING   = 1,          // Thread is waiting for something to happen
   COUNTSLEEP = 2,          // Sleep in counted manner
   SUSPENDED  = 4,          // Suspend count is non-zero
   CREATING   = 8,          // Thread is being created
   EXITED     = 16,         // Thread has exited
   // This is the set of bits that must be cleared by a generic
   // wake() or release().
   SLEEPSET   = (SLEEPING | COUNTSLEEP)
};

enum cyg_reason                     // sleep/wakeup reason codes
{
   NONE,                           // No recorded reason
   WAIT,                           // Wait with no timeout
   DELAY,                          // Simple time delay
   TIMEOUT,                        // Wait with timeout/timeout expired
   BREAK,                          // forced break out of sleep
   DESTRUCT,                       // wait object destroyed[note]
   EXIT,                           // forced termination
   DONE                            // Wait/delay complete
};


typedef struct{
	uint32_t ticks;
	uint32_t count;
} timer_unit_t;

extern timer_unit_t timer;

extern struct           rtmon_header hdr;
extern uint32_t         first_tick ;

extern char             thd_tag[THREAD_ID_MAX][RTMON_TAGSIZE];
extern event_tag_t      user_event_tag[RTMON_PKT_EVENT_MAX];
extern event_tag_t      user_u32_tag[RTMON_PKT_USER_U32_MAX];

extern struct lt_trace  *lt;
extern uint32_t         *buf;
extern uint32_t         buf_index;
extern uint32_t         buf_size;

extern int              verbose;
extern int              process_thread_states;
extern unsigned int	   start_time;
extern unsigned int	   end_time;

extern const char       *exefile;
extern FILE             *prof_file;


extern void stat_init           ( void );
extern void write_stat_file     ( const char * name);
extern void write_savefile      ( const char *name);
extern void init_savefile       ( void ); 
extern void stat_isr	        ( uint32_t type,  uint32_t vec, 
                                  uint32_t ticks, uint32_t count, 
                                  int      call_dsr);
extern void stat_dsr	        ( uint32_t type,  uint32_t vec, 
                                  uint32_t ticks, uint32_t count );
extern void stat_thread         ( uint32_t id,    uint32_t ticks, 
                                  uint32_t count );
extern void stat_log_user_start ( uint32_t event, uint32_t ticks,
                                  uint32_t count );
extern void stat_log_user_stop  ( uint32_t event, uint32_t ticks, 
                                  uint32_t count );

extern void stat_priority_inversion_start(  uint32_t ticks, 
                                            uint32_t count );

extern void stat_priority_inversion_stop(   uint32_t ticks, 
                                            uint32_t count );

extern void stat_thread_activity( uint32_t ticks, uint32_t count,
                           uint32_t pid, uint8_t pstate, uint8_t preason,
                           uint32_t nid, uint8_t nstate, uint8_t nreason );

extern int  process_pkt         ( void);
extern char *get_srctag         ( uint32_t addr );



#endif //_RTD2LXT_H_

