/**
*******************************************************************************
* @file  rtmon_hal.h
* @brief Real-Time Monitor / HAL-dependent routines and constants
*
* Copyright (C) 2006 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2006-12-01
*******************************************************************************
*/
#ifndef CYGONCE_RTMON_HAL_P5_H
#define CYGONCE_RTMON_HAL_P5_H

//-----------------------------------------------------------------------------
// Parrot5/5+ platform
#include <cyg/hal/p5dev.h>

#define RTMON_TIMER_PERIOD  CYGNUM_HAL_RTC_PERIOD

// Set time unit to 100 ns = 10^-7 s
#define RTMON_HAL_TIMESCALE                             (-7)
// A tick is RTMON_TIMER_PERIOD timer cycles
#define RTMON_HAL_TICKPERIOD                            (RTMON_TIMER_PERIOD)
// System clock has 100/65 beats per time unit (100 ns)
#define RTMON_HAL_TIMEFACT_MULT                         (100)
#define RTMON_HAL_TIMEFACT_DIV                          (65)

//<! code to execute during a RTC interrupt
#define RTMON_HAL_LOG_RTC()    RTMON_TICK()

#define RTMON_LOG_STAMP(_type_,_data_) ({                                     \
    cyg_uint32 _r, _pkt;                                                      \
    volatile cyg_uint32 *_base = (volatile cyg_uint32 *)PARROT5_SYS;          \
    _pkt = _base[_SYS_TIM3VAL >> 2];                                          \
    _r = ((_base[_SYS_FLAGS >> 2] >> 4) & 1) << RTMON_PKT_FLAG_PEND_TICK;     \
    /* re-read value to avoid wrapping issues */                              \
    _pkt = ( _r )? _base[_SYS_TIM3VAL >> 2] : _pkt;                           \
    _pkt |= (_r|((_data_) << 16)|((_type_) << 24));                           \
    RTMON_LOG(_pkt);                                                          \
    _pkt;                                                                     \
})

#define RTMON_MKTIME32(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    (rtmon.ticks+1+_corr)*RTMON_TIMER_PERIOD-_val-1;                          \
})

#define RTMON_MKTIME64(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    ((cyg_uint64)(rtmon.ticks+1+_corr))*RTMON_TIMER_PERIOD -_val-1;           \
})

#define RTMON_FIX_PKT(_pkt) ({                                                \
    cyg_uint32 _opkt = _pkt;                                                  \
    (_opkt & 0xffff0000)|((RTMON_TIMER_PERIOD-1)-(_opkt & 0xffff));           \
})

//! Get Link Register
#define RTMON_GET_CALLER() ({                                                 \
    cyg_uint32 _lr;                                                           \
    asm volatile ( "mov %0,lr" : "=r"(_lr));                                  \
    _lr-4;                                                                    \
})

//! Get Program Counter
#define RTMON_GET_PC() ({                                                     \
    cyg_uint32 _pc;                                                           \
    asm volatile ( "mov %0,pc" : "=r"(_pc));                                  \
    _pc-4;                                                                    \
})

/*
#define RTMON_ARM_CHECK_FP(_sbase,_ssize,_x) ({                               \
    cyg_uint32 _cfp = _x;                                                     \
    (( _cfp >= (_sbase))&&                                                    \
     ( _cfp < (_sbase)+(_ssize))&&                                            \
     ( !( _cfp & 0x3 )));                                                     \
})

#define RTMON_GET_CALLER2(_sb, _ss) ({                                        \
    cyg_uint32 _fp, _ret = 0;                                                 \
    asm volatile ("mov %0,fp" : "=r"(_fp));                                   \
    if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                                  \
        _fp = *(volatile cyg_uint32 *)(_fp-12);                               \
        if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                              \
            _ret = (*(volatile cyg_uint32 *)(_fp-4))-4;                       \
        }                                                                     \
    }                                                                         \
    _ret;                                                                     \
})
*/
//! IRQ tag list
#define RTMON_HAL_IRQ_TAGS {                                                  \
    [CYGNUM_HAL_INTERRUPT_IRQ_EBC] =  "IRQ_EBC",                              \
    [CYGNUM_HAL_INTERRUPT_FIQ_EBC] =  "FIQ_EBC",                              \
    [CYGNUM_HAL_INTERRUPT_CAN] =      "CAN",                                  \
    [CYGNUM_HAL_INTERRUPT_AAI] =      "AAI",                                  \
    [CYGNUM_HAL_INTERRUPT_MOST] =     "MOST",                                 \
    [CYGNUM_HAL_INTERRUPT_MMCI0] =    "MMCI0",                                \
    [CYGNUM_HAL_INTERRUPT_MMCI1] =    "MMCI1",                                \
    [CYGNUM_HAL_INTERRUPT_UART0] =    "UART0",                                \
    [CYGNUM_HAL_INTERRUPT_UART1] =    "UART1",                                \
    [CYGNUM_HAL_INTERRUPT_UART2] =    "UART2",                                \
    [CYGNUM_HAL_INTERRUPT_UARTSIM0] = "UARTSIM0",                             \
    [CYGNUM_HAL_INTERRUPT_UARTSIM1] = "UARTSIM1",                             \
    [CYGNUM_HAL_INTERRUPT_UARTSIM2] = "UARTSIM2",                             \
    [CYGNUM_HAL_INTERRUPT_SPI] =      "SPI",                                  \
    [CYGNUM_HAL_INTERRUPT_I2CS] =     "I2CS",                                 \
    [CYGNUM_HAL_INTERRUPT_GPIO0] =    "GPIO0",                                \
    [CYGNUM_HAL_INTERRUPT_GPIO1] =    "GPIO1",                                \
    [CYGNUM_HAL_INTERRUPT_TICK] =     "TICK",                                 \
    [CYGNUM_HAL_INTERRUPT_TIMER0] =   "TIMER0",                               \
    [CYGNUM_HAL_INTERRUPT_TIMER1] =   "TIMER1",                               \
    [CYGNUM_HAL_INTERRUPT_TIMER2] =   "TIMER2",                               \
    [CYGNUM_HAL_INTERRUPT_TIMER3] =   "TIMER3",                               \
    [CYGNUM_HAL_INTERRUPT_CAMIF] =    "CAMIF",                                \
    [CYGNUM_HAL_INTERRUPT_ME] =       "ME",                                   \
    [CYGNUM_HAL_INTERRUPT_CV] =       "CV",                                   \
    [CYGNUM_HAL_INTERRUPT_MON] =      "MON",                                  \
    [CYGNUM_HAL_INTERRUPT_RTCC] =     "RTCC",                                 \
    [CYGNUM_HAL_INTERRUPT_COMMRX] =   "COMMRX",                               \
    [CYGNUM_HAL_INTERRUPT_COMMTX] =   "COMMTX",                               \
    [CYGNUM_HAL_INTERRUPT_I2CM] =     "I2CM",                                 \
    [CYGNUM_HAL_INTERRUPT_FREE1] =    "USB0",                                 \
    [CYGNUM_HAL_INTERRUPT_FREE2] =    "USB1",                                 \
}

#define RTMON_HAL_US2TIME(_usec) \
    (((_usec)*10*RTMON_HAL_TIMEFACT_DIV)/RTMON_HAL_TIMEFACT_MULT)


#endif /* CYGONCE_RTMON_HAL_P5_H */
