/**
 * @file   rtmon_hal_p6.h
 * @brief  RTmon HAL routines and constants
 *
 * @author ivan.djelic@parrot.com
 * @date   2010-04-30
 *
 * Copyright (C) 2004-2010 Parrot S.A.
 */
#ifndef CYGONCE_RTMON_HAL_P6_H
#define CYGONCE_RTMON_HAL_P6_H

//-----------------------------------------------------------------------------
// Parrot6/6i platform
#include <cyg/hal/p6dev.h>

extern cyg_uint32 p6i_rtmon_timer_period;
extern cyg_uint32 p6i_rtmon_timefact_mult;
extern cyg_uint32 p6i_rtmon_timefact_div;

//Get the value from the timer register
#define RTMON_TIMER_PERIOD                      (p6i_rtmon_timer_period)
#define RTMON_HAL_TIMEFACT_MULT                 (p6i_rtmon_timefact_mult)
#define RTMON_HAL_TIMEFACT_DIV                  (p6i_rtmon_timefact_div)

// Set time unit to 100 ns = 10^-7 s
#define RTMON_HAL_TIMESCALE                     (-7)
// A tick is RTMON_TIMER_PERIOD timer cycles
#define RTMON_HAL_TICKPERIOD                    RTMON_TIMER_PERIOD

//<! code to execute during a RTC interrupt
#define RTMON_HAL_LOG_RTC()    RTMON_TICK()

#define RTMON_LOG_STAMP(_type_,_data_) ({                               \
            cyg_uint32 _r, _pkt;                                        \
            volatile cyg_uint32 *_base =                                \
                (volatile cyg_uint32 *)PARROT6_SYS;                     \
            _pkt = (_base[_P6_SYS_TIM3CNT >> 2] >> 1);                  \
            _r = ((_base[_P6_SYS_STATUS >> 2] >> 4) & 1) <<             \
                RTMON_PKT_FLAG_PEND_TICK;                               \
            /* re-read value to avoid wrapping issues */                \
            _pkt = (_r)? (_base[_P6_SYS_TIM3CNT >> 2] >> 1) : _pkt;     \
            _pkt |= (_r|((_data_) << 16)|((_type_) << 24));             \
            RTMON_LOG(_pkt);                                            \
            _pkt;                                                       \
        })

#define RTMON_MKTIME32(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    (rtmon.ticks+1+_corr)*RTMON_TIMER_PERIOD-_val-1;                          \
})

#define RTMON_MKTIME64(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    ((cyg_uint64)(rtmon.ticks+1+_corr))*RTMON_TIMER_PERIOD -_val-1;           \
})

#define RTMON_FIX_PKT(_pkt) ({                                                \
    cyg_uint32 _opkt = _pkt;                                                  \
    (_opkt & 0xffff0000)|((RTMON_TIMER_PERIOD-1)-(_opkt & 0xffff));           \
})

//! Get Link Register
#define RTMON_GET_CALLER() ({                                                 \
    cyg_uint32 _lr;                                                           \
    asm volatile ( "mov %0,lr" : "=r"(_lr));                                  \
    _lr-4;                                                                    \
})

//! Get Program Counter
#define RTMON_GET_PC() ({                                                     \
    cyg_uint32 _pc;                                                           \
    asm volatile ( "mov %0,pc" : "=r"(_pc));                                  \
    _pc-4;                                                                    \
})

/*
#define RTMON_ARM_CHECK_FP(_sbase,_ssize,_x) ({                               \
    cyg_uint32 _cfp = _x;                                                     \
    (( _cfp >= (_sbase))&&                                                    \
     ( _cfp < (_sbase)+(_ssize))&&                                            \
     ( !( _cfp & 0x3 )));                                                     \
})

#define RTMON_GET_CALLER2(_sb, _ss) ({                                        \
    cyg_uint32 _fp, _ret = 0;                                                 \
    asm volatile ("mov %0,fp" : "=r"(_fp));                                   \
    if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                                  \
        _fp = *(volatile cyg_uint32 *)(_fp-12);                               \
        if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                              \
            _ret = (*(volatile cyg_uint32 *)(_fp-4))-4;                       \
        }                                                                     \
    }                                                                         \
    _ret;                                                                     \
})
*/

#define IRQ_TAG(_name) [CYGNUM_HAL_INTERRUPT_##_name] = #_name

//! IRQ tag list
#define RTMON_HAL_IRQ_TAGS  {                                           \
        IRQ_TAG(LCDC),                                                  \
        IRQ_TAG(CAN),                                                   \
        IRQ_TAG(AAI),                                                   \
        IRQ_TAG(MOST),                                                  \
        IRQ_TAG(SDIO),                                                  \
        IRQ_TAG(UART0),                                                 \
        IRQ_TAG(UART1),                                                 \
        IRQ_TAG(UART2),                                                 \
        IRQ_TAG(UART3),                                                 \
        IRQ_TAG(UARTSIM0),                                              \
        IRQ_TAG(NANDMC),                                                \
        IRQ_TAG(SPI),                                                   \
        IRQ_TAG(I2CS),                                                  \
        IRQ_TAG(GPIO),                                                  \
        IRQ_TAG(DMAC),                                                  \
        IRQ_TAG(TICK),                                                  \
        IRQ_TAG(TIMER0),                                                \
        IRQ_TAG(TIMER1),                                                \
        IRQ_TAG(TIMER2),                                                \
        IRQ_TAG(TIMER3),                                                \
        IRQ_TAG(H264),                                                  \
        IRQ_TAG(CAMIF0),                                                \
        IRQ_TAG(CAMIF1),                                                \
        IRQ_TAG(CV),                                                    \
        IRQ_TAG(MON),                                                   \
        IRQ_TAG(PARINT),                                                \
        IRQ_TAG(COMMRX),                                                \
        IRQ_TAG(COMMTX),                                                \
        IRQ_TAG(I2CM),                                                  \
        IRQ_TAG(USB0),                                                  \
        IRQ_TAG(USB1),                                                  \
        IRQ_TAG(MMCIX),                                                 \
}

#define RTMON_HAL_US2TIME(_usec) \
    (((_usec)*10*RTMON_HAL_TIMEFACT_DIV)/RTMON_HAL_TIMEFACT_MULT)

#endif /* CYGONCE_RTMON_HAL_P6_H */
